@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2024 CZ.NIC
+ * Copyright (C) 2014-2025 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,8 +24,11 @@
 #include <cstdlib>
 #include <QDebug>
 #include <QDir>
+#include <QDirIterator>
 #include <QFile>
 #include <QFileInfo>
+#include <QMimeDatabase>
+#include <QMimeType>
 #include <QTextStream>
 
 #include "src/cli/cli.h"
@@ -33,6 +36,7 @@
 #include "src/datovka_shared/compat/compiler.h" /* macroStdMove */
 #include "src/datovka_shared/compat_qt/misc.h" /* _Qt_endl */
 #include "src/datovka_shared/log/log.h"
+#include "src/datovka_shared/isds/message_interface_vodz.h"
 #include "src/datovka_shared/isds/type_conversion.h"
 #include "src/datovka_shared/utility/strings.h"
 #include "src/datovka_shared/worker/pool.h"
@@ -41,19 +45,24 @@
 #include "src/identifiers/message_id.h"
 #include "src/io/account_db.h"
 #include "src/io/dbs.h"
+#include "src/io/exports.h"
 #include "src/io/filesystem.h"
 #include "src/io/isds_helper.h"
 #include "src/isds/to_text_conversion.h"
 #include "src/model_interaction/account_interaction.h"
+#include "src/settings/prefs_specific.h"
 #include "src/worker/task_download_message.h"
 #include "src/worker/task_download_message_list.h"
+#include "src/worker/task_import_zfo.h"
 #include "src/worker/task_search_owner.h"
 #include "src/worker/task_send_message.h"
+#include "src/worker/task_upload_attachment.h"
 
 const QSet<QString> serviceSet = QSet<QString>() << SER_LOGIN <<
 SER_GET_MSG_LIST << SER_SEND_MSG << SER_GET_MSG << SER_GET_DEL_INFO <<
 SER_GET_USER_INFO << SER_GET_OWNER_INFO << SER_CHECK_ATTACHMENT <<
-SER_GET_MSG_IDS << SER_FIND_DATABOX;
+SER_GET_MSG_IDS << SER_FIND_DATABOX << SER_EXPORT_MSG << SER_EXPORT_MSGS <<
+SER_IMPORT_MSG << SER_IMPORT_MSGS;
 
 // Known attributes definition
 static const QStringList connectAttrs = QStringList()
@@ -76,6 +85,15 @@ static const QStringList getDelInfoAttrs = QStringList() << "dmID" << "zfoFile"
 static const QStringList getMsgIdsAttrs = QStringList() << "dmType";
 static const QStringList findDataboxAttrs = QStringList() << "dbType" << "dbID"
     << "ic" << "firmName" << "pnFirstName" << "pnLastName" << "adZipCode";
+static const QStringList exportMsgAttrs = QStringList() << "dmID" << "exportPath"
+     << "exportMsgType" << "exportDelInfo" << "msgName" << "infoName";
+static const QStringList exportMsgListAttrs = QStringList()
+    << "dmType" << "exportPath" << "exportMsgType" << "exportDelInfo"
+    << "msgName" << "infoName" << "fromDate" << "toDate";
+static const QStringList importMsgAttrs = QStringList()
+    << "importFile" << "verifyIsds";
+static const QStringList importMsgListAttrs = QStringList()
+    << "importDir" << "includeSubDirs" << "verifyIsds";
 
 /* ========================================================================= */
 static
@@ -126,10 +144,8 @@ const QString createErrorMsg(const QString &msg)
 	return QString(CLI_PREFIX) + QString(PARSER_PREFIX) + msg;
 }
 
-/* ========================================================================= */
 static
 bool checkAttributeIfExists(const QString &service, const QString &attribute)
-/* ========================================================================= */
 {
 	if (service == SER_LOGIN) {
 		return connectAttrs.contains(attribute);
@@ -145,6 +161,14 @@ bool checkAttributeIfExists(const QString &service, const QString &attribute)
 		return getMsgIdsAttrs.contains(attribute);
 	} else if (service == SER_FIND_DATABOX) {
 		return findDataboxAttrs.contains(attribute);
+	} else if (service == SER_EXPORT_MSG) {
+		return exportMsgAttrs.contains(attribute);
+	} else if (service == SER_EXPORT_MSGS) {
+		return exportMsgListAttrs.contains(attribute);
+	} else if (service == SER_IMPORT_MSG) {
+		return importMsgAttrs.contains(attribute);
+	} else if (service == SER_IMPORT_MSGS) {
+		return importMsgListAttrs.contains(attribute);
 	}
 	return false;
 }
@@ -171,6 +195,424 @@ const QStringList parseDbIDRecipient(const QString &dbIDRecipient)
 	return dbIDRecipient.split(";");
 }
 
+static
+cli_error exportMsg(const QMap<QString, QVariant> &map, MessageDbSet *msgDbSet,
+    QString &errmsg)
+{
+	qDebug() << CLI_PREFIX << "Export message" << map["dmID"].toString();
+
+	const QString userName = map["username"].toString();
+	MsgId msgId = msgDbSet->msgsMsgId(map["dmID"].toLongLong());
+
+	if (Q_UNLIKELY(msgId.dmId() < 0)) {
+		errmsg = QString("Message doesn't exist in the database for user '%1'.").arg(userName);
+		qDebug() << CLI_PREFIX << errmsg;
+		return CLI_ERROR;
+	}
+	MessageDb *messageDb =
+	    msgDbSet->accessMessageDb(msgId.deliveryTime(), false);
+	if (Q_UNLIKELY(messageDb == Q_NULLPTR)) {
+		errmsg = "Database doesn't exists for user " + userName;
+		qDebug() << CLI_PREFIX << errmsg;
+		return CLI_ERROR;
+	}
+
+	/* Mandatory */
+	if (!map.contains("exportPath") ||
+	        map["exportPath"].toString().isEmpty()) {
+		errmsg = "Parameter exportPath was not specified.";
+		qDebug() << CLI_PREFIX << errmsg;
+		return CLI_ATR_VAL_ERR;
+	}
+
+	/* Optional */
+	QString exportMsgType("zfo");
+	if (map.contains("exportMsgType") &&
+	        !map["exportMsgType"].toString().isEmpty()) {
+		exportMsgType = map["exportMsgType"].toString();
+	}
+
+	/* Optional */
+	QString msgName;
+	if (map.contains("msgName") && !map["msgName"].toString().isEmpty()) {
+		msgName = map["msgName"].toString();
+	}
+
+	QString errStr;
+	const QFileInfo fi(map["exportPath"].toString());
+	const QString path = fi.path();
+	const QString accountName = GlobInstcs::acntMapPtr->acntData(
+	    GlobInstcs::acntMapPtr->acntIdFromUsername(userName)).accountName();
+	const QString dbId = GlobInstcs::accntDbPtr->dbId(
+	    AccountDb::keyFromLogin(userName));
+
+	if (exportMsgType == "zfo" || exportMsgType == "all") {
+		if (Q_UNLIKELY(Exports::EXP_SUCCESS != Exports::exportAs(
+		        *msgDbSet, Exports::ZFO_MESSAGE, path,
+		        msgName, userName, accountName, dbId, msgId,
+		        errStr))) {
+			errmsg = "Cannot export complete message to ZFO.";
+			qDebug() << CLI_PREFIX << errmsg << errStr;
+			return CLI_ERROR;
+		}
+	}
+
+	if (exportMsgType == "pdf" || exportMsgType == "all") {
+		if (Q_UNLIKELY(Exports::EXP_SUCCESS != Exports::exportAs(
+		        *msgDbSet, Exports::PDF_ENVELOPE, path,
+		        msgName, userName, accountName, dbId, msgId,
+		        errStr))) {
+			errmsg = "Cannot export message envelope to PDF.";
+			qDebug() << CLI_PREFIX << errmsg << errStr;
+			return CLI_ERROR;
+		}
+	}
+
+	/* Optional */
+	QString exportDelInfo("zfo");
+	if (map.contains("exportDelInfo") &&
+	        !map["exportDelInfo"].toString().isEmpty()) {
+		exportDelInfo = map["exportDelInfo"].toString();
+	}
+
+	/* Optional */
+	QString infoName;
+	if (map.contains("infoName") && !map["infoName"].toString().isEmpty()) {
+		infoName = map["infoName"].toString();
+	}
+
+	if (exportDelInfo == "zfo" || exportDelInfo == "all") {
+		if (Q_UNLIKELY(Exports::EXP_SUCCESS != Exports::exportAs(
+		        *msgDbSet, Exports::ZFO_DELIVERY, path,
+		        infoName, userName, accountName, dbId, msgId,
+		        errStr))) {
+			errmsg = "Cannot export delivery info to ZFO.";
+			qDebug() << CLI_PREFIX << errmsg << errStr;
+			return CLI_ERROR;
+		}
+	}
+
+	if (exportDelInfo == "pdf" || exportDelInfo == "all") {
+		if (Q_UNLIKELY(Exports::EXP_SUCCESS != Exports::exportAs(
+		        *msgDbSet, Exports::PDF_DELIVERY, path,
+		        infoName, userName, accountName, dbId, msgId,
+		        errStr))) {
+			errmsg = "Cannot export delivery info to PDF.";
+			qDebug() << CLI_PREFIX << errmsg << errStr;
+			return CLI_ERROR;
+		}
+	}
+
+	errmsg = "Export of message was successful.";
+	qDebug() << CLI_PREFIX << errmsg;
+
+	return CLI_SUCCESS;
+}
+
+static
+cli_error exportMsg(qint64 msgId, QMap<QString, QVariant> &map,
+    MessageDbSet *msgDbSet, QString &errmsg)
+{
+	map["dmID"] = msgId;
+	return exportMsg(map, msgDbSet, errmsg);
+}
+
+static
+cli_error exportMsgs(QMap<QString, QVariant> &map, MessageDbSet *msgDbSet,
+    QString &errmsg)
+{
+	qDebug() << CLI_PREFIX << "Export messages for username" <<
+	    map["username"].toString();
+
+	if ((map["dmType"].toString() != MT_RECEIVED) &&
+	    (map["dmType"].toString() != MT_SENT) &&
+	    (map["dmType"].toString() != MT_SENT_RECEIVED)) {
+		errmsg = "Wrong dmType value: " + map["dmType"].toString();
+		qDebug() << CLI_PREFIX << errmsg;
+		return CLI_ATR_VAL_ERR;
+	}
+
+	QList<qint64> msgList;
+	QList<qint64> msgListOk;
+	QList<qint64> msgListErr;
+
+	if (!map.contains("fromDate") && !map.contains("toDate")) {
+
+		if (map["dmType"].toString() == MT_RECEIVED) {
+			msgList = msgDbSet->getAllMessageIDs(
+			    MessageDb::TYPE_RECEIVED);
+		} else if (map["dmType"].toString() == MT_SENT) {
+			msgList = msgDbSet->getAllMessageIDs(
+			    MessageDb::TYPE_SENT);
+		} else {
+			msgList = msgDbSet->getAllMessageIDs(
+			    MessageDb::TYPE_RECEIVED);
+			msgList += msgDbSet->getAllMessageIDs(
+			    MessageDb::TYPE_SENT);
+		}
+		for (qint64 msgId : msgList) {
+			if (CLI_SUCCESS == exportMsg(msgId, map,
+			        msgDbSet, errmsg)) {
+				msgListOk.append(msgId);
+			} else  {
+				msgListErr.append(msgId);
+			}
+		}
+
+	} else {
+
+		/* Optional */
+		QDate fromDate(2009,7,1); /* First start of ISDS. */
+		if (map.contains("fromDate") &&
+		        !map["fromDate"].toString().isEmpty()) {
+			const QDate date = map["fromDate"].toDate();
+			if (date.isValid()) {
+				fromDate = date;
+			} else {
+				errmsg = "Wrong fromDate value: " +
+				    map["fromDate"].toString();
+				qDebug() << CLI_PREFIX << errmsg;
+				return CLI_ATR_VAL_ERR;
+			}
+		}
+
+		/* Optional */
+		QDate toDate(QDate::currentDate());
+		if (map.contains("toDate") &&
+		        !map["toDate"].toString().isEmpty()) {
+			const QDate date = map["toDate"].toDate();
+			if (date.isValid()) {
+				toDate = date;
+			} else {
+				errmsg = "Wrong toDate value: " +
+				    map["toDate"].toString();
+				qDebug() << CLI_PREFIX << errmsg;
+				return CLI_ATR_VAL_ERR;
+			}
+		}
+
+		QList<MsgId> msgIdList;
+		if (map["dmType"].toString() == MT_RECEIVED) {
+			msgIdList = msgDbSet->msgsDateInterval(fromDate,
+			     toDate, MessageDb::TYPE_RECEIVED);
+		} else if (map["dmType"].toString() == MT_SENT) {
+			msgIdList = msgDbSet->msgsDateInterval(fromDate,
+			     toDate, MessageDb::TYPE_SENT);
+		} else {
+			msgIdList = msgDbSet->msgsDateInterval(fromDate,
+			     toDate, MessageDb::TYPE_RECEIVED);
+			msgIdList += msgDbSet->msgsDateInterval(fromDate,
+			     toDate, MessageDb::TYPE_SENT);
+		}
+
+		for (MsgId msgId : msgIdList) {
+			if (CLI_SUCCESS == exportMsg(msgId.dmId(), map,
+			        msgDbSet, errmsg)) {
+				msgListOk.append(msgId.dmId());
+			} else  {
+				msgListErr.append(msgId.dmId());
+			}
+		}
+	}
+
+	if (!msgListErr.isEmpty()) {
+		errmsg = "Export of these messages have failed.";
+		qDebug() << CLI_PREFIX << errmsg;
+		printDataToStdOut(msgListErr);
+	}
+
+	errmsg = "Export of all messages was successful.";
+	qDebug() << CLI_PREFIX << errmsg;
+
+	printDataToStdOut(msgListOk);
+
+	return CLI_SUCCESS;
+}
+
+static
+cli_error importMsg(const QMap<QString, QVariant> &map, MessageDbSet *msgDbSet,
+    QString &errmsg)
+{
+	qDebug() << CLI_PREFIX << "Import message";
+
+	const AcntId acntId = GlobInstcs::acntMapPtr->acntIdFromUsername(
+	    map["username"].toString());
+	QList<AcntIdDb> acntDbList;
+	acntDbList.append(AcntIdDb(acntId, msgDbSet));
+
+	/* Mandatory */
+	if (!map.contains("importFile") ||
+	        map["importFile"].toString().isEmpty()) {
+		errmsg = "Parameter importFile was not specified.";
+		qDebug() << CLI_PREFIX << errmsg;
+		return CLI_ATR_VAL_ERR;
+	}
+
+	QString importFile(map["importFile"].toString());
+
+	/* Optional */
+	bool authenticate = false;
+	if (map.contains("verifyIsds") &&
+	        !map["verifyIsds"].toString().isEmpty()) {
+		authenticate = (map["verifyIsds"].toString() == "yes");
+	}
+
+	switch (TaskImportZfo::determineFileType(importFile)) {
+	case TaskImportZfo::ZT_MESSAGE:
+		{
+			TaskImportZfo *task = new (::std::nothrow) TaskImportZfo(
+			    acntDbList, importFile, TaskImportZfo::ZT_MESSAGE,
+			    true, authenticate);
+			if (Q_UNLIKELY(task == Q_NULLPTR)) {
+				errmsg = "Cannot import ZFO message.";
+				qDebug() << CLI_PREFIX << errmsg;
+				return CLI_ERROR;
+			}
+			task->setAutoDelete(false);
+			GlobInstcs::workPoolPtr->runSingle(task);
+			if (TaskImportZfo::IMP_SUCCESS == task->m_result) {
+				errmsg = "Import of message info was successful.";
+			} else if (TaskImportZfo::IMP_DB_EXISTS == task->m_result) {
+				errmsg = "Message already exists in the database.";
+			} else {
+				errmsg = "Cannot import ZFO message.";
+				qDebug() << CLI_PREFIX << errmsg << task->m_resultDesc;
+				delete task; task = Q_NULLPTR;
+				return CLI_ERROR;
+			}
+			delete task; task = Q_NULLPTR;
+		}
+		break;
+	case TaskImportZfo::ZT_DELIVERY_INFO:
+		{
+			TaskImportZfo *task = new (::std::nothrow) TaskImportZfo(
+			    acntDbList, importFile, TaskImportZfo::ZT_DELIVERY_INFO,
+			    true, authenticate);
+			if (Q_UNLIKELY(task == Q_NULLPTR)) {
+				errmsg = "Cannot import ZFO delivery info.";
+				qDebug() << CLI_PREFIX << errmsg;
+				return CLI_ERROR;
+			}
+			task->setAutoDelete(false);
+			GlobInstcs::workPoolPtr->runSingle(task);
+			if (TaskImportZfo::IMP_SUCCESS == task->m_result) {
+				errmsg = "Import of delivery info was successful.";
+			} else if (TaskImportZfo::IMP_DB_EXISTS == task->m_result) {
+				errmsg = "Delivery info already exists in the database.";
+			} else {
+				errmsg = "Cannot import ZFO delivery info.";
+				qDebug() << CLI_PREFIX << errmsg << task->m_resultDesc;
+				delete task; task = Q_NULLPTR;
+				return CLI_ERROR;
+			}
+			delete task; task = Q_NULLPTR;
+		}
+		break;
+	default:
+		errmsg = "Wrong ZFO format. File does not contain correct data for import.";
+		qDebug() << CLI_PREFIX << errmsg;
+		return CLI_ERROR;
+		break;
+	}
+
+	qDebug() << CLI_PREFIX << errmsg;
+	return CLI_SUCCESS;
+}
+
+static
+cli_error importMsg(const QString &importFile, QMap<QString, QVariant> &map,
+    MessageDbSet *msgDbSet, QString &errmsg)
+{
+	map["importFile"] = importFile;
+	return importMsg(map, msgDbSet, errmsg);
+}
+
+static
+cli_error importMsgs(QMap<QString, QVariant> &map, MessageDbSet *msgDbSet,
+    QString &errmsg)
+{
+	qDebug() << CLI_PREFIX << "Import messages for username" <<
+	    map["username"].toString();
+
+	/* Mandatory */
+	if (!map.contains("importDir") ||
+	        map["importDir"].toString().isEmpty()) {
+		errmsg = "Parameter importDir was not specified.";
+		qDebug() << CLI_PREFIX << errmsg;
+		return CLI_ATR_VAL_ERR;
+	}
+
+	QString importDir = map["importDir"].toString();
+
+	/* Optional */
+	bool includeSubDirs = false;
+	if (map.contains("includeSubDirs") &&
+	        !map["includeSubDirs"].toString().isEmpty()) {
+		includeSubDirs = (map["includeSubDirs"].toString() == "yes");
+	}
+
+	QStringList fileList;
+	const QStringList nameFilter("*.zfo");
+	if (includeSubDirs) {
+		/* Include subdirectories. */
+		QDirIterator it(importDir, nameFilter, QDir::Files,
+		    QDirIterator::Subdirectories);
+		while (it.hasNext()) {
+			fileList.append(it.next());
+		}
+	} else {
+		QStringList files = QDir(importDir).entryList(nameFilter);
+		for (int i = 0; i < files.size(); ++i) {
+			fileList.append(
+			    importDir + QDir::separator() + files.at(i));
+		}
+	}
+
+	if (Q_UNLIKELY(fileList.isEmpty())) {
+		errmsg = "No zfo files in the importDir.";
+		qDebug() << CLI_PREFIX << errmsg;
+		return CLI_ERROR;
+	}
+
+	/* Detect type of zfo content. */
+	QStringList msgList;
+	QStringList delInfoList;
+	for (const QString &file : fileList) {
+		enum TaskImportZfo::ZfoType ret =
+		    TaskImportZfo::determineFileType(file);
+		if (TaskImportZfo::ZT_MESSAGE == ret) {
+			msgList.append(file);
+		} else if (TaskImportZfo::ZT_DELIVERY_INFO == ret) {
+			delInfoList.append(file);
+		}
+	}
+
+	QStringList errList;
+	/* First import messages. */
+	for (const QString &file : msgList) {
+		if (CLI_SUCCESS != importMsg(file, map, msgDbSet, errmsg)) {
+			errList.append(file);
+		}
+	}
+	/* Second delivery info. */
+	for (const QString &file : delInfoList) {
+		if (CLI_SUCCESS != importMsg(file, map, msgDbSet, errmsg)) {
+			errList.append(file);
+		}
+	}
+
+	if (Q_UNLIKELY(errList.isEmpty())) {
+		errmsg = "Import of all messages was successful.";
+		qDebug() << CLI_PREFIX << errmsg;
+		printDataToStdOut(errList);
+		return CLI_ERROR;
+	}
+
+	errmsg = "Import of all messages was successful.";
+	qDebug() << CLI_PREFIX << errmsg;
+	return CLI_SUCCESS;
+}
+
 /* ========================================================================= */
 static
 cli_error getMsgList(const QMap<QString, QVariant> &map, MessageDbSet *msgDbSet,
@@ -733,6 +1175,112 @@ cli_error findDatabox(const QMap<QString, QVariant> &map, QString &errmsg)
 	return CLI_SUCCESS;
 }
 
+/*!
+ * @brief Get MIME type from file.
+ *
+ * @param[in] fi File info.
+ * @return MIME type.
+ */
+static
+const QString getMimeType(const QFileInfo &fi)
+{
+	if (fi.suffix().toLower() == "zfo") {
+		return QStringLiteral("application/vnd.software602.filler.form-xml-zip");
+	}
+
+	QMimeDatabase db;
+	QMimeType mimeType = db.mimeTypeForFile(fi);
+	if (mimeType.isValid()) {
+		return mimeType.name();
+	}
+
+	return QStringLiteral("application/octet-stream");
+}
+
+/*!
+ * @brief Build DmExtFile list and upload attachments.
+ *
+ * @param[in] acntId Account id.
+ * @param[in] filePaths File list.
+ * @return DmExtFile list.
+ */
+static
+QList<Isds::DmExtFile> buildExtFiles(const AcntId &acntId,
+    const QStringList &filePaths)
+{
+	const QString timeStr = QDateTime::currentDateTimeUtc().toString();
+	QList<Isds::DmExtFile> dmExtFiles;
+
+	for (int i = 0; i < filePaths.size(); ++i) {
+		const QString &filePath = filePaths.at(i);
+
+		const QString transactionId = QString("%1_%2_%3_%4")
+		    .arg(acntId.username()).arg(timeStr).arg(i)
+		    .arg(Utility::generateRandomString(6));
+
+		Isds::DmExtFile dmExtFile;
+
+		const QFileInfo fi(filePath);
+		if (!fi.isReadable() || !fi.isFile()) {
+			logErrorNL(CLI_PREFIX "Wrong file name '%s' or file is missing.",
+			    filePath.toUtf8().constData());
+			goto fail;
+		}
+
+		/*
+		 * First document must have dmFileMetaType set to
+		 * FILEMETATYPE_MAIN. Remaining documents have
+		 * FILEMETATYPE_ENCLOSURE.
+		 */
+		dmExtFile.setFileMetaType((i == 0) ?
+		    Isds::Type::FMT_MAIN : Isds::Type::FMT_ENCLOSURE);
+
+		{
+			Isds::DmFile dmFile;
+			TaskUploadAttachment *task = Q_NULLPTR;
+
+			/* Prepare dmFile content. */
+			{
+				dmFile.setFileDescr(fi.fileName());
+				/*
+				 * The FileMetaType is ignored when uploading separate
+				 * attachments. But a value must be provided to pass it
+				 * successfully to the underlying library.
+				 */
+				dmFile.setFileMetaType((i == 0) ?
+				    Isds::Type::FMT_MAIN : Isds::Type::FMT_ENCLOSURE);
+				dmFile.setMimeType(getMimeType(fi));
+
+				QFile file(QDir::fromNativeSeparators(filePath));
+				if (file.exists()) {
+					if (!file.open(QIODevice::ReadOnly)) {
+						logErrorNL(CLI_PREFIX "Couldn't open file '%s'.",
+						    filePath.toUtf8().constData());
+						goto fail;
+					}
+					dmFile.setBinaryContent(file.readAll());
+				}
+			}
+
+			task = new (::std::nothrow) TaskUploadAttachment(
+			    acntId, macroStdMove(dmFile), transactionId);
+			if (Q_UNLIKELY(task == Q_NULLPTR)) {
+				goto fail;
+			}
+			task->setAutoDelete(false);
+			GlobInstcs::workPoolPtr->runSingle(task);
+			Isds::DmAtt dmAtt = task->m_dmAtt;
+			dmExtFile.setDmAtt(dmAtt);
+			delete task; task = Q_NULLPTR;
+			dmExtFiles.append(dmExtFile);
+		}
+	}
+
+	return dmExtFiles;
+fail:
+	return QList<Isds::DmExtFile>();
+}
+
 /* ========================================================================= */
 static
 QList<Isds::Document> buildDocuments(const QStringList &filePaths)
@@ -743,7 +1291,7 @@ QList<Isds::Document> buildDocuments(const QStringList &filePaths)
 
 	foreach (const QString &filePath, filePaths) {
 
-		if (filePath.isEmpty()) {
+		if (Q_UNLIKELY(filePath.isEmpty())) {
 			continue;
 		}
 
@@ -866,6 +1414,48 @@ Isds::Envelope buildEnvelope(const QMap<QString, QVariant> &map)
 	return envelope;
 }
 
+/*!
+ * @brief Test if message must be sent as big message.
+ *
+ * @param[in] files File list.
+ * @return True if message will be sent as big message.
+ */
+static
+bool sendAsVodz(const QStringList &files)
+{
+	if (!PrefsSpecific::enableVodzSending(*GlobInstcs::prefsPtr)) {
+		return false;
+	}
+
+	const qint64 maxSizeBytes =
+	    PrefsSpecific::bytesInMByte(*GlobInstcs::prefsPtr) *
+	    PrefsSpecific::maxAttachmentMBytes(*GlobInstcs::prefsPtr);
+
+	qint64 totalSizeBytes = 0;
+	for (const QString &file : files) {
+
+		if (Q_UNLIKELY(file.isEmpty())) {
+			continue;
+		}
+
+		QFileInfo fi(file);
+		if (!fi.isReadable() || !fi.isFile()) {
+			logErrorNL(CLI_PREFIX "Wrong file name '%s' or file is missing.",
+			    file.toUtf8().constData());
+			continue;
+		}
+
+		int fileSizeBytes = fi.size();
+		if (fileSizeBytes >= maxSizeBytes) {
+			return true;
+		}
+
+		totalSizeBytes += fileSizeBytes;
+	}
+
+	return (totalSizeBytes >= maxSizeBytes);
+}
+
 /* ========================================================================= */
 static
 cli_error createAndSendMsg(const QMap<QString, QVariant> &map,
@@ -879,7 +1469,15 @@ cli_error createAndSendMsg(const QMap<QString, QVariant> &map,
 	QStringList sendID;
 	Isds::Envelope evelope = buildEnvelope(map);
 	Isds::Message message;
-	message.setDocuments(buildDocuments(map["dmAttachment"].toStringList()));
+	const AcntId acntId = GlobInstcs::acntMapPtr->acntIdFromUsername(
+	    map["username"].toString());
+	const bool isVodz = sendAsVodz(map["dmAttachment"].toStringList());
+
+	if (isVodz) {
+		message.setExtFiles(buildExtFiles(acntId, map["dmAttachment"].toStringList()));
+	} else {
+		message.setDocuments(buildDocuments(map["dmAttachment"].toStringList()));
+	}
 
 	foreach (const QString &recipientId,
 	    map.value("dbIDRecipient").toStringList()) {
@@ -894,10 +1492,9 @@ cli_error createAndSendMsg(const QMap<QString, QVariant> &map,
 		    QDateTime::currentDateTimeUtc().toString() + "_" +
 		    Utility::generateRandomString(6);
 
-		const AcntId acntId = GlobInstcs::acntMapPtr->acntIdFromUsername(map["username"].toString());
 		TaskSendMessage *task = new (::std::nothrow) TaskSendMessage(
 		    AcntIdDb(acntId, msgDbSet), transactId, message,
-		    "Databox ID: " + recipientId, "unknown", false, false,
+		    "Databox ID: " + recipientId, "unknown", false, isVodz,
 		    STORE_RAW_INTO_DB);
 		if (Q_UNLIKELY(task == Q_NULLPTR)) {
 			ret = CLI_ERROR;
@@ -1225,8 +1822,7 @@ cli_error checkMandatoryAttributes(const QString &service,
 	} else if (service == SER_FIND_DATABOX) {
 		return checkFindDataboxMandatoryAttributes(map, errmsg);
 	}
-	errmsg = "Unknown service name";
-	return CLI_UNKNOWN_SER;
+	return CLI_SUCCESS;
 }
 
 /* ========================================================================= */
@@ -1365,11 +1961,9 @@ cli_error parsePamamString(const QString &service, const QString &paramString,
 	return CLI_SUCCESS;
 }
 
-/* ========================================================================= */
 static
-cli_error doService(const QString &service, const QMap<QString, QVariant> &map,
+cli_error doService(const QString &service, QMap<QString, QVariant> &map,
     MessageDbSet *msgDbSet, bool needsISDS, QString &errmsg)
-/* ========================================================================= */
 {
 
 	if (service == SER_GET_MSG_LIST) {
@@ -1390,15 +1984,21 @@ cli_error doService(const QString &service, const QMap<QString, QVariant> &map,
 		return getMsgIds(map, msgDbSet, errmsg);
 	} else if (service == SER_FIND_DATABOX) {
 		return findDatabox(map, errmsg);
+	} else if (service == SER_EXPORT_MSG) {
+		return exportMsg(map, msgDbSet, errmsg);
+	} else if (service == SER_EXPORT_MSGS) {
+		return exportMsgs(map, msgDbSet, errmsg);
+	} else if (service == SER_IMPORT_MSG) {
+		return importMsg(map, msgDbSet,  errmsg);
+	} else if (service == SER_IMPORT_MSGS) {
+		return importMsgs(map, msgDbSet, errmsg);
 	}
 	errmsg = "Unknown service name";
 	return CLI_UNKNOWN_SER;
 }
 
-/* ========================================================================= */
-int runService(const QString &lParam,
-    const QString &service, const QString &sParam)
-/* ========================================================================= */
+int runService(const QString &lParam, const QString &service,
+    const QString &sParam)
 {
 	QMap<QString, QVariant> loginMap;
 	QMap<QString, QVariant> serviceMap;
