@@ -35,11 +35,37 @@ int CLIParser::setupCmdLineParser(QCommandLineParser &parser)
 	parser.addHelpOption();
 	parser.addVersionOption();
 	/* Options with values. */
+#ifndef PORTABLE_APPLICATION
+	/* Prohibit explicit base directory selection for portable application. */
+	if (!parser.addOption(QCommandLineOption(CONF_BASEDIR_OPT,
+	        tr("Use <%1> as base directory for configuration."
+	            " It is the path where the subdirectory is located."
+	            " Default value depends on the operating system environment.")
+	            .arg(tr("conf-basedir")),
+	        tr("conf-basedir")))) {
+		return -1;
+	}
+#endif /* !PORTABLE_APPLICATION */
 	if (!parser.addOption(QCommandLineOption(CONF_SUBDIR_OPT,
-	        tr("Use <%1> subdirectory for configuration.").arg(tr("conf-subdir")),
+	        tr("Use <%1> as a subdirectory for configuration."
+	            " The subdirectory is located relative to the base directory."
+	            " Default value is '.dsgui'.") /* Defined in src/settings/preferences.cpp */
+	            .arg(tr("conf-subdir")),
 	        tr("conf-subdir")))) {
 		return -1;
 	}
+#ifndef PORTABLE_APPLICATION
+	/* Prohibit explicit directory selection for portable application. */
+	if (!parser.addOption(QCommandLineOption(CONF_DIR_OPT,
+	        tr("Use <%1> as directory for configuration."
+	            " This is the complete path."
+	            " Default value consists of default base directory and default subdirectory."
+	            " This option cannot be combined with 'conf-basedir' or 'conf-subdir'.")
+	            .arg(tr("conf-dir")),
+	        tr("conf-dir")))) {
+		return -1;
+	}
+#endif /* !PORTABLE_APPLICATION */
 	if (!parser.addOption(QCommandLineOption(LOAD_CONF_OPT,
 	        tr("On start load <%1> file.").arg(tr("conf")), tr("conf")))) {
 		return -1;
