@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2024 CZ.NIC
+ * Copyright (C) 2014-2025 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 
 #include <QDir>
 #include <QFileInfo>
+#include <QMutexLocker>
 #include <QSqlError>
 #include <QSqlQuery>
 #include <QVariant>
@@ -318,6 +319,8 @@ void SQLiteDb::closeDb(void)
 
 bool SQLiteDb::checkDb(bool quick)
 {
+	QMutexLocker locker(&m_lock);
+
 	QSqlQuery query(m_db);
 	bool ret = false;
 
@@ -565,6 +568,8 @@ bool SQLiteDb::backup(const QString &fileName)
 
 	backupDb.setDatabaseName(QDir::toNativeSeparators(fileName));
 
+	QMutexLocker locker(&m_lock);
+
 	if (Q_UNLIKELY(!backupDb.open())) {
 		logErrorNL("Cannot open back-up database file '%s'.",
 		    fileName.toUtf8().constData());
@@ -597,6 +602,8 @@ fail:
 
 qint64 SQLiteDb::dbSize(void)
 {
+	QMutexLocker locker(&m_lock);
+
 	qint64 pageSize = -1;
 	qint64 pageCount = -1;
 	bool iOk = false;
