@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2024 CZ.NIC
+ * Copyright (C) 2014-2025 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -754,6 +754,47 @@ namespace Isds {
 
 	void swap(CreditEventStorageSet &first, CreditEventStorageSet &second) Q_DECL_NOTHROW;
 
+	class CreditEventDeletedMessageRecoveredPrivate;
+	class CreditEventDeletedMessageRecovered {
+		Q_DECLARE_PRIVATE(CreditEventDeletedMessageRecovered)
+
+	public:
+		CreditEventDeletedMessageRecovered(void);
+		CreditEventDeletedMessageRecovered(const CreditEventDeletedMessageRecovered &other);
+#ifdef Q_COMPILER_RVALUE_REFS
+		CreditEventDeletedMessageRecovered(CreditEventDeletedMessageRecovered &&other) Q_DECL_NOEXCEPT;
+#endif /* Q_COMPILER_RVALUE_REFS */
+		~CreditEventDeletedMessageRecovered(void);
+
+		CreditEventDeletedMessageRecovered &operator=(const CreditEventDeletedMessageRecovered &other) Q_DECL_NOTHROW;
+#ifdef Q_COMPILER_RVALUE_REFS
+		CreditEventDeletedMessageRecovered &operator=(CreditEventDeletedMessageRecovered &&other) Q_DECL_NOTHROW;
+#endif /* Q_COMPILER_RVALUE_REFS */
+
+		bool operator==(const CreditEventDeletedMessageRecovered &other) const;
+		bool operator!=(const CreditEventDeletedMessageRecovered &other) const;
+
+		friend void swap(CreditEventDeletedMessageRecovered &first, CreditEventDeletedMessageRecovered &second) Q_DECL_NOTHROW;
+
+		bool isNull(void) const;
+
+		/* name of user who initiated this change */
+		const QString &initiator(void) const;
+		void setInitiator(const QString &i);
+#ifdef Q_COMPILER_RVALUE_REFS
+		void setInitiator(QString &&i);
+#endif /* Q_COMPILER_RVALUE_REFS */
+
+	private:
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 12, 0))
+		::std::unique_ptr<CreditEventDeletedMessageRecoveredPrivate> d_ptr;
+#else /* < Qt-5.12 */
+		QScopedPointer<CreditEventDeletedMessageRecoveredPrivate> d_ptr;
+#endif /* >= Qt-5.12 */
+	};
+
+	void swap(CreditEventDeletedMessageRecovered &first, CreditEventDeletedMessageRecovered &second) Q_DECL_NOTHROW;
+
 	class CreditEventPrivate;
 	/*!
 	 * @brief Based on tDBCreditInfoOutput (dbTypes.xsd)
@@ -824,6 +865,11 @@ namespace Isds {
 #endif /* Q_COMPILER_RVALUE_REFS */
 		/* Expired has no actual additional data. */
 		void setExpired(void);
+		const CreditEventDeletedMessageRecovered &deletedMessageRecovered(void) const;
+		void setDeletedMessageRecovered(const CreditEventDeletedMessageRecovered &cedmr);
+#ifdef Q_COMPILER_RVALUE_REFS
+		void setDeletedMessageRecovered(CreditEventDeletedMessageRecovered &&cedmr);
+#endif /* Q_COMPILER_RVALUE_REFS */
 
 	private:
 #if (QT_VERSION >= QT_VERSION_CHECK(5, 12, 0))
