@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2024 CZ.NIC
+ * Copyright (C) 2014-2025 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,6 +35,7 @@
 #include "src/gui/dlg_timestamp_expir.h"
 #include "src/gui/mw_status.h"
 #include "src/identifiers/message_id.h"
+#include "src/json/message_id.h" /* Json::MsgId2List is used in slots. Q_MOC_INCLUDE */
 #include "src/json/tag_assignment.h" /* Json::TagAssignmentList is used in slots. Q_MOC_INCLUDE */
 #include "src/json/tag_entry.h" /* Json::TagEntryList is used in slots. Q_MOC_INCLUDE */
 #include "src/models/accounts_model.h"
@@ -49,9 +50,10 @@ class AcntId; /* Forward declaration. */
 class AcntIdDb; /* Forward declaration. */
 class MessageDbSet; /* Forward declaration. */
 class Prefs; /* Forward declaration. */
-class QLabel; /* Forward declaration. */
+class QComboBox; /* Forward declaration. */
 class QLineEdit; /* Forward declaration. */
 class QProgressBar; /* Forward declaration. */
+class QUrl; /* Forward declaration. */
 
 namespace Ui {
 	class MainWindow;
@@ -126,7 +128,12 @@ private:
 	void loadSettings(void);
 
 	/*!
-	 * @brief Store current settings to configuration.
+	 * @brief Store settings to preferecnes database.
+	 */
+	void savePrefsSettings(void) const;
+
+	/*!
+	 * @brief Store current settings to INI file and preferences database.
 	 */
 	void saveSettings(void) const;
 
@@ -150,6 +157,11 @@ private:
 	 */
 	void messageBarAssignActions(void);
 
+	/*!
+	 * @brief Connect settings container signals to appropriate slots.
+	 */
+	void settingsConnectActions(void);
+
 	/*!
 	 * @brief Connect database container signals to appropriate slots.
 	 */
@@ -475,6 +487,18 @@ private slots:
 	 */
 	void viewTimestampExpirationDlg(void);
 
+	/*!
+	 * @brief Show non-modal timestamp renewal dialogue window.
+	 */
+	void viewTimestampRenewalDlg(void);
+
+	/*!
+	 * @brief Timestamp renewal dialogue closed.
+	 *
+	 * @param[in] result Dialogue window closing status.
+	 */
+	void closeTimestampRenewalDlg(int result);
+
 	/*!
 	 * @brief Show non-modal search message dialogue window.
 	 */
@@ -488,15 +512,67 @@ private slots:
 	void closeMessageSearchDlg(int result);
 
 	/*!
-	 * @brief Show tag manager dialogue window.
+	 * @brief Show available tag manager dialogue window.
 	 */
-	void viewTagDlg(void);
+	void viewAvailableTagsDlg(void);
 
 	/*!
 	 * @brief Show convert and import from mobile app dialogue window.
 	 */
 	void viewConvertImportFromMobileAppDlg(void);
 
+	/*!
+	 * @brief Show convert and create zip archive for mobile app.
+	 */
+	void viewConvertDataForMobileAppDlg(void);
+
+	/*!
+	 * @brief Quit application and continue with database repair.
+	 */
+	void repairDatabaseFiles(void);
+
+	/*!
+	 * @brief Quit application and continue with database repair.
+	 *
+	 * @param[in] link File URL.
+	 */
+	void repairDatabaseFile(const QUrl &link);
+
+	/*!
+	 * @brief Store top tool bar visibility into preferences.
+	 */
+	void toolBarSettingsToggled(bool checked);
+
+	/*!
+	 * @brief Store attachment and message list tool bar visibility into preferences.
+	 */
+	void topPanelBarsToggled(bool checked);
+
+	/*!
+	 * @brief Show top tool bar configuration dialogue.
+	 */
+	void viewCustomiseToolBar(void);
+
+	/*!
+	 * @brief Show account list tool bar configuration dialogue.
+	 */
+	void viewCustomiseAccountBar(void);
+
+	/*!
+	 * @brief Show message list tool bar configuration dialogue.
+	 */
+	void viewCustomiseMessageBar(void);
+
+	/*!
+	 * @brief Show message detail tool bar configuration dialogue.
+	 */
+	void viewCustomiseDetailBar(void);
+
+	/*!
+	 * @brief Show attachment list tool bar configuration dialogue.
+	 */
+	void viewCustomiseAttachmentBar(void);
+
 	/*!
 	 * @brief Show non-modal log dialogue.
 	 */
@@ -514,11 +590,6 @@ private slots:
 	 */
 	void viewAllSettingsDlg(void);
 
-	/*!
-	 * @brief Show tool bar configuration dialogue.
-	 */
-	void viewCustomiseToolBar(void);
-
 /* Help menu. */
 	/*!
 	 * @brief Show a dialogue window about the application.
@@ -622,10 +693,16 @@ private slots:
 	 */
 	void messageItemsSelectedMarkSettled(void);
 
+/* Filter-field-related actions. */
 	/*!
-	 * @brief Assign tags to selected messages.
+	 * @brief Show message list filter.
 	 */
-	void messageItemsSelectedAssignTags(void);
+	void showMessageFilter(void);
+
+	/*!
+	 * @brief Hide message list filter.
+	 */
+	void hideMessageFilter(void);
 
 /* Window state change. */
 	/*!
@@ -672,6 +749,71 @@ private slots:
 	void adaptToAttachmentSelection(const QItemSelection &selected,
 	    const QItemSelection &deselected = QItemSelection());
 
+/* Mouse on widget clicks. */
+	/*!
+	 * @brief Generates a context menu for the menu bar.
+	 *
+	 * @param[in] point Invocation position.
+	 */
+	void viewMenuBarContextMenu(const QPoint &point);
+
+	/*!
+	 * @brief Generates a context menu for the top tool bar.
+	 *
+	 * @param[in] point Invocation position.
+	 */
+	void viewToolBarContextMenu(const QPoint &point);
+
+	/*!
+	 * @brief Generates a context menu for the account list tool bar.
+	 *
+	 * @param[in] point Invocation position.
+	 */
+	void viewAccountToolBarContextMenu(const QPoint &point);
+
+	/*!
+	 * @brief Generates a context menu for the message list tool bar.
+	 *
+	 * @param[in] point Invocation position.
+	 */
+	void viewMessageToolBarContextMenu(const QPoint &point);
+
+	/*!
+	 * @brief Generates a context menu for the message detail tool bar.
+	 *
+	 * @param[in] point Invocation position.
+	 */
+	void viewDetailToolBarContextMenu(const QPoint &point);
+
+	/*!
+	 * @brief Generates a context menu for the attachment list tool bar.
+	 *
+	 * @param[in] point Invocation position.
+	 */
+	void viewAttachmentToolBarContextMenu(const QPoint &point);
+
+	/*!
+	 * @brief Show classic tag assignment dialogue or pop-up.
+	 */
+	void viewTagAssignmentInterface(void);
+
+	/*!
+	 * @brief Show classic tag assignment dialogue.
+	 */
+	void viewTagAssignmentDialogue(void);
+
+	/*!
+	 * @brief Show tag assignment pop-up.
+	 */
+	void viewTagAssignmentPopup(void);
+
+	/*!
+	 * @brief Enable timestamp tracking for selected messages.
+	 *
+	 * @param[in] enable Whether to enable/disable tracking.
+	 */
+	void toggleTimestampTracking(bool enable);
+
 /* Mouse on view clicks. */
 	/*!
 	 * @brief Generates a context menu depending on the item and selection
@@ -681,6 +823,19 @@ private slots:
 	 */
 	void viewAccountListContextMenu(const QPoint &point);
 
+	/*!
+	 * @brief Create custom context menu when clicking on anchors in the
+	 *     account text info.
+	 */
+	void viewAccountTextInfoContextMenu(const QPoint &point);
+
+	/*!
+	 * @brief Open an URL when clicked on a link in the account text info.
+	 *
+	 * @param[in] link File URL.
+	 */
+	void viewAccountTextInfoAnchorClicked(const QUrl &link);
+
 	/*!
 	 * @brief Generates a context menu depending on the item and selection
 	 *     it was invoked on.
@@ -709,6 +864,12 @@ private slots:
 	 */
 	void viewSelectedMessage(void);
 
+/* Handle settings changes. */
+	/*!
+	 * @brief Store INI settings.
+	 */
+	void watchSettingsChange(void);
+
 /* Handle database signals. */
 	/*!
 	 * @brief Update window data when message locally read status changes.
@@ -786,6 +947,22 @@ private slots:
 	 */
 	void watchTagContReset(void);
 
+	/*!
+	 * @brief Update model data when timestamp tracking enabled.
+	 *
+	 * @param[in] msgIds Message identifiers.
+	 * @param[in] acntId Account identifier.
+	 */
+	void watchIdentificationsInserted(const Json::MsgId2List &msgIds,
+	    const AcntId &acntId);
+
+	/*!
+	 * @brief Update model data when timestamp tracking disabled.
+	 *
+	 * @param[in] msgIds Message identifiers.
+	 */
+	void watchIdentificationsDeleted(const Json::MsgId2List &msgIds);
+
 	/*!
 	 * @brief Update model data when records management entry is deleted.
 	 *
@@ -836,12 +1013,29 @@ public:
 	 */
 	bool connectToIsds(const AcntId &acntId, bool shadow = false);
 
+public slots:
+	/*!
+	 * @brief Changes the application theme.
+	 *
+	 * @param[in] index Theme index.
+	 */
+	void changeTheme(int index);
+
 private slots:
 	/*!
 	 * @brief Refresh account list.
 	 */
 	void refreshAccountList(const AcntId &acntId);
 
+	/*!
+	 * @brief Display a status bar notification that database file cannot
+	 *     be opened.
+	 *
+	 * @param[in] acntId Account identifier.
+	 * @param[in] fileName Database file name.
+	 */
+	void watchOpeningFailed(const AcntId &acntId, const QString &fileName);
+
 	/*!
 	 * @brief Processes messages from single instance emitter.
 	 */
@@ -1074,13 +1268,29 @@ private:
 	void viewSelectedMessageViaFilter(QObject *mwPtr);
 
 	/*!
-	 * @brief Used to clear message list filter via event filter.
+	 * @brief Used to clear message list filter line via event filter.
 	 *
 	 * @param[in] mwPtr Pointer to main window.
 	 */
 	static
 	void clearFilterLineViaFilter(QObject *mwPtr);
 
+	/*!
+	 * @brief Used to view message list filter line via event filter.
+	 *
+	 * @param[in] mwPtr Pointer to main window.
+	 */
+	static
+	void showFilterLineViaFilter(QObject *mwPtr);
+
+	/*!
+	 * @brief Used to hide message list filter line via event filter.
+	 *
+	 * @param[in] mwPtr Pointer to main window.
+	 */
+	static
+	void hideFilterLineViaFilter(QObject *mwPtr);
+
 	/*!
 	 * @brief Set info status bar from worker.
 	 */
@@ -1206,14 +1416,6 @@ private:
 	 */
 	bool regenerateAllAccountModelYears(void);
 
-	/*!
-	 * @brief Generate overall account information only for sent or
-	 *     received messages.
-	 */
-	QString createAccountInfoMessagesCount(const QString &accountName,
-	    const QList< QPair<QString, int> > &counts,
-	    enum MessageDb::MessageType type) const;
-
 	/*!
 	 * @brief Delete message from long term storage in ISDS and
 	 * local database - based on delFromIsds parameter.
@@ -1311,9 +1513,10 @@ private:
 	enum AccountModel::NodeType
 	    m_lastStoredAccountNodeType; /*!< Last stored account position. */
 
+	QDialog *m_timestampRenewalDlg; /*!< Non-null if timestamp renewal dialogue is active. */
 	QDialog *m_messageSearchDlg; /*!< Non-null if message search dialogue is active. */
 	QDialog *m_logDlg; /*!< Non-null if log dialogue is active. */
-	QDialog *m_askCloseDlg; /*!< Non-null if ask cancel stask dialogue is active. */
+	QDialog *m_askCloseDlg; /*!< Non-null if ask cancel task dialogue is active. */
 
 	QVector<int> m_colWidth; /*!< Message list column widths. */
 	int m_sortCol; /*!< Column with enabled sorting. */
@@ -1323,12 +1526,22 @@ private:
 	QList<DbMsgsTblModel::AppendedCol> m_msgTblAppendedCols; /*< Appended columns. */
 
 	/* User interface elements. */
-	QAction *mui_firstUnconfigurableToolbarElement; /*!< First tool bar element that cannot be configured. */
-	QLineEdit *mui_filterLine; /*!< Search filter line object. */
+	QAction *mui_firstUnconfigurableToolbarElement; /*!< First top tool bar element that cannot be configured. */
+	QAction *mui_lastUnconfigurableAccountToolBarElement; /*!< Last account list tool bar element that cannot be configured. */
+	QToolBar *mui_accountToolBar; /*!< Account list tool bar. */
+	QAction *mui_lastUnconfigurableMessageToolBarElement; /*!< Last message list tool bar element that cannot be configured. */
+	QAction *mui_firstUnconfigurableMessageToolBarElement; /*!< First message list tool bar element that cannot be configured. */
+	QToolBar *mui_messageToolBar; /*!< Message list tool bar. */
+	QAction *mui_lastUnconfigurableDetailToolBarElement; /*!< Last message detail tool bar element that cannot be configured. */
+	QComboBox *mui_messageStateCombo; /*!< Message state combo box placed on the message detail tool bar. */
+	QToolBar *mui_detailToolBar; /*!< Message detail tool bar. */
+	QAction *mui_lastUnconfigurableAttachmentToolBarElement; /*!< Last attachment list tool bar element that cannot be configured. */
+	QToolBar *mui_attachmentToolBar; /*!< Attachment list tool bar. */
 	QStatusBar *mui_statusBar; /*!< Status bar. */
-	QLabel *mui_statusDbMode; /*!< Database status label. */
-	QLabel *mui_statusTagMode; /*!< Tag storage status label. */
-	QLabel *mui_statusOnlineLabel; /*< On-line/off-line status label. */
+	QWidget *mui_dbStatusMode; /*!< Database status label/widget. */
+	QWidget *mui_statusLabelTimestamps; /*!< Timestamp status lalbel/widget. */
+	QWidget *mui_statusLabelTags; /*!< Tag status label/widget. */
+	QWidget *mui_statusAccounts; /*!< Account status label/widget. */
 	QProgressBar *mui_statusProgressBar; /*!< Progress bar. */
 	QMenu mui_dockMenu; /*!< Dock menu for macOS. */
 };
