@@ -41,8 +41,8 @@
 #include "src/io/filesystem.h"
 #include "src/io/message_db_set.h"
 #include "src/io/tag_container.h"
+#include "src/json/message_id.h"
 #include "src/json/tag_assignment_hash.h"
-#include "src/json/tag_message_id.h"
 #include "src/settings/accounts.h"
 #include "src/settings/prefs_specific.h"
 #include "ui_dlg_correspondence_overview.h"
@@ -278,10 +278,10 @@ QString tagHtmlEntry(TagContainer &tagCont, const AcntId &acntId, qint64 dmId,
 
 	TagItemList tagList;
 	{
-		const Json::TagMsgId msgId(
+		const Json::MsgId1 msgId(
 		    acntId.testing() ? Isds::Type::BOOL_TRUE : Isds::Type::BOOL_FALSE,
 		    dmId);
-		Json::TagMsgIdList msgIds({msgId});
+		Json::MsgId1List msgIds({msgId});
 		Json::TagAssignmentHash assignments;
 		if (Q_UNLIKELY(!tagCont.getMessageTags(msgIds, assignments))) {
 			return QString();
