@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2024 CZ.NIC
+ * Copyright (C) 2014-2025 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@
 #include <QMessageBox>
 
 #include "src/datovka_shared/compat/compiler.h" /* macroStdMove */
+#include "src/datovka_shared/html/html_export.h"
 #include "src/datovka_shared/isds/type_conversion.h"
 #include "src/datovka_shared/isds/type_description.h"
 #include "src/datovka_shared/log/log.h"
@@ -127,9 +128,6 @@ DlgCreateAccount::DlgCreateAccount(const AcntData &accountInfo,
 	m_ui->splitter->setChildrenCollapsible(false);
 	m_ui->splitter_2->setChildrenCollapsible(false);
 
-	m_ui->addCertButton->setIcon(
-	    IconContainer::construcIcon(IconContainer::ICON_PLUS));
-
 	m_ui->loginMethodComboBox->addItem(tr("Password"));
 	m_ui->loginMethodComboBox->addItem(tr("Certificate"));
 	m_ui->loginMethodComboBox->addItem(tr("Certificate + Password"));
@@ -180,6 +178,10 @@ DlgCreateAccount::DlgCreateAccount(const AcntData &accountInfo,
 	/* Set dialogue content for existing account. */
 	if (ACT_ADDNEW != m_action) {
 		setContent(accountInfo, FC_EDITABLE);
+	} else {
+		/* Change acceptance button text if a new account is created. */
+		m_ui->buttonBox->button(QDialogButtonBox::Ok)->setText(
+		    tr("Sign in and Add Data Box"));
 	}
 }
 
@@ -271,7 +273,7 @@ bool DlgCreateAccount::manage(AccountsMap &regularAccounts,
 	    SLOT(refreshUsersInformation()));
 
 	dlg.m_ui->userTableView->setNarrowedLineHeight();
-	dlg.m_ui->userTableView->horizontalHeader()->setDefaultAlignment(Qt::AlignLeft);
+	dlg.m_ui->userTableView->horizontalHeader()->setDefaultAlignment(Qt::AlignVCenter | Qt::AlignLeft);
 	dlg.m_ui->userTableView->setSelectionMode(
 	    QAbstractItemView::SingleSelection);
 	dlg.m_ui->userTableView->setSelectionBehavior(
@@ -368,196 +370,25 @@ QString DlgCreateAccount::htmlAccountInfo(const AcntData &acntData,
 {
 	Q_ASSERT(acntIdFromData(acntData).isValid());
 
-	QString html;
-	DbEntry userEntry;
-	DbEntry accountEntry;
-
-	html.append(indentDivStart);
-	html.append("<h3>");
-
-
-	if (acntData.isTestAccount()) {
-		html.append(tr("Test environment data box"));
-	} else {
-		html.append(tr("Production environment data box"));
-	}
-	html.append("<br/><img src=\":/dschranka.png\" alt=\"ISDS\">");
-	html.append("</h3>");
-
-	html.append(strongAccountInfoLine(tr("Data-box name"),
-	    acntData.accountName()));
-
 	const QString acntDbKey(AccountDb::keyFromLogin(acntData.userName()));
-	if (GlobInstcs::accntDbPtr->dbId(acntDbKey).isEmpty()) {
-		/*
-		 * Generate this message if no account information can
-		 * be obtained.
-		 */
-		html.append(QString("<div><strong>") +
-		    tr("Data-box and user information could not be acquired.") +
-		    QString("</strong></div>"));
-		goto lastPart;
-	}
-
-	html.append("<table cellpadding=\"5\"><tr><td>");
-	html.append(QString("<div><strong>") + tr("User information") +
-	    QString("</strong></div>"));
-	html.append("</td><td width=\"100\"></td><td>");
-	html.append(QString("<div><strong>") + tr("Databox information") +
-	    QString("</strong></div>"));
-	html.append("</td></tr><tr><td>");
-
-	userEntry = GlobInstcs::accntDbPtr->userEntry(acntDbKey);
-	html.append(strongAccountInfoLine(tr("User name"), acntData.userName()));
-	/* Print non-empty entries. */
-	for (int i = 0; i < userinfTbl.knownAttrs.size(); ++i) {
-		const QString &key = userinfTbl.knownAttrs[i].first;
-		if (userEntry.hasValue(key) &&
-		    !userinfTbl.attrProps[key].desc.isEmpty()) {
-			switch (userinfTbl.knownAttrs[i].second) {
-			case DB_INTEGER:
-				if (key == "ic") {
-					if (userEntry.value(key).toInt() > 0) {
-						html.append(strongAccountInfoLine(
-						    userinfTbl.attrProps[key].desc,
-						    QString("%1").arg(
-						        accountEntry.value(key).toInt(), 8, 10, QLatin1Char('0'))));
-					}
-				} else if (key == "userPrivils") {
-					html.append(strongAccountInfoLineNoEscape(
-					    userinfTbl.attrProps[key].desc,
-					    Isds::Description::htmlDescrPrivileges(
-					        Isds::variant2Privileges(userEntry.value(key)))));
-				} else {
-					html.append(strongAccountInfoLine(
-					    userinfTbl.attrProps[key].desc,
-					    QString::number(userEntry.
-					        value(key).toInt())));
-				}
-				break;
-			case DB_TEXT:
-				if (key == "userType") {
-					html.append(strongAccountInfoLine(
-					    userinfTbl.attrProps[key].desc,
-					    Isds::Description::descrSenderType(
-					        Isds::variant2SenderType(userEntry.value(key)))));
-				} else {
-					html.append(strongAccountInfoLine(
-					    userinfTbl.attrProps[key].desc,
-					    userEntry.value(key).toString()));
-				}
-				break;
-			case DB_DATE:
-				html.append(strongAccountInfoLine(
-				    userinfTbl.attrProps[key].desc,
-				    dateStrFromDbFormat(
-				        userEntry.value(key).toString(),
-				        Utility::dateDisplayFormat)));
-				break;
-			default:
-				Q_ASSERT(0);
-				break;
-			}
-		}
-	}
-
-	html.append("</td><td></td><td>");
-
-	accountEntry = GlobInstcs::accntDbPtr->accountEntry(acntDbKey);
-	/* Print non-empty entries. */
-	for (int i = 0; i < accntinfTbl.knownAttrs.size(); ++i) {
-		const QString &key = accntinfTbl.knownAttrs[i].first;
-		if (accountEntry.hasValue(key) &&
-		    !accntinfTbl.attrProps[key].desc.isEmpty()) {
-			switch (accntinfTbl.knownAttrs[i].second) {
-			case DB_INTEGER:
-				if (key == "dbState") {
-					html.append(strongAccountInfoLine(
-					    accntinfTbl.attrProps[key].desc,
-					    Isds::Description::descrDbState(
-					        Isds::variant2DbState(accountEntry.value(key)))));
-				} else if (key == "ic") {
-					if (accountEntry.value(key).toInt() > 0) {
-						html.append(strongAccountInfoLine(
-						    accntinfTbl.attrProps[key].desc,
-						    QString("%1").arg(
-						        accountEntry.value(key).toInt(), 8, 10, QLatin1Char('0'))));
-					}
-				} else {
-					html.append(strongAccountInfoLine(
-					    accntinfTbl.attrProps[key].desc,
-					    QString::number(accountEntry.
-					        value(key).toInt())));
-				}
-				break;
-			case DB_TEXT:
-				html.append(strongAccountInfoLine(
-				    accntinfTbl.attrProps[key].desc,
-				    accountEntry.value(key).toString()));
-				break;
-			case DB_BOOLEAN:
-				html.append(strongAccountInfoLine(
-				    accntinfTbl.attrProps[key].desc,
-				    accountEntry.value(key).toBool() ?
-				        tr("Yes") : tr("No")));
-				break;
-			case DB_DATETIME:
-				html.append(strongAccountInfoLine(
-				    accntinfTbl.attrProps[key].desc,
-				    dateTimeStrFromDbFormat(
-				        accountEntry.value(key).toString(),
-				        Utility::dateTimeDisplayFormat)));
-				break;
-			case DB_DATE:
-				html.append(strongAccountInfoLine(
-				    accntinfTbl.attrProps[key].desc,
-				    dateStrFromDbFormat(
-				        accountEntry.value(key).toString(),
-				        Utility::dateDisplayFormat)));
-				break;
-			default:
-				Q_ASSERT(0);
-				break;
-			}
-		}
-	}
-
-	html.append("</td></tr></table>");
-
-lastPart:
-
-	QString info(qDateTimeToDbFormat(GlobInstcs::accntDbPtr->getPwdExpirFromDb(acntDbKey)));
-	if (info.isEmpty()) {
-		info = tr("unknown or without expiration");
-	} else {
-		info = info.split(".")[0];
-	}
-
-	html.append(strongAccountInfoLine(tr("Password expiration date"),
-	    info));
+	QString html;
 
+	html.append(Html::Export::htmlDataboxOwnerInfoApp(
+	    GlobInstcs::accntDbPtr->getOwnerInfo2(acntDbKey)));
+	html.append(Html::Export::htmlDataboxUserInfoApp(
+	    GlobInstcs::accntDbPtr->getUserInfo2(acntDbKey)));
+	html.append(Html::Export::htmlPwdExpirDateInfoApp(qDateTimeToDbFormat(
+	    GlobInstcs::accntDbPtr->getPwdExpirFromDb(acntDbKey))));
 	if (dbSet != Q_NULLPTR) {
-		QStringList dbFilePaths = dbSet->fileNames();
-		if ((dbFilePaths.size() == 1) &&
-		    (SQLiteDb::memoryLocation == dbFilePaths.first())) {
-			QString dbFilePath = tr("Database is stored in memory. "
-			    "Data will be lost on application exit.");
-			html.append(strongAccountInfoLine(
-			    tr("Local database file location"), dbFilePath));
-		} else {
-			foreach (const QString &path, dbFilePaths) {
-				html.append(strongAccountInfoLine(
-				    tr("Local database file location"), path));
-			}
-		}
-	} else {
-		html.append(strongAccountInfoLine2(
-		    tr("Data box has no own message database associated.")));
+		html.append(Html::Export::htmlMsgDbLocationInfoApp(
+		    dbSet->fileNames()));
 	}
 
-	html.append(divEnd);
-
-	return html;
+	return htmlMarginsApp(htmlSectionLabel(
+	    acntData.isTestAccount() ?
+	    tr("Test Environment Data Box") :
+	    tr("Production Environment Data Box")),
+	    html);
 }
 
 void DlgCreateAccount::activateContent(int loginMethodIdx)
@@ -696,7 +527,7 @@ void DlgCreateAccount::addCertificateFile(void)
 	if (!certFileName.isEmpty()) {
 		m_ui->addCertButton->setText(QFileInfo(certFileName).fileName());
 		m_ui->addCertButton->setIcon(IconContainer::construcIcon(
-		    IconContainer::ICON_DATOVKA_STOCK_KEY));
+		    IconContainer::ICON_APP_OK));
 		m_ui->addCertButton->setToolTip(certFileName);
 		m_certPath = certFileName;
 		checkInputFields();
@@ -1279,166 +1110,6 @@ bool updateDataBoxUser2(AcntData &acntData, const QString &isdsID,
 	return ret;
 }
 
-/*!
- * @brief Generate user description HTML.
- *
- * @param[in] userInfo Use description.
- * @return HTML text.
- */
-static
-QString htmlUserInfo(const Isds::DbUserInfoExt2 &userInfo)
-{
-	if (Q_UNLIKELY(userInfo.isNull())) {
-		return QString();
-	}
-
-	QString html;
-
-	html.append(indentDivStart);
-
-	html.append("<h3>");
-	html.append(DlgCreateAccount::tr("User"));
-	html.append("</h3>");
-
-	/* aifoIsds -- Ignored for now. */
-
-	html.append(indentDivStart);
-	html.append(strongAccountInfoLineNoEscape(
-	    DlgCreateAccount::tr("ISDS ID"), userInfo.isdsID()));
-	html.append(divEnd);
-
-	{
-		const Isds::PersonName2 &personName = userInfo.personName();
-		if (!personName.isNull()) {
-			html.append("<div>&nbsp;</div>");
-			html.append(indentDivStart);
-
-			if (!personName.givenNames().isEmpty()) {
-				html.append(strongAccountInfoLineNoEscape(
-				    DlgCreateAccount::tr("Name"), personName.givenNames()));
-			}
-			if (!personName.lastName().isEmpty()) {
-				html.append(strongAccountInfoLineNoEscape(
-				    DlgCreateAccount::tr("Surname"), personName.lastName()));
-			}
-			if (userInfo.biDate().isValid()) {
-				html.append(strongAccountInfoLineNoEscape(
-				    DlgCreateAccount::tr("Birth date"), userInfo.biDate().toString(Utility::dateDisplayFormat)));
-			}
-
-			html.append(divEnd);
-		}
-	}
-
-	{
-		const Isds::AddressExt2 &address = userInfo.address();
-		if (!address.isNull()) {
-			html.append("<div>&nbsp;</div>");
-			html.append(strongAccountInfoLineNoEscape(
-			    DlgCreateAccount::tr("Address"), ""));
-			html.append(indentDivStart);
-
-			if (!address.city().isEmpty()) { /* CZ: Obec */
-				html.append(strongAccountInfoLineNoEscape(
-				    DlgCreateAccount::tr("City"), address.city()));
-			}
-			if (!address.district().isEmpty()) { /* CZ: Cast obce */
-				html.append(strongAccountInfoLineNoEscape(
-				    DlgCreateAccount::tr("District"), address.district()));
-			}
-			if (!address.street().isEmpty()) { /* CZ: Ulice */
-				html.append(strongAccountInfoLineNoEscape(
-				    DlgCreateAccount::tr("Street"), address.street()));
-			}
-			if (!address.numberInStreet().isEmpty()) { /* CZ: Cislo orientacni */
-				html.append(strongAccountInfoLineNoEscape(
-				    DlgCreateAccount::tr("Number in street"), address.numberInStreet()));
-			}
-			if (!address.numberInMunicipality().isEmpty()) { /* CZ: Cislo popisne/evidencni */
-				html.append(strongAccountInfoLineNoEscape(
-				    DlgCreateAccount::tr("Number in municipality"), address.numberInMunicipality()));
-			}
-			if (!address.zipCode().isEmpty()) { /* CZ: PSC */
-				html.append(strongAccountInfoLineNoEscape(
-				    DlgCreateAccount::tr("Postal code"), address.zipCode()));
-			}
-			if (!address.state().isEmpty()) { /* CZ: Stat */
-				html.append(strongAccountInfoLineNoEscape(
-				    DlgCreateAccount::tr("State"), address.state()));
-			}
-
-			html.append(divEnd);
-		}
-	}
-
-	{
-		const QString &ic = userInfo.ic();
-		const QString &firmName = userInfo.firmName();
-		if ((!ic.isEmpty()) || (!firmName.isEmpty())) {
-			html.append("<div>&nbsp;</div>");
-			html.append(strongAccountInfoLineNoEscape(
-			    DlgCreateAccount::tr("Firm"), ""));
-			html.append(indentDivStart);
-
-			if (!ic.isEmpty()) {
-				html.append(strongAccountInfoLineNoEscape(
-				    DlgCreateAccount::tr("ic"), ic));
-			}
-			if (!firmName.isEmpty()) {
-				html.append(strongAccountInfoLineNoEscape(
-				    DlgCreateAccount::tr("Name"), firmName));
-			}
-
-			html.append(divEnd);
-		}
-	}
-
-	{
-		const QString &street = userInfo.caStreet();
-		const QString &city = userInfo.caCity();
-		const QString &zip = userInfo.caZipCode();
-		const QString &state = userInfo.caState();
-		if ((!street.isEmpty()) || (!city.isEmpty()) ||
-		    (!zip.isEmpty()) || (!state.isEmpty())) {
-			html.append("<div>&nbsp;</div>");
-			html.append(strongAccountInfoLineNoEscape(
-			    DlgCreateAccount::tr("Contact address"), ""));
-			html.append(indentDivStart);
-
-			if (!city.isEmpty()) { /* CZ: Obec */
-				html.append(strongAccountInfoLineNoEscape(
-				    DlgCreateAccount::tr("City"), city));
-			}
-			if (!street.isEmpty()) { /* CZ: Ulice */
-				html.append(strongAccountInfoLineNoEscape(
-				    DlgCreateAccount::tr("Street"), street));
-			}
-			if (!zip.isEmpty()) { /* CZ: PSC */
-				html.append(strongAccountInfoLineNoEscape(
-				    DlgCreateAccount::tr("Postal code"), zip));
-			}
-			if (!state.isEmpty()) { /* CZ: Stat */
-				html.append(strongAccountInfoLineNoEscape(
-				    DlgCreateAccount::tr("State"), state));
-			}
-
-			html.append(divEnd);
-		}
-	}
-
-	html.append("<div>&nbsp;</div>");
-	html.append(strongAccountInfoLineNoEscape(
-	    DlgCreateAccount::tr("User type"),
-	    Isds::Description::descrUserType(userInfo.userType())));
-	html.append(strongAccountInfoLineNoEscape(
-	    DlgCreateAccount::tr("Privileges"),
-	    Isds::Description::htmlDescrPrivileges(userInfo.userPrivils())));
-
-	html.append(divEnd);
-
-	return html;
-}
-
 void DlgCreateAccount::userCurrentChanged(const QModelIndex &current,
     const QModelIndex &previous)
 {
@@ -1450,9 +1121,9 @@ void DlgCreateAccount::userCurrentChanged(const QModelIndex &current,
 	}
 
 	const QModelIndex userIndex = m_userListProxyModel.mapToSource(current);
-
-	m_ui->userTextInfo->setText(
-	    htmlUserInfo(m_usersModel.contentAtRow(userIndex.row())));
+	m_ui->userTextInfo->setText(htmlMarginsApp("",
+	    Html::Export::htmlDataboxUserInfoApp(
+	    m_usersModel.contentAtRow(userIndex.row()))));
 
 	/* Don't allow modifying own permissions or deleting yourself. */
 	bool yoursef =
@@ -2229,18 +1900,16 @@ void DlgCreateAccount::setContent(const AcntData &acntData, int properties)
 		if (fileInfo.exists() && fileInfo.isFile() &&
 		    fileInfo.isReadable()) {
 			m_ui->addCertButton->setIcon(IconContainer::construcIcon(
-			    IconContainer::ICON_DATOVKA_STOCK_KEY));
+			    IconContainer::ICON_APP_OK));
 			m_ui->addCertButton->setToolTip(acntData.p12File());
 		} else {
 			m_ui->addCertButton->setIcon(IconContainer::construcIcon(
-			    IconContainer::ICON_WARNING));
+			    IconContainer::ICON_APP_WARNING));
 			m_ui->addCertButton->setToolTip(tr("File does not exists or cannot be read."));
 		}
 		m_certPath = QDir::toNativeSeparators(acntData.p12File());
 	} else {
-		m_ui->addCertButton->setIcon(IconContainer::construcIcon(
-		    IconContainer::ICON_PLUS));
-		m_ui->addCertButton->setText(tr("Add"));
+		m_ui->addCertButton->setText(tr("Select File"));
 		m_ui->addCertButton->setToolTip(tr("Select a certificate."));
 	}
 
