@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2024 CZ.NIC
+ * Copyright (C) 2014-2025 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,6 +38,7 @@
 #include "src/settings/prefs_specific.h"
 #include "src/views/table_home_end_filter.h"
 #include "src/views/table_tab_ignore_filter.h"
+#include "src/worker/message_emitter.h"
 #include "src/worker/task_download_message.h"
 #include "ui_dlg_download_messages.h"
 
@@ -60,7 +61,7 @@ QList<MsgOriginAndUpladTarget> MsgOriginAndUpladTarget::fromMsgOriginList(
 {
 	QList<MsgOriginAndUpladTarget> originsAndTargets;
 
-	foreach (const MsgOrigin &origin, originList) {
+	for (const MsgOrigin &origin : originList) {
 		originsAndTargets.append(fromMsgOrigin(origin));
 	}
 
@@ -72,7 +73,7 @@ QList<MsgOrigin> MsgOriginAndUpladTarget::toMsgOriginList(
 {
 	QList<MsgOrigin> originList;
 
-	foreach (const MsgOriginAndUpladTarget &oaut, oautList) {
+	for (const MsgOriginAndUpladTarget &oaut : oautList) {
 		originList.append(oaut.toMsgOrigin());
 	}
 
@@ -105,7 +106,8 @@ void MessageDownloadThread::run(void)
 		}
 #define STORE_RAW_INTO_DB (!oaut.isVodz) /* Set to false if to store as files -- determined according to VoDZ. */
 		TaskDownloadMessage *task = new (::std::nothrow) TaskDownloadMessage(
-		    oaut.acntIdDb, oaut.direction, oaut.msgId, STORE_RAW_INTO_DB,
+		    oaut.acntIdDb, QString(),
+		    oaut.direction, oaut.msgId, STORE_RAW_INTO_DB,
 		    oaut.taskFlags, oaut.recMgmtHierarchyId);
 #undef STORE_RAW_INTO_DB
 		if (Q_UNLIKELY(task == Q_NULLPTR)) {
@@ -144,17 +146,20 @@ DlgDownloadMessages::DlgDownloadMessages(
     : QDialog(parent),
     m_ui(new (::std::nothrow) Ui::DlgDownloadMessages),
     m_finalProposal(finalProposal),
-    m_downloadListProxyModel(this),
+    m_downloadSortProxyModel(this),
+    m_downloadProgressProxyModel(this),
+    m_downloadProgressDelegate(this),
     m_downloadModel(this),
     m_breakDownloadLoop(false),
-    m_downloadThread(originAndTargetList, m_breakDownloadLoop)
+    m_downloadThread(originAndTargetList, m_breakDownloadLoop),
+    m_speeds()
 {
 	m_ui->setupUi(this);
 
 	initDialogue();
 
 	/* Load model content. */
-	foreach (const MsgOriginAndUpladTarget &oaut, originAndTargetList) {
+	for (const MsgOriginAndUpladTarget &oaut : originAndTargetList) {
 		m_downloadModel.appendData(oaut.acntIdDb, oaut.msgId,
 		    MsgDownloadModel::STAT_NOT_ATTEMPTED, QString());
 	}
@@ -252,7 +257,7 @@ QList<MsgOrigin> DlgDownloadMessages::download(
 		/* Collect all successfully downloaded messages. */
 		QList<MsgOrigin> downloadedList;
 		typedef QPair<AcntIdDb, MsgId> IdPair;
-		foreach (const IdPair &pair, dlg.m_downloadModel.downloaded()) {
+		for (const IdPair &pair : dlg.m_downloadModel.downloaded()) {
 			int i = containsEntry(originAndTargetList,
 			    pair.first, pair.second.dmId());
 			if (i >= 0) {
@@ -282,7 +287,7 @@ static
 bool containsDownloaded(
     const QList<MsgOriginAndUpladTarget> &originAndTargetList)
 {
-	foreach (const MsgOriginAndUpladTarget &oaut, originAndTargetList) {
+	for (const MsgOriginAndUpladTarget &oaut : originAndTargetList) {
 		if (oaut.downloaded) {
 			return true;
 		}
@@ -302,7 +307,7 @@ static
 bool containsUndownloaded(
     const QList<MsgOriginAndUpladTarget> &originAndTargetList)
 {
-	foreach (const MsgOriginAndUpladTarget &oaut, originAndTargetList) {
+	for (const MsgOriginAndUpladTarget &oaut : originAndTargetList) {
 		if (!oaut.downloaded) {
 			return true;
 		}
@@ -323,7 +328,7 @@ QList<MsgOriginAndUpladTarget> undownloaded(
     const QList<MsgOriginAndUpladTarget> &originAndTargetList)
 {
 	QList<MsgOriginAndUpladTarget> notDownloaded;
-	foreach (const MsgOriginAndUpladTarget &oaut, originAndTargetList) {
+	for (const MsgOriginAndUpladTarget &oaut : originAndTargetList) {
 		if (!oaut.downloaded) {
 			notDownloaded.append(oaut);
 		}
@@ -344,7 +349,7 @@ QSet<AcntIdDb> originAcntIds(
     const QList<MsgOriginAndUpladTarget> &originAndTargetList)
 {
 	QSet<AcntIdDb> acntSet;
-	foreach (const MsgOriginAndUpladTarget &oaut, originAndTargetList) {
+	for (const MsgOriginAndUpladTarget &oaut : originAndTargetList) {
 		acntSet.insert(oaut.acntIdDb);
 	}
 	return acntSet;
@@ -366,7 +371,7 @@ void DlgDownloadMessages::downloadAll(
 		    tr("The content of the following messages has already been downloaded:");
 		{
 			QStringList idList;
-			foreach (const MsgOriginAndUpladTarget &oaut, originAndTargetList) {
+			for (const MsgOriginAndUpladTarget &oaut : originAndTargetList) {
 				if (oaut.downloaded) {
 					idList.append(QString::number(
 					    oaut.msgId.dmId()));
@@ -391,7 +396,7 @@ void DlgDownloadMessages::downloadAll(
 	}
 
 	/* Try connection to all required accounts. */
-	foreach (const AcntIdDb &acntIdDb, originAcntIds(notDownloadedYet)) {
+	for (const AcntIdDb &acntIdDb : originAcntIds(notDownloadedYet)) {
 		if (!GlobInstcs::isdsSessionsPtr->isConnectedToIsds(acntIdDb.username())) {
 			if (mw != Q_NULLPTR) {
 				mw->connectToIsds(acntIdDb);
@@ -433,7 +438,7 @@ QList<MsgOrigin> DlgDownloadMessages::offerDownloadForMissing(
 		    tr("The content of the following messages has not been completely downloaded:");
 		{
 			QStringList idList;
-			foreach (const MsgOriginAndUpladTarget &oaut, originAndTargetList) {
+			for (const MsgOriginAndUpladTarget &oaut : originAndTargetList) {
 				if (!oaut.downloaded) {
 					idList.append(QString::number(
 					    oaut.msgId.dmId()));
@@ -454,7 +459,7 @@ QList<MsgOrigin> DlgDownloadMessages::offerDownloadForMissing(
 		if (res == QMessageBox::No) {
 			/* Remove messages with missing content. */
 			QList<MsgOrigin> list;
-			foreach (const MsgOrigin &origin, originAndTargetList) {
+			for (const MsgOrigin &origin : originAndTargetList) {
 				if (origin.downloaded) {
 					list.append(origin);
 				}
@@ -469,7 +474,7 @@ QList<MsgOrigin> DlgDownloadMessages::offerDownloadForMissing(
 	{
 		QList<MsgOriginAndUpladTarget> notDownloadedYet = undownloaded(originAndTargetList);
 		/* Try connection to all required accounts. */
-		foreach (const AcntIdDb &acntIdDb, originAcntIds(notDownloadedYet)) {
+		for (const AcntIdDb &acntIdDb : originAcntIds(notDownloadedYet)) {
 			if (!GlobInstcs::isdsSessionsPtr->isConnectedToIsds(acntIdDb.username())) {
 				if (mw != Q_NULLPTR) {
 					mw->connectToIsds(acntIdDb);
@@ -499,7 +504,7 @@ QList<MsgOrigin> DlgDownloadMessages::offerDownloadForMissing(
 				return QList<MsgOrigin>();
 			}
 			/* Set downloaded flag for all just downloaded messages. */
-			foreach (const MsgOrigin &downloaded, justDownloaded) {
+			for (const MsgOrigin &downloaded : justDownloaded) {
 				int i = containsEntry(downloadedList,
 				    downloaded.acntIdDb,
 				    downloaded.msgId.dmId());
@@ -614,10 +619,53 @@ void DlgDownloadMessages::downloadedMessage(int i, int total,
 	    error);
 }
 
+#define MSG_PROGRESS_COL MsgDownloadModel::COL_ERROR
+
+void DlgDownloadMessages::watchDownloadProgress(const AcntId &acntId,
+    const QString &transactId, const MsgId &msgId, qint64 downloadTotal,
+    qint64 downloadCurrent)
+{
+	Q_UNUSED(transactId);
+
+	if (Q_UNLIKELY(!m_speeds.contains(msgId))) {
+		m_speeds[msgId] = NetworkSpeed::createCounter(downloadCurrent);
+	} else {
+		NetworkSpeed &netSpeed = m_speeds[msgId];
+		netSpeed.updateCounter(downloadCurrent);
+	}
+
+	const NetworkSpeed &netSpeed = m_speeds[msgId];
+
+	int row = m_downloadModel.rowOf(acntId, msgId);
+	if (row >= 0) {
+		m_downloadProgressProxyModel.setProgress(row, MSG_PROGRESS_COL,
+		    0, downloadTotal, downloadCurrent, netSpeed.speedOverview());
+		return;
+	}
+}
+
+void DlgDownloadMessages::watchDownloadProgressFinished(const AcntId &acntId,
+    const QString &transactId, const MsgId &msgId, int result,
+    const QString &resultDesc)
+{
+	Q_UNUSED(transactId);
+	Q_UNUSED(result);
+	Q_UNUSED(resultDesc);
+
+	m_speeds.remove(msgId);
+
+	int row = m_downloadModel.rowOf(acntId, msgId);
+	if (row >= 0) {
+		m_downloadProgressProxyModel.clearProgress(
+		    row, MSG_PROGRESS_COL);
+		return;
+	}
+}
+
 void DlgDownloadMessages::initDialogue(void)
 {
 	m_ui->downloadView->setNarrowedLineHeight();
-	m_ui->downloadView->horizontalHeader()->setDefaultAlignment(Qt::AlignLeft);
+	m_ui->downloadView->horizontalHeader()->setDefaultAlignment(Qt::AlignVCenter | Qt::AlignLeft);
 	m_ui->downloadView->setSelectionMode(QAbstractItemView::NoSelection);
 	m_ui->downloadView->setSelectionBehavior(QAbstractItemView::SelectRows);
 
@@ -626,10 +674,13 @@ void DlgDownloadMessages::initDialogue(void)
 	m_ui->downloadView->installEventFilter(
 	    new (::std::nothrow) TableTabIgnoreFilter(m_ui->downloadView));
 
-	m_downloadListProxyModel.setSortRole(MsgDownloadModel::ROLE_PROXYSORT);
-	m_downloadListProxyModel.setSourceModel(&m_downloadModel);
+	m_downloadProgressProxyModel.setSourceModel(&m_downloadModel);
 
-	m_ui->downloadView->setModel(&m_downloadListProxyModel);
+	m_downloadSortProxyModel.setSortRole(MsgDownloadModel::ROLE_PROXYSORT);
+	m_downloadSortProxyModel.setSourceModel(&m_downloadProgressProxyModel);
+
+	m_ui->downloadView->setModel(&m_downloadSortProxyModel);
+	m_ui->downloadView->setItemDelegateForColumn(MSG_PROGRESS_COL, &m_downloadProgressDelegate);
 
 	m_ui->downloadView->setSortingEnabled(true);
 	m_ui->downloadView->sortByColumn(MsgDownloadModel::COL_MSG_ID, Qt::AscendingOrder);
@@ -644,11 +695,21 @@ void DlgDownloadMessages::initDialogue(void)
 	connect(m_ui->buttonBox, SIGNAL(rejected()), this, SLOT(interruptDownload()));
 
 	connect(&m_downloadThread, SIGNAL(started(int, int)),
-	    this, SLOT(downloadStarted(int, int)));
+	    this, SLOT(downloadStarted(int, int)),
+	    Qt::QueuedConnection); /* Qt::QueuedConnection because it must be executed in receiver's thread. */
 	connect(&m_downloadThread, SIGNAL(stopped(bool)),
-	    this, SLOT(downloadStopped(bool)));
+	    this, SLOT(downloadStopped(bool)),
+	    Qt::QueuedConnection); /* Qt::QueuedConnection because it must be executed in receiver's thread. */
 	connect(&m_downloadThread, SIGNAL(processed(int, int, AcntIdDb, MsgId, bool, QString)),
-	    this, SLOT(downloadedMessage(int, int, AcntIdDb, MsgId, bool, QString)));
+	    this, SLOT(downloadedMessage(int, int, AcntIdDb, MsgId, bool, QString)),
+	    Qt::QueuedConnection); /* Qt::QueuedConnection because it must be executed in receiver's thread. */
+
+	connect(GlobInstcs::msgProcEmitterPtr, SIGNAL(downloadProgress(AcntId, QString, MsgId, qint64, qint64)),
+	    this, SLOT(watchDownloadProgress(AcntId, QString, MsgId, qint64, qint64)),
+	    Qt::QueuedConnection); /* Qt::QueuedConnection because it must be executed in receiver's thread. */
+	connect(GlobInstcs::msgProcEmitterPtr, SIGNAL(downloadProgressFinished(AcntId, QString, MsgId, int, QString)),
+	    this, SLOT(watchDownloadProgressFinished(AcntId, QString, MsgId, int, QString)),
+	    Qt::QueuedConnection); /* Qt::QueuedConnection because it must be executed in receiver's thread. */
 
 	/* Disable accept button. */
 	m_ui->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
