@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2024 CZ.NIC
+ * Copyright (C) 2014-2025 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -100,12 +100,9 @@ DlgDsSearch::DlgDsSearch(const AcntId &acntId, const QString &dbType,
 	m_ui->setupUi(this);
 	/* Tab order is defined in UI file. */
 
-	m_ui->searchPushButton->setIcon(
-	    IconContainer::construcIcon(IconContainer::ICON_SEARCH));
-
 	/* Set default line height for table views/widgets. */
 	m_ui->contactTableView->setNarrowedLineHeight();
-	m_ui->contactTableView->horizontalHeader()->setDefaultAlignment(Qt::AlignLeft);
+	m_ui->contactTableView->horizontalHeader()->setDefaultAlignment(Qt::AlignVCenter | Qt::AlignLeft);
 	m_ui->contactTableView->setSelectionMode(
 	    QAbstractItemView::ExtendedSelection);
 	m_ui->contactTableView->setSelectionBehavior(
