@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2024 CZ.NIC
+ * Copyright (C) 2014-2025 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,6 +23,8 @@
 
 #include <QMenu>
 
+#include "src/datovka_shared/html/html_export.h"
+#include "src/datovka_shared/io/records_management_db.h"
 #include "src/datovka_shared/json/basic.h"
 #include "src/datovka_shared/log/log.h"
 #include "src/dimensions/dimensions.h"
@@ -32,6 +34,7 @@
 #include "src/gui/dlg_msg_search.h"
 #include "src/gui/icon_container.h"
 #include "src/gui/message_operations.h"
+#include "src/gui/styles.h"
 #include "src/io/message_db_set_container.h"
 #include "src/io/message_db_set.h"
 #include "src/io/tag_container.h"
@@ -83,7 +86,7 @@ DlgMsgSearch::DlgMsgSearch(const QList<AcntIdDb> &acntIdDbList,
 	/* Set default line height for table views/widgets. */
 	m_ui->resultsTableView->setNarrowedLineHeight();
 	m_ui->resultsTableView->horizontalHeader()->setDefaultAlignment(
-	    Qt::AlignLeft);
+	    Qt::AlignVCenter | Qt::AlignLeft);
 
 	m_ui->resultsTableView->setSelectionMode(
 	    QAbstractItemView::ExtendedSelection);
@@ -123,6 +126,9 @@ DlgMsgSearch::DlgMsgSearch(const QList<AcntIdDb> &acntIdDbList,
 		}
 	}
 
+	GuiStyles::setCalendarStyle(m_ui->fromDateEdit->calendarWidget());
+	GuiStyles::setCalendarStyle(m_ui->toDateEdit->calendarWidget());
+
 	setupActions();
 	initSearchWindow(acntId);
 }
@@ -180,6 +186,7 @@ void DlgMsgSearch::checkInputFields(void)
 	m_ui->rcpntFileMarkLine->setEnabled(msgIdMissing);
 	m_ui->addressLine->setEnabled(msgIdMissing);
 	m_ui->toHandsLine->setEnabled(msgIdMissing);
+	/* Always leave from and to date input fields enabled. */
 	m_ui->tagLine->setEnabled(msgIdMissing);
 	m_ui->attachNameLine->setEnabled(msgIdMissing);
 
@@ -222,19 +229,39 @@ void DlgMsgSearch::checkInputFields(void)
 	    (m_ui->rcpntBoxIdLine->text().isEmpty() ||
 	     (m_ui->rcpntBoxIdLine->text().size() == BOX_ID_LEN));
 
-	/* only 3 fields can be set together */
-	bool isNotFillManyFileds = true;
+	/* Only 3 fields can be set together (+ optional date restriction). */
+	bool adequateNumberOfFields = true;
 
 	const int itemsWithoutTag = filledInExceptTags();
 	if (itemsWithoutTag > 3) {
-		isNotFillManyFileds = false;
+		/* Too many fields. */
+		adequateNumberOfFields = false;
 		m_ui->tooManyFields->show();
 	} else if ((itemsWithoutTag < 1) && m_ui->tagLine->text().isEmpty()) {
-		isNotFillManyFileds = false;
+		/* Not enough if no date specified. */
+		adequateNumberOfFields =
+		    m_ui->fromDateEdit->isEnabled() || m_ui->toDateEdit->isEnabled();
 	}
 
-	m_ui->searchPushButton->setEnabled(isAnyMsgTypeChecked &&
-	    boxIdCorrect && tagCorrect && isNotFillManyFileds);
+	m_ui->searchPushButton->setEnabled(
+	    (isAnyMsgTypeChecked && adequateNumberOfFields) &&
+	    (boxIdCorrect && tagCorrect));
+}
+
+void DlgMsgSearch::fromDateEnable(bool enable)
+{
+	m_ui->fromDateEdit->setEnabled(enable);
+}
+
+void DlgMsgSearch::toDateEnable(bool enable)
+{
+	m_ui->toDateEdit->setEnabled(enable);
+}
+
+void DlgMsgSearch::reflectDateChange(void)
+{
+	m_ui->toDateEdit->setMinimumDate(m_ui->fromDateEdit->date());
+	checkInputFields();
 }
 
 /*!
@@ -257,8 +284,8 @@ QList<MessageDb::SoughtMsg> dataIntersect(
 
 	QList<MessageDb::SoughtMsg> result;
 
-	foreach (const qint64 msgId, tagData) {
-		foreach (const MessageDb::SoughtMsg msg, envelData) {
+	for (const qint64 msgId : tagData) {
+		for (const MessageDb::SoughtMsg &msg : envelData) {
 			if (msg.mId.dmId() == msgId) {
 				MessageDb::SoughtMsg msgData(
 				    messageDbSet->msgsGetMsgDataFromId(msgId));
@@ -290,7 +317,7 @@ QList<MessageDb::SoughtMsg> displayableTagData(const QList<qint64> &tagData,
 
 	QList<MessageDb::SoughtMsg> result;
 
-	foreach (const qint64 msgId, tagData) {
+	for (const qint64 msgId : tagData) {
 		MessageDb::SoughtMsg msgData(
 		    messageDbSet->msgsGetMsgDataFromId(msgId));
 
@@ -314,7 +341,7 @@ void setMessageDetails(MsgFoundModel &foundTableModel,
     const AcntIdDb &accountDescr,
     const QList<MessageDb::SoughtMsg> &msgDataList)
 {
-	foreach (const MessageDb::SoughtMsg &msgData, msgDataList) {
+	for (const MessageDb::SoughtMsg &msgData : msgDataList) {
 		MessageDb *messageDb =
 		    accountDescr.messageDbSet()->constAccessMessageDb(
 		        msgData.mId.deliveryTime());
@@ -323,8 +350,12 @@ void setMessageDetails(MsgFoundModel &foundTableModel,
 			continue;
 		}
 		foundTableModel.setMessageDetail(accountDescr, msgData.mId,
-		    messageDb->descriptionHtml(msgData.mId.dmId(),
-		        QString(), false));
+		    htmlMarginsApp("",
+		    Html::Export::htmlMessageInfoApp(
+		    messageDb->getMessageEnvelope(msgData.mId.dmId()),
+		    messageDb->messageAuthorJsonStr(msgData.mId.dmId()),
+		    GlobInstcs::recMgmtDbPtr->storedMsgLocations(msgData.mId.dmId()),
+		    true, false, true)));
 	}
 }
 
@@ -396,9 +427,6 @@ void DlgMsgSearch::searchMessages(void)
 		tagResults = results[request];
 	}
 
-	/* How many envelope fields (without tags) are supplied. */
-	const int envelopeItems = filledInExceptTags();
-
 	/* Fill search envelope items. */
 	Isds::Envelope searchEnvelope;
 	searchEnvelope.setDmId(m_ui->msgIdLine->text().isEmpty() ? -1 :
@@ -430,13 +458,13 @@ void DlgMsgSearch::searchMessages(void)
 		envelResults.clear();
 		resultsToBeDisplayed.clear();
 
-		if (envelopeItems > 0) {
-			/* Search in envelope envelope data. */
-			envelResults = acntIdDb.messageDbSet()->msgsSearch(
-			    searchEnvelope, msgType,
-			    m_ui->attachNameLine->text(),
-			    m_ui->logicalAndRelationCheckBox->isChecked());
-		}
+		/* Search in envelope envelope data. */
+		envelResults = acntIdDb.messageDbSet()->msgsSearch(
+		    searchEnvelope, msgType,
+		    m_ui->attachNameLine->text(),
+		    m_ui->logicalAndRelationCheckBox->isChecked(),
+		    m_ui->fromDateEdit->isEnabled() ? m_ui->fromDateEdit->date() : QDate(),
+		    m_ui->toDateEdit->isEnabled() ? m_ui->toDateEdit->date() : QDate());
 
 		if (searchTags && !tagResults.isEmpty() && !envelResults.isEmpty()) {
 			/*
@@ -752,7 +780,7 @@ void DlgMsgSearch::setupActions(void)
 	m_actionDownloadMessage = new (::std::nothrow) QAction(this);
 	if (m_actionDownloadMessage != Q_NULLPTR) {
 		m_actionDownloadMessage->setObjectName(QString::fromUtf8("actionDownloadMessage"));
-		m_actionDownloadMessage->setIcon(IconContainer::construcIcon(IconContainer::ICON_DATOVKA_MESSAGE_DOWNLOAD));
+		m_actionDownloadMessage->setIcon(IconContainer::construcIcon(IconContainer::ICON_MESSAGE_DOWNLOAD));
 		m_actionDownloadMessage->setMenuRole(QAction::NoRole);
 		m_actionDownloadMessage->setText(tr("Download Signed Message"));
 		m_actionDownloadMessage->setToolTip(tr("Download the complete message, i.e. including attachments, and verify its signature."));
@@ -865,7 +893,7 @@ void DlgMsgSearch::initSearchWindow(const AcntId &acntId)
 	Q_ASSERT(acntId.isValid());
 
 	/* Set accounts into combobox - account name and user name. */
-	foreach (const AcntIdDb &acntIdDb, m_acntIdDbList) {
+	for (const AcntIdDb &acntIdDb : m_acntIdDbList) {
 		const AcntId aId(acntIdDb);
 		const QString accountName =
 		    GlobInstcs::acntMapPtr->acntData(aId).accountName() +
@@ -886,8 +914,8 @@ void DlgMsgSearch::initSearchWindow(const AcntId &acntId)
 	m_ui->tooManyFields->setStyleSheet("QLabel { color: red }");
 	m_ui->tooManyFields->hide();
 
-	m_ui->searchPushButton->setIcon(
-	    IconContainer::construcIcon(IconContainer::ICON_SEARCH));
+	m_ui->fromDateEdit->setEnabled(false);
+	m_ui->toDateEdit->setEnabled(false);
 
 	connect(m_ui->searchRcvdMsgCheckBox, SIGNAL(clicked()),
 	    this, SLOT(checkInputFields()));
@@ -921,6 +949,19 @@ void DlgMsgSearch::initSearchWindow(const AcntId &acntId)
 	connect(m_ui->toHandsLine, SIGNAL(textChanged(QString)),
 	    this, SLOT(checkInputFields()));
 
+	connect(m_ui->fromDateCheckBox, SIGNAL(toggled(bool)),
+	    this, SLOT(fromDateEnable(bool)));
+	connect(m_ui->fromDateCheckBox, SIGNAL(toggled(bool)),
+	    this, SLOT(checkInputFields()));
+	connect(m_ui->toDateCheckBox, SIGNAL(toggled(bool)),
+	    this, SLOT(toDateEnable(bool)));
+	connect(m_ui->toDateCheckBox, SIGNAL(toggled(bool)),
+	    this, SLOT(checkInputFields()));
+	connect(m_ui->fromDateEdit, SIGNAL(dateChanged(QDate)),
+	    this, SLOT(reflectDateChange()));
+	connect(m_ui->toDateEdit, SIGNAL(dateChanged(QDate)),
+	    this, SLOT(reflectDateChange()));
+
 	connect(m_ui->tagLine, SIGNAL(textChanged(QString)),
 	    this, SLOT(checkInputFields()));
 
@@ -948,6 +989,8 @@ void DlgMsgSearch::initSearchWindow(const AcntId &acntId)
 	m_ui->resultsTableView->installEventFilter(
 	    new (::std::nothrow) TableTabIgnoreFilter(m_ui->resultsTableView));
 
+	m_ui->toDateEdit->setDate(QDate::currentDate());
+
 	databaseConnectActions();
 }
 
