@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2024 CZ.NIC
+ * Copyright (C) 2014-2025 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 #include <QFileDialog>
 #include <QMessageBox>
 #include <QString>
+#include <QToolTip>
 
 #include "src/common.h"
 #include "src/datovka_shared/graphics/colour.h"
@@ -35,20 +36,13 @@
 #include "src/gui/dlg_pin_setup.h"
 #include "src/gui/dlg_preferences.h"
 #include "src/gui/dlg_view_filename_param_list.h"
+#include "src/initialisation_gui.h" /* loadAppUiThemeFromValues() */
 #include "src/settings/prefs_specific.h"
 #include "ui_dlg_preferences.h"
 
 #define MSEC_IN_SEC 1000 /* One second in milliseconds. */
 #define MSEC_IN_MIN 60000 /* One minute in milliseconds. */
 
-#define TAB_DOWNLOADING 0
-#define TAB_SECURITY 1
-#define TAB_NAVIGATION 2
-#define TAB_INTERFACE 3
-#define TAB_DIRECTORIES 4
-#define TAB_SAVING 5
-#define TAB_LANGUAGE 6
-
 /*!
  * @brief Language order as they are listed in the combo box.
  */
@@ -59,7 +53,7 @@ enum LangIndex {
 };
 
 DlgPreferences::DlgPreferences(const Prefs &prefs, const PinSettings &pinSett,
-    QWidget *parent)
+    int tabNum, QWidget *parent)
     : QDialog(parent),
     m_ui(new (::std::nothrow) Ui::DlgPreferences),
     m_pinSett(pinSett),
@@ -79,6 +73,10 @@ DlgPreferences::DlgPreferences(const Prefs &prefs, const PinSettings &pinSett,
 	}
 	initDialogue(prefs);
 	activatePinButtons(m_pinSett);
+
+	if ((0 <= tabNum) && (tabNum < MAX_TABNUM)) {
+		m_ui->prefWidget->setCurrentIndex(tabNum);
+	}
 }
 
 DlgPreferences::~DlgPreferences(void)
@@ -87,9 +85,10 @@ DlgPreferences::~DlgPreferences(void)
 	delete m_ui;
 }
 
-bool DlgPreferences::modify(Prefs &prefs, PinSettings &pinSett, QWidget *parent)
+bool DlgPreferences::modify(Prefs &prefs, PinSettings &pinSett, int tabNum,
+    QWidget *parent)
 {
-	DlgPreferences dlg(prefs, pinSett, parent);
+	DlgPreferences dlg(prefs, pinSett, tabNum, parent);
 	const QString dlgName("preferences");
 
 	const QSize dfltSize = dlg.size();
@@ -102,6 +101,8 @@ bool DlgPreferences::modify(Prefs &prefs, PinSettings &pinSett, QWidget *parent)
 		}
 	}
 
+	connect(&dlg, SIGNAL(themeChanged(int)), parent, SLOT(changeTheme(int)));
+
 	int ret = dlg.exec();
 
 	PrefsSpecific::setDlgSize(*GlobInstcs::prefsPtr, dlgName,
@@ -112,6 +113,8 @@ bool DlgPreferences::modify(Prefs &prefs, PinSettings &pinSett, QWidget *parent)
 		return true;
 	}
 
+	/* Reset theme to value stored in preferences. */
+	emit dlg.themeChanged(-1);
 	return false;
 }
 
@@ -249,6 +252,21 @@ void DlgPreferences::setMsgDelColour(void)
 	}
 }
 
+void DlgPreferences::setApplicationThemeIndex(int index)
+{
+	/*
+	 * Reload entire theme via slot in main window connected to the
+	 * emitted themeChanged signal.
+	 */
+	emit themeChanged(index);
+}
+
+void DlgPreferences::setApplicationThemeFontScale(int percent)
+{
+	loadAppUiThemeFromValues(m_ui->appThemeComboBox->currentIndex(),
+	    percent);
+}
+
 void DlgPreferences::activateAllAttachDelivInfoFileNameFmt(void)
 {
 	m_ui->allAttachDelInfoFileNameFmt->setEnabled(
@@ -293,6 +311,27 @@ void DlgPreferences::onTabChangedCloseParamListDlg(int index)
 	}
 }
 
+void DlgPreferences::showBuildReportDetail(void)
+{
+	QToolTip::showText(
+	    m_ui->collectBuildReportLabel->mapToGlobal(QPoint(0, 0)),
+	    m_ui->collectBuildReportLabel->toolTip());
+}
+
+void DlgPreferences::showConfReportDetail(void)
+{
+	QToolTip::showText(
+	    m_ui->collectConfReportLabel->mapToGlobal(QPoint(0, 0)),
+	    m_ui->collectConfReportLabel->toolTip());
+}
+
+void DlgPreferences::showUsageReportDetail(void)
+{
+	QToolTip::showText(
+	    m_ui->collectUsageReport->mapToGlobal(QPoint(0, 0)),
+	    m_ui->collectUsageReport->toolTip());
+}
+
 /*!
  * @brief Converts selected language index to language name.
  *
@@ -351,8 +390,6 @@ void DlgPreferences::saveSettings(Prefs &prefs, PinSettings &pinSett) const
 	    m_ui->timeoutMarkMsgSpinBox->value() * MSEC_IN_SEC);
 	PrefsSpecific::setCheckNewVersions(prefs,
 	    m_ui->checkNewVersions->isChecked());
-	PrefsSpecific::setSendStatsWithVersionChecks(prefs,
-	    m_ui->sendStatsWithVersionChecks->isChecked());
 
 	/* security */
 	prefs.setBoolVal("storage.database.messages.on_disk.enabled",
@@ -384,6 +421,12 @@ void DlgPreferences::saveSettings(Prefs &prefs, PinSettings &pinSett) const
 
 	/* interface */
 	{
+		prefs.setIntVal(
+		    "application.ui.theme",
+		    m_ui->appThemeComboBox->currentIndex());
+		prefs.setIntVal(
+		    "application.ui.theme.custom.font_scale.percent",
+		    m_ui->fontScaleSpinBox->value());
 		prefs.setIntVal(
 		    "window.main.message.isds_deletion.notify_ahead.days",
 		    m_ui->messageToBeDeletedSpinBox->value());
@@ -427,6 +470,9 @@ void DlgPreferences::saveSettings(Prefs &prefs, PinSettings &pinSett) const
 	/* language */
 	prefs.setStrVal("translation.language",
 	    indexToSettingsLang(m_ui->langComboBox->currentIndex()));
+
+	/* data collection */
+	PrefsSpecific::setStatsReportUsage(prefs, m_ui->collectUsageReport->isChecked());
 }
 
 void DlgPreferences::initDialogue(const Prefs &prefs)
@@ -602,6 +648,7 @@ void DlgPreferences::initDialogue(const Prefs &prefs)
 			}
 		}
 	}
+
 	m_ui->labelToBeDeletedHighlightNote->setText(tr(
 	    "Note: Messages to be deleted from the ISDS within the specified period are going to be highlighted using the set colour in the message listing. "
 	    "Use 0 to disable the function."));
@@ -610,6 +657,27 @@ void DlgPreferences::initDialogue(const Prefs &prefs)
 	connect(m_ui->changeMessageToBeDeletedColourButton, SIGNAL(clicked()),
 	    this, SLOT(setMsgDelColour()));
 
+#if defined(Q_OS_MACOS)
+	m_ui->appThemeComboBox->addItem(tr("Default"));
+	m_ui->appThemeComboBox->addItem(tr("Custom"));
+#else /* !defined(Q_OS_MACOS) */
+	m_ui->appThemeComboBox->addItem(tr("Default"));
+	m_ui->appThemeComboBox->addItem(tr("Light"));
+	m_ui->appThemeComboBox->addItem(tr("Dark"));
+#endif /* defined(Q_OS_MACOS) */
+
+	connect(m_ui->appThemeComboBox, SIGNAL(currentIndexChanged(int)),
+	    this, SLOT(setApplicationThemeIndex(int)));
+
+	m_ui->appThemeComboBox->setCurrentIndex(
+	    PrefsSpecific::appUiTheme(*GlobInstcs::prefsPtr));
+
+	connect(m_ui->fontScaleSpinBox, SIGNAL(valueChanged(int)),
+	    this, SLOT(setApplicationThemeFontScale(int)));
+
+	m_ui->fontScaleSpinBox->setValue(
+	    PrefsSpecific::appUiThemeFontScalePercent(*GlobInstcs::prefsPtr));
+
 	/* directories */
 	m_ui->enableGlobalPaths->setChecked(
 	    PrefsSpecific::useGlobalPaths(prefs));
@@ -716,6 +784,21 @@ void DlgPreferences::initDialogue(const Prefs &prefs)
 		}
 	}
 
+	/* data collection*/
+	m_ui->collectUsageReport->setChecked(PrefsSpecific::statsReportUsage(prefs));
+
+	/* Set detail tool tips. */
+	m_ui->buildReportToolButton->setToolTip(m_ui->collectBuildReportLabel->toolTip());
+	m_ui->confReportToolButton->setToolTip(m_ui->collectConfReportLabel->toolTip());
+	m_ui->usageReportToolButton->setToolTip(m_ui->collectUsageReport->toolTip());
+
+	connect(m_ui->buildReportToolButton, SIGNAL(clicked()),
+	    this, SLOT(showBuildReportDetail()));
+	connect(m_ui->confReportToolButton, SIGNAL(clicked()),
+	    this, SLOT(showConfReportDetail()));
+	connect(m_ui->usageReportToolButton, SIGNAL(clicked()),
+	    this, SLOT(showUsageReportDetail()));
+
 	connect(m_ui->prefWidget, SIGNAL(currentChanged(int)),
 	    this, SLOT(onTabChangedCloseParamListDlg(int)));
 }
