@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2024 CZ.NIC
+ * Copyright (C) 2014-2025 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -99,7 +99,6 @@ DlgSendMessage::DlgSendMessage(const QList<AcntIdDb> &acntIdDbList,
     const QString &composeSerialised, class MainWindow *mw, QWidget *parent)
     : QDialog(parent),
     m_ui(new (::std::nothrow) Ui::DlgSendMessage),
-    m_keepAliveTimer(),
     m_acntIdDbList(acntIdDbList),
     m_acntId(acntId),
     m_boxId(),
@@ -149,16 +148,14 @@ DlgSendMessage::DlgSendMessage(const QList<AcntIdDb> &acntIdDbList,
 	/* Generate random directory where files can be created. */
 	m_tmpDirPath = createTemporarySubdir(TMP_DIR_NAME);
 
-	setIcons();
-
 	/* Set default line height for table views/widgets. */
 	m_ui->recipTableView->setNarrowedLineHeight();
-	m_ui->recipTableView->horizontalHeader()->setDefaultAlignment(Qt::AlignLeft);
+	m_ui->recipTableView->horizontalHeader()->setDefaultAlignment(Qt::AlignVCenter | Qt::AlignLeft);
 	m_ui->recipTableView->setSelectionBehavior(
 	    QAbstractItemView::SelectRows);
 
 	m_ui->attachTableView->setNarrowedLineHeight();
-	m_ui->attachTableView->horizontalHeader()->setDefaultAlignment(Qt::AlignLeft);
+	m_ui->attachTableView->horizontalHeader()->setDefaultAlignment(Qt::AlignVCenter | Qt::AlignLeft);
 	m_ui->attachTableView->setSelectionBehavior(
 	    QAbstractItemView::SelectRows);
 
@@ -420,9 +417,9 @@ void DlgSendMessage::openSelectedAttachment(const QModelIndex &index)
 	    index);
 }
 
-void DlgSendMessage::pingIsdsServer(void) const
+void DlgSendMessage::selectTabTextMessage(void)
 {
-	GuiHelper::pingIsdsServer(m_acntId);
+	m_ui->tabWidget->setCurrentIndex(TAB_TEXT_MESSAGE);
 }
 
 /*!
@@ -480,14 +477,17 @@ void DlgSendMessage::setAccountInfo(int fromComboIdx)
 		return;
 	}
 
+	QStringList recipientBoxIds;
+
 	/* Remove all recipients if account was changed. */
 	if (m_acntId != acntId) {
+		/* Remember old recipients. */
+		recipientBoxIds = m_recipTableModel.boxIdentifiers(BoxContactsModel::ANY);
 		m_recipTableModel.removeRows(0, m_recipTableModel.rowCount());
 		m_acntId = acntId;
 	}
 
-	m_isLoggedIn = GuiHelper::isLoggedIn(m_keepAliveTimer, m_mw,
-	    m_acntId);
+	m_isLoggedIn = GuiHelper::isLoggedIn(m_mw, m_acntId);
 
 	m_dbSet = GuiHelper::getDbSet(m_acntIdDbList, m_acntId);
 	if (Q_UNLIKELY(Q_NULLPTR == m_dbSet)) {
@@ -533,6 +533,19 @@ void DlgSendMessage::setAccountInfo(int fromComboIdx)
 	if (!m_dbEffectiveOVM) {
 		setPDZInfos(getPdzInfoFromIsds(m_acntId, m_boxId));
 	}
+
+	/* Fill in previously filled in recipients. */
+	if (!recipientBoxIds.isEmpty()) {
+		int reply = DlgMsgBoxDetail::message(this,
+			    QMessageBox::Question, tr("Retain Recipients?"),
+			    tr("The sender data box has been changed to '%1'. It's likely that it won't be possible to send messages to some already filled-in recipients from the newly selected data box.").arg(m_boxId),
+			    tr("Do you want to keep all previously filled-in recipients in the recipient list?"),
+			    QString(),
+			    QMessageBox::Yes | QMessageBox::No, QMessageBox::Yes);
+			if (reply == QMessageBox::Yes) {
+				addRecipientBoxes(recipientBoxIds);
+			}
+	}
 }
 
 void DlgSendMessage::sendMessage(void)
@@ -659,6 +672,10 @@ void DlgSendMessage::appendPdf(void)
 {
 	QString filePath(createPdf());
 
+	m_ui->warningText->setVisible(false);
+	m_ui->warningText->setStyleSheet(QString());
+	m_ui->warningText->setText(QString());
+
 	if (!filePath.isEmpty()) {
 		m_attachModel.insertAttachmentFile(filePath,
 		    m_attachModel.rowCount());
@@ -666,6 +683,33 @@ void DlgSendMessage::appendPdf(void)
 	}
 }
 
+void DlgSendMessage::watchAppendedPdfModification(void)
+{
+	const bool alreadyAttached = m_attachModel.holdsAttachmentFile(createdPdfPath());
+	const bool sourcesEmpty = m_ui->textMessageEdit->toPlainText().isEmpty();
+
+	QString warningtext;
+	if (alreadyAttached) {
+		warningtext =
+		    tr("The source text of the text message has been modified since it has been appended to the attachments as a PDF file.")
+		    + "\n" +
+		    tr("Append the text message using the '%1' button on the '%2' tab in order to apply all modifications to the attachment.")
+		        .arg(m_ui->appendPdfButton->text())
+		        .arg(m_ui->tabWidget->tabText(TAB_TEXT_MESSAGE));
+	} else {
+		warningtext =
+		    tr("The source text of the text message has been edited but it isn't attached as a PDF file.")
+		    + "\n" +
+		    tr("Append the text message using the '%1' button on the '%2' tab after you've finished editing.")
+		        .arg(m_ui->appendPdfButton->text())
+		        .arg(m_ui->tabWidget->tabText(TAB_TEXT_MESSAGE));
+	}
+
+	m_ui->warningText->setVisible(alreadyAttached || !sourcesEmpty);
+	m_ui->warningText->setStyleSheet("QLabel { color: red }");
+	m_ui->warningText->setText(warningtext);
+}
+
 void DlgSendMessage::preview(void)
 {
 	/* Setup taken from Utility::printPDF(). */
@@ -762,7 +806,7 @@ void DlgSendMessage::watchUploadProgress(const AcntId &acntId,
 		/* Update attachment progress. */
 		m_attachProgressProxyModel.setProgress(
 		    row, ATTACH_PROGRESS_COL,
-		    0, uploadTotal, uploadCurrent);
+		    0, uploadTotal, uploadCurrent, QString());
 		return;
 	}
 
@@ -771,7 +815,7 @@ void DlgSendMessage::watchUploadProgress(const AcntId &acntId,
 		/* Update message progress. */
 		m_recipProgressProxyModel.setProgress(
 		    row, RECIP_PROGRESS_COL,
-		    0, uploadTotal, uploadCurrent);
+		    0, uploadTotal, uploadCurrent, QString());
 		return;
 	}
 }
@@ -891,6 +935,10 @@ void DlgSendMessage::watchUploadProgressFinished(const AcntId &acntId,
 
 void DlgSendMessage::initContent(enum Action action, const QList<MsgId> &msgIds)
 {
+	m_ui->warningText->setVisible(false);
+	m_ui->warningText->setStyleSheet(QString());
+	m_ui->warningText->setText(QString());
+
 	m_recipTableModel.setHeader();
 	m_recipProgressProxyModel.setSourceModel(&m_recipTableModel);
 	m_ui->recipTableView->setModel(&m_recipProgressProxyModel);
@@ -974,6 +1022,8 @@ void DlgSendMessage::initContent(enum Action action, const QList<MsgId> &msgIds)
 	    SLOT(deleteSelectedAttachmentFiles()));
 	connect(m_ui->openAttachButton, SIGNAL(clicked()), this,
 	    SLOT(openSelectedAttachment()));
+	connect(m_ui->addTextButton, SIGNAL(clicked()), this,
+	    SLOT(selectTabTextMessage()));
 
 	connect(m_ui->recipTableView->selectionModel(),
 	    SIGNAL(selectionChanged(QItemSelection, QItemSelection)), this,
@@ -989,6 +1039,8 @@ void DlgSendMessage::initContent(enum Action action, const QList<MsgId> &msgIds)
 	    this, SLOT(checkInputFields()));
 	connect(&m_attachModel, SIGNAL(rowsRemoved(QModelIndex, int, int)),
 	    this, SLOT(checkInputFields()));
+	connect(&m_attachModel, SIGNAL(rowsRemoved(QModelIndex, int, int)),
+	    this, SLOT(watchAppendedPdfModification()));
 	connect(&m_attachModel,
 	    SIGNAL(dataChanged(QModelIndex, QModelIndex, QVector<int>)),
 	    this, SLOT(checkInputFields()));
@@ -1031,7 +1083,8 @@ void DlgSendMessage::initContent(enum Action action, const QList<MsgId> &msgIds)
 	    SIGNAL(sendMessageFinished(AcntId, QString, int, QString,
 	        QString, QString, bool, bool, qint64, int, QString)), this,
 	    SLOT(collectSendMessageStatus(AcntId, QString, int, QString,
-	        QString, QString, bool, bool, qint64, int, QString )));
+	        QString, QString, bool, bool, qint64, int, QString )),
+	    Qt::QueuedConnection); /* Qt::QueuedConnection because it must be executed in receiver's thread. */
 
 	connect(GlobInstcs::msgProcEmitterPtr, SIGNAL(uploadProgress(AcntId, QString, qint64, qint64)),
 	    this, SLOT(watchUploadProgress(AcntId, QString, qint64, qint64)),
@@ -1042,10 +1095,6 @@ void DlgSendMessage::initContent(enum Action action, const QList<MsgId> &msgIds)
 	    this, SLOT(watchUploadProgressFinished(AcntId, QString, int, QString, QVariant)),
 	    Qt::QueuedConnection); /* Qt::QueuedConnection because it must be executed in receiver's thread. */
 
-	m_keepAliveTimer.start(DLG_ISDS_KEEPALIVE_MS);
-
-	connect(&m_keepAliveTimer, SIGNAL(timeout()), this, SLOT(pingIsdsServer()));
-
 	{
 		int item = 0;
 		/* Always have plain text at disposal. */
@@ -1078,6 +1127,8 @@ void DlgSendMessage::initContent(enum Action action, const QList<MsgId> &msgIds)
 
 	connect(m_ui->textMessageEdit, SIGNAL(textChanged()),
 	    this, SLOT(textMessageChanged()));
+	connect(m_ui->textMessageEdit, SIGNAL(textChanged()),
+	    this, SLOT(watchAppendedPdfModification()));
 	connect(m_ui->previewButton, SIGNAL(clicked()),
 	    this, SLOT(preview()));
 	connect(m_ui->viewPdfButton, SIGNAL(clicked()),
@@ -1276,6 +1327,47 @@ void DlgSendMessage::fillContentFromTemplate(const QList<MsgId> &msgIds)
 	}
 }
 
+/*!
+ * @brief Select account from the combo box.
+ *
+ * @param[in,out] fromComboBox Combo box containing the accounts.
+ * @param[in] sender Sender identifier. It may contain the data-box identifier or the username.
+ * @param[in] parent Parent widget for the error message box.
+ */
+static
+void selectAccount(QComboBox &fromComboBox, const QString &sender, QWidget *parent)
+{
+	int idx = 0;
+
+	for (; idx < fromComboBox.count(); ++idx) {
+		QVariant itemData = fromComboBox.itemData(idx);
+		if (Q_UNLIKELY(!itemData.canConvert<AcntId>())) {
+			continue;
+		}
+		AcntId acntId = qvariant_cast<AcntId>(itemData);
+
+		const QString acntDbKey(AccountDb::keyFromLogin(acntId.username()));
+		const Isds::DbOwnerInfoExt2 dbOwnerInfo =
+		    GlobInstcs::accntDbPtr->getOwnerInfo2(acntDbKey);
+
+		if ((dbOwnerInfo.dbID() == sender) || (acntId.username() == sender)) {
+			break;
+		}
+	}
+
+	/* Set select matching item. */
+	if (idx < fromComboBox.count()) {
+		fromComboBox.setCurrentIndex(idx);
+	} else {
+		QMessageBox::warning(parent, DlgSendMessage::tr("Non-Existent Sender"),
+		    DlgSendMessage::tr(
+		        "No data box matches the sender identifier '%1'."
+		        "\n\n"
+		        "Cannot select data box with supplied data-box ID or username.").arg(sender),
+		    QMessageBox::Ok, QMessageBox::Ok);
+	}
+}
+
 void DlgSendMessage::fillContentCompose(const QString &composeSerialised)
 {
 	debugFuncCall();
@@ -1287,6 +1379,9 @@ void DlgSendMessage::fillContentCompose(const QString &composeSerialised)
 		return;
 	}
 
+	if (!composeCmd.sender().isEmpty()) {
+		selectAccount(*(m_ui->fromComboBox), composeCmd.sender(), this);
+	}
 	if (!composeCmd.dbIDRecipient().isEmpty()) {
 		addRecipientBoxes(composeCmd.dbIDRecipient());
 	}
@@ -1906,7 +2001,9 @@ bool DlgSendMessage::startUploadAttachments(void)
 
 		Isds::DmExtFile dmExtFile;
 		const QString transactionId = QString("%1_%2_%3_%4")
-		    .arg(m_acntId.username()).arg(timeStr).arg(row)
+		    .arg(m_acntId.username())
+		    .arg(timeStr)
+		    .arg(row)
 		    .arg(Utility::generateRandomString(6));
 
 		/* Remember order and parts of data for envelope. */
@@ -2196,32 +2293,6 @@ finish:
 	this->close();
 }
 
-void DlgSendMessage::setIcons(void)
-{
-	{
-		const QIcon ico(
-		    IconContainer::construcIcon(IconContainer::ICON_PLUS));
-		m_ui->addRecipButton->setIcon(ico);
-		m_ui->addAttachButton->setIcon(ico);
-	}
-
-	{
-		const QIcon ico(
-		    IconContainer::construcIcon(IconContainer::ICON_DELETE));
-		m_ui->removeRecipButton->setIcon(ico);
-		m_ui->removeAttachButton->setIcon(ico);
-	}
-
-	m_ui->findRecipButton->setIcon(
-	    IconContainer::construcIcon(IconContainer::ICON_SEARCH));
-
-	m_ui->enterBoxIdButton->setIcon(
-	    IconContainer::construcIcon(IconContainer::ICON_PENCIL));
-
-	m_ui->openAttachButton->setIcon(
-	    IconContainer::construcIcon(IconContainer::ICON_FOLDER));
-}
-
 void DlgSendMessage::setPDZInfos(const QList<Isds::PDZInfoRec> &pdzInfos)
 {
 	m_pdzInfos = pdzInfos;
@@ -2284,10 +2355,15 @@ void DlgSendMessage::setPDZInfos(const QList<Isds::PDZInfoRec> &pdzInfos)
 	m_ui->databoxInfo->setText(m_ui->databoxInfo->text() + pdzSending);
 }
 
+QString DlgSendMessage::createdPdfPath(void) const
+{
+	return QDir::fromNativeSeparators(
+	    m_tmpDirPath % QDir::separator() % SHORT_TEXT_MESSAGE_PDF_NAME);
+}
+
 QString DlgSendMessage::createPdf(void)
 {
-	QString fileName(
-	    m_tmpDirPath % QDir::separator() % SHORT_TEXT_MESSAGE_PDF_NAME);
+	QString fileName(createdPdfPath());
 
 	enum Utility::TextFormat format =
 	    int2TextFormat(m_ui->textFormatComboBox->currentData().toInt());
