@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2024 CZ.NIC
+ * Copyright (C) 2014-2025 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,13 +21,14 @@
  * the two.
  */
 
-#include <cstdlib> /* ::std::free */
 #include <QDateTime>
 #include <QSslCertificate>
 #include <QTimeZone>
 
 #include "src/crypto/crypto.h"
 #include "src/crypto/crypto_funcs.h"
+#include "src/crypto/crypto_wrapped.h"
+#include "src/datovka_shared/isds/types.h"
 #include "src/datovka_shared/log/log.h"
 #include "src/datovka_shared/utility/date_time.h"
 #include "src/dimensions/dimensions.h"
@@ -36,6 +37,7 @@
 #include "src/gui/icon_container.h"
 #include "src/io/message_db.h"
 #include "src/io/message_db_set.h"
+#include "src/isds/message_functions.h"
 #include "src/settings/prefs_specific.h"
 #include "ui_dlg_signature_detail.h"
 
@@ -46,8 +48,8 @@ DlgSignatureDetail::DlgSignatureDetail(const QByteArray &msgDER,
     QWidget *parent)
     : QDialog(parent),
     m_ui(new (::std::nothrow) Ui::DlgSignatureDetail),
+    m_isCMS(false),
     m_msgDER(msgDER),
-    m_tstDER(tstDER),
     m_constructedFromDb(constructedFromDb),
     m_dbIsVerified(dbIsVerified),
     m_dlgSize(),
@@ -65,9 +67,37 @@ DlgSignatureDetail::DlgSignatureDetail(const QByteArray &msgDER,
 	connect(m_ui->showCertDetail, SIGNAL(stateChanged(int)),
 	    this, SLOT(showCertificateDetail(int)));
 
+	/* Store envelope timestamp. */
+	TstEntries timeStamps;
+	if (Q_UNLIKELY(!timeStamps.addTst(DER_XML_TST, tstDER))) {
+		logErrorNL("%s", "Cannot store envelope timestamp.");
+	}
+
+	{
+		const enum Isds::Type::RawType rawType = Isds::guessDataRawType(msgDER);
+		switch (rawType) {
+		case Isds::Type::RT_CMS_SIGNED_INCOMING_MESSAGE:
+		case Isds::Type::RT_CMS_SIGNED_OUTGOING_MESSAGE:
+		case Isds::Type::RT_CMS_SIGNED_DELIVERYINFO:
+			m_isCMS = true;
+			break;
+		default:
+			break;
+		}
+	}
+	/* Read timestamps from CMS container. */
+	if (m_isCMS) {
+		if (Q_UNLIKELY(!timeStamps.addCmsTsts(msgDER))) {
+			logErrorNL("%s", "Cannot read timestamps from CMS.");
+		}
+	}
+
+	/* Read timestamp times and sort archive timestamps. */
+	timeStamps.sortTimestamps();
+
 	validateMessageSignature();
 	validateSigningCertificate();
-	validateMessageTimestamp();
+	validateMessageTimestamp(timeStamps);
 
 	/* Remember the size of the dialogue. */
 	m_dlgSize = this->sizeHint();
@@ -134,73 +164,6 @@ void DlgSignatureDetail::detail(const QByteArray &msgDER,
 	dlg.exec();
 }
 
-/*!
- * @brief Returns signing certificate inception and expiration date.
- *
- * @param[in]  DER Raw message or time stamp data.
- * @param[out] incTime Inception time.
- * @param[out] expTime Expiration time.
- * @return True on success.
- */
-static
-bool signingCertTimes(const QByteArray &DER, QDateTime &incTime,
-    QDateTime &expTime)
-{
-	struct x509_crt *x509_crt = NULL;
-	int64_t incept, expir;
-
-	debugFuncCall();
-
-	if (DER.isEmpty()) {
-		return false;
-	}
-
-	x509_crt = raw_cms_signing_cert(DER.data(), DER.size());
-	if (NULL == x509_crt) {
-		return false;
-	}
-
-	if (0 != x509_crt_date_info(x509_crt, &incept, &expir)) {
-		x509_crt_destroy(x509_crt); x509_crt = NULL;
-		return false;
-	}
-
-	x509_crt_destroy(x509_crt); x509_crt = NULL;
-
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 8, 0))
-	incTime = QDateTime::fromSecsSinceEpoch(incept);
-	expTime = QDateTime::fromSecsSinceEpoch(expir);
-#else /* < Qt-5.8 */
-	incTime = QDateTime::fromMSecsSinceEpoch(incept * 1000);
-	expTime = QDateTime::fromMSecsSinceEpoch(expir * 1000);
-#endif /* >= Qt-5.8 */
-
-	return true;
-}
-
-bool DlgSignatureDetail::signingCertExpiresBefore(const QByteArray &DER,
-    int days, const QDateTime &dDate)
-{
-	QDateTime expir;
-	QDateTime now;
-	{
-		QDateTime incep;
-		if (!signingCertTimes(DER, incep, expir)) {
-			return false;
-		}
-	}
-
-	if (dDate.isValid()) {
-		now = dDate;
-	} else {
-		now = QDateTime::currentDateTime();
-	}
-
-	int difference = now.daysTo(expir);
-
-	return difference < days;
-}
-
 void DlgSignatureDetail::showCertificateDetail(int checkState)
 {
 	m_ui->certDetailWidget->setHidden(Qt::Unchecked == checkState);
@@ -228,8 +191,8 @@ void DlgSignatureDetail::validateMessageSignature(void)
 	QIcon ico;
 	QString resStr;
 
-	if (m_msgDER.isEmpty()) {
-		ico = IconContainer::construcIcon(IconContainer::ICON_WARNING);
+	if (Q_UNLIKELY(m_msgDER.isEmpty() || (!m_isCMS))) {
+		ico = IconContainer::construcIcon(IconContainer::ICON_APP_WARNING);
 		resStr = tr("Message signature is not present.");
 	} else {
 		bool verified = false;
@@ -244,12 +207,12 @@ void DlgSignatureDetail::validateMessageSignature(void)
 
 		if (!verified) {
 			ico = IconContainer::construcIcon(
-			    IconContainer::ICON_DATOVKA_ERROR);
+			    IconContainer::ICON_APP_ERROR);
 			resStr = "<b>" + tr("Valid") + ": </b>";
 			resStr += NO;
 		} else {
 			ico = IconContainer::construcIcon(
-			    IconContainer::ICON_DATOVKA_OK);
+			    IconContainer::ICON_APP_OK);
 			resStr = "<b>" + tr("Valid") + ": </b>";
 			resStr += YES;
 		}
@@ -260,95 +223,21 @@ void DlgSignatureDetail::validateMessageSignature(void)
 	m_ui->mSignatureStatus->setText(resStr);
 }
 
-/*!
- * @brief Returns signing certificate of message.
- *
- * @param[in]  DER Raw message or time stamp data.
- * @param[out] saId Signature algorithm identifier.
- * @param[out] saName Signature algorithm name.
- * @return Null certificate on failure.
- */
-static
-QSslCertificate signingCert(const QByteArray &DER,
-    QString &saId, QString &saName)
-{
-	struct x509_crt *x509_crt = NULL;
-	void *der = NULL;
-	size_t der_size;
-	char *sa_id = NULL, *sa_name = NULL;
-
-	debugFuncCall();
-
-	if (DER.isEmpty()) {
-		return QSslCertificate();
-	}
-
-	x509_crt = raw_cms_signing_cert(DER.data(), DER.size());
-	if (NULL == x509_crt) {
-		return QSslCertificate();
-	}
-
-	if (0 != x509_crt_to_der(x509_crt, &der, &der_size)) {
-		x509_crt_destroy(x509_crt); x509_crt = NULL;
-		return QSslCertificate();
-	}
-
-	if (0 != x509_crt_algorithm_info(x509_crt, &sa_id, &sa_name)) {
-		x509_crt_destroy(x509_crt); x509_crt = NULL;
-		return QSslCertificate();
-	}
-
-	x509_crt_destroy(x509_crt); x509_crt = NULL;
-
-	saId = sa_id;
-	saName = sa_name;
-
-	::std::free(sa_id); sa_id = NULL;
-	::std::free(sa_name); sa_name = NULL;
-
-	QByteArray certRawBytes((char *) der, der_size);
-	::std::free(der); der = NULL;
-
-	return QSslCertificate(certRawBytes, QSsl::Der);
-}
-
-/*!
- * @brief Return whether signing certificate is valid.
- *
- * @param[in] DER Raw message or time stamp data.
- * @return True is signing certificate was verified successfully.
- */
-static
-bool signingCertValid(const QByteArray &DER, struct crt_verif_outcome &cvo)
-{
-	struct x509_crt *signing_cert = NULL;
-	int ret;
-
-	debugFuncCall();
-
-	if (DER.isEmpty()) {
-		return false;
-	}
-
-	signing_cert = raw_cms_signing_cert(DER.data(), DER.size());
-	if (NULL == signing_cert) {
-		return false;
-	}
-
-	ret = x509_crt_verify(signing_cert, NULL);
-
-	x509_crt_track_verification(signing_cert, &cvo);
-
-	x509_crt_destroy(signing_cert); signing_cert = NULL;
-	return 1 == ret;
-}
-
 void DlgSignatureDetail::validateSigningCertificate(void)
 {
 	m_ui->showCertDetail->setHidden(false);
 	m_ui->showVerifyDetail->setHidden(false);
 
-	if (m_msgDER.isEmpty()) {
+	QIcon ico;
+	QString resStr;
+
+	if (Q_UNLIKELY(m_msgDER.isEmpty() || (!m_isCMS))) {
+		ico = IconContainer::construcIcon(IconContainer::ICON_APP_WARNING);
+		resStr = tr("Message signature is not present.");
+
+		m_ui->cImage->setIcon(ico);
+		m_ui->cStatus->setText(resStr);
+
 		m_ui->showCertDetail->setHidden(true);
 		m_ui->showVerifyDetail->setHidden(true);
 
@@ -357,16 +246,14 @@ void DlgSignatureDetail::validateSigningCertificate(void)
 
 	struct crt_verif_outcome cvo;
 
-	QString resStr("<b>" + tr("Valid") + ": </b>");
-
-	QIcon ico;
+	resStr = ("<b>" + tr("Valid") + ": </b>");
 	if (!signingCertValid(m_msgDER, cvo)) {
 		ico = IconContainer::construcIcon(
-		    IconContainer::ICON_DATOVKA_ERROR);
+		    IconContainer::ICON_APP_ERROR);
 		resStr += NO;
 	} else {
 		ico = IconContainer::construcIcon(
-		    IconContainer::ICON_DATOVKA_OK);
+		    IconContainer::ICON_APP_OK);
 		resStr += YES;
 	}
 
@@ -512,152 +399,261 @@ void DlgSignatureDetail::validateSigningCertificate(void)
 }
 
 /*!
- * @brief Signing certificate issuer information.
+ * @brief Read timestamp information and generate data to be displayed.
  *
- * @param[in]  DER Raw message or time stamp data.
- * @param[out] oStr Organisation name.
- * @param[out] ouStr Organisation unit name.
- * @param[out] nStr Common name.
- * @param[out] cStr Country name.
- * @return False on failure.
+ * @param[in] utcTstTime Timestamping time.
+ * @param[in] tstValid Whether timstamp has been successfully validated.
+ * @param[in] tstDER Raw time-stamp data.
+ * @param[out] tImage Verification pictogram.
+ * @param[out] tStatus Verification result.
+ * @param[out] tDetail Verification detail.
  */
 static
-bool signingCertIssuerInfo(const QByteArray &DER, QString &oStr, QString &ouStr,
-    QString &nStr, QString &cStr)
-{
-	struct x509_crt *signing_cert = NULL;
-	struct crt_issuer_info cii;
-
-	debugFuncCall();
-
-	crt_issuer_info_init(&cii);
-
-	if (DER.isEmpty()) {
-		return false;
-	}
-
-	signing_cert = raw_cms_signing_cert(DER.data(), DER.size());
-	if (NULL == signing_cert) {
-		goto fail;
-	}
-
-	if (0 != x509_crt_issuer_info(signing_cert, &cii)) {
-		goto fail;
-	}
-
-	x509_crt_destroy(signing_cert); signing_cert = NULL;
-
-	if (NULL != cii.o) {
-		oStr = cii.o;
-	}
-
-	if (NULL != cii.ou) {
-		ouStr = cii.ou;
-	}
-
-	if (NULL != cii.n) {
-		nStr = cii.n;
-	}
-
-	if (NULL != cii.c) {
-		cStr = cii.c;
-	}
-
-	crt_issuer_info_clear(&cii);
-
-	return true;
-
-fail:
-	if (NULL != signing_cert) {
-		x509_crt_destroy(signing_cert);
-	}
-	crt_issuer_info_clear(&cii);
-	return false;
-}
-
-void DlgSignatureDetail::validateMessageTimestamp(void)
+void timestampOverview(qint64 utcTstTime, bool tstValid, const QByteArray &tstDER,
+    QPushButton &tImage, QLabel &tStatus, QLabel &tDetail)
 {
 	QIcon ico;
 	QString resStr;
 	QString detailStr;
-	QDateTime tst;
 
-	if (m_tstDER.isEmpty()) {
-		ico = IconContainer::construcIcon(IconContainer::ICON_WARNING);
-		resStr = tr("Time stamp not present.");
+	if (Q_UNLIKELY(tstDER.isEmpty())) {
+		ico = IconContainer::construcIcon(IconContainer::ICON_APP_WARNING);
+		resStr = DlgSignatureDetail::tr("Time stamp not present.");
 	} else {
-		int64_t utc_time = 0;
-		int ret = raw_tst_verify(m_tstDER.data(), m_tstDER.size(),
-		    &utc_time);
+		QDateTime tstTime;
 
-		if (-1 != ret) {
+		if (utcTstTime >= 0) {
 #if (QT_VERSION >= QT_VERSION_CHECK(5, 8, 0))
-			tst = QDateTime::fromSecsSinceEpoch(utc_time);
+			tstTime = QDateTime::fromSecsSinceEpoch(utcTstTime);
 #else /* < Qt-5.8 */
-			tst = QDateTime::fromMSecsSinceEpoch(utc_time * 1000);
+			tstTime = QDateTime::fromMSecsSinceEpoch(utcTstTime * 1000);
 #endif /* >= Qt-5.8 */
 		}
-		resStr = "<b>" + tr("Valid") + ": </b>";
-		if (1 != ret) {
+		resStr = "<b>" + DlgSignatureDetail::tr("Valid") + ": </b>";
+		if (!tstValid) {
 			ico = IconContainer::construcIcon(
-			    IconContainer::ICON_DATOVKA_ERROR);
+			    IconContainer::ICON_APP_ERROR);
 			resStr += NO;
 		} else {
 			ico = IconContainer::construcIcon(
-			    IconContainer::ICON_DATOVKA_OK);
+			    IconContainer::ICON_APP_OK);
 			resStr += YES;
 		}
 
-		detailStr = "<b>" + tr("Time") + ": </b> " +
-		    tst.toString(Utility::dateTimeDisplayFormat) + " " +
-		    tst.timeZone().abbreviation(tst) + "<br/>";
+		detailStr = "<b>" + DlgSignatureDetail::tr("Time") + ": </b> " +
+		    tstTime.toString(Utility::dateTimeDisplayFormat) + " " +
+		    tstTime.timeZone().abbreviation(tstTime) + "<br/>";
 
 		QString o, ou, n, c;
-		signingCertIssuerInfo(m_tstDER, o, ou, n, c);
+		signingCertIssuerInfo(tstDER, o, ou, n, c);
 
-		detailStr += "<b>" + tr("Issuer") + ": </b><br/>";
+		detailStr += "<b>" + DlgSignatureDetail::tr("Issuer") + ": </b><br/>";
 		if (!o.isEmpty()) {
-			detailStr += "&nbsp;&nbsp;" + tr("Organisation") +
+			detailStr += "&nbsp;&nbsp;" + DlgSignatureDetail::tr("Organisation") +
 			    ": " + o + "<br/>";
 		}
 		if (!ou.isEmpty()) {
 			detailStr += "&nbsp;&nbsp;" +
-			    tr("Organisational unit") + ": " + ou + "<br/>";
+			    DlgSignatureDetail::tr("Organisational unit") + ": " + ou + "<br/>";
 		}
 		if (!n.isEmpty()) {
-			detailStr += "&nbsp;&nbsp;" + tr("Name") + ": " +
+			detailStr += "&nbsp;&nbsp;" + DlgSignatureDetail::tr("Name") + ": " +
 			    n + "<br/>";
 		}
 		if (!c.isEmpty()) {
-			detailStr += "&nbsp;&nbsp;" + tr("Country") + ": " +
+			detailStr += "&nbsp;&nbsp;" + DlgSignatureDetail::tr("Country") + ": " +
 			    c + "<br/>";
 		}
 
 		QDateTime incept, expir;
-		if (signingCertTimes(m_tstDER, incept, expir)) {
-			detailStr += "<b>" + tr("Validity") + ": </b><br/>";
-			detailStr += "&nbsp;&nbsp;" + tr("Valid from") + ": " +
+		if (signingCertTimes(tstDER, incept, expir)) {
+			detailStr += "<b>" + DlgSignatureDetail::tr("Validity") + ": </b><br/>";
+			detailStr += "&nbsp;&nbsp;" + DlgSignatureDetail::tr("Valid from") + ": " +
 			    incept.toString(Utility::dateTimeDisplayFormat) + " " +
 			    incept.timeZone().abbreviation(incept) + "<br/>";
-			detailStr += "&nbsp;&nbsp;" + tr("Valid to") + ": " +
+			detailStr += "&nbsp;&nbsp;" + DlgSignatureDetail::tr("Valid to") + ": " +
 			    expir.toString(Utility::dateTimeDisplayFormat) + " " +
 			    expir.timeZone().abbreviation(expir) + "<br/>";
 		}
 
-		m_ui->tDetail->setAlignment(Qt::AlignLeft);
-		m_ui->tDetail->setTextFormat(Qt::RichText);
-		m_ui->tDetail->setText(detailStr);
 	}
 
-	m_ui->tImage->setIcon(ico);
-	m_ui->tStatus->setTextFormat(Qt::RichText);
-	m_ui->tStatus->setText(resStr);
+	tImage.setIcon(ico);
+	tStatus.setTextFormat(Qt::RichText);
+	tStatus.setText(resStr);
+
+	tDetail.setAlignment(Qt::AlignLeft);
+	tDetail.setTextFormat(Qt::RichText);
+	tDetail.setText(detailStr);
+}
+
+/*!
+ * @brief Generate tab timestamp content.
+ *
+ * @param[in] utcTstTime Timestamping time.
+ * @param[in] tstValid Whether timstamp has been successfully validated.
+ * @param[in] entry Timestamp entry.
+ * @param[in] idx Tab counter - user to create object names.
+ * @return Pointer to tab content.
+ */
+static
+QWidget *newTab(qint64 utcTstTime, bool tstValid, const struct der_data_list_entry *entry, int idx)
+{
+	if (Q_UNLIKELY(NULL == entry)) {
+		return Q_NULLPTR;
+	}
+
+	QFont font1;
+	font1.setBold(false);
+
+	QWidget *tab = new (::std::nothrow) QWidget();
+	if (Q_UNLIKELY(Q_NULLPTR == tab)) {
+		return NULL;
+	}
+
+	tab->setObjectName(QString("tab_%1_envelope").arg(idx));
+	QVBoxLayout *verticalLayout_00 = new (::std::nothrow) QVBoxLayout(tab);
+	verticalLayout_00->setObjectName(QString("tab_%1_verticalLayout_00").arg(idx));
+	QHBoxLayout *horizontalLayout_00 = new (::std::nothrow) QHBoxLayout();
+	horizontalLayout_00->setObjectName(QString("tab_%1_horizontalLayout_00").arg(idx));
+	QLabel *tStatus = new (::std::nothrow) QLabel(tab);
+	tStatus->setObjectName(QString("tab_%1_tStatus").arg(idx));
+	tStatus->setFont(font1);
+
+	horizontalLayout_00->addWidget(tStatus);
+
+	QSpacerItem *tHorizontalSpacer = new (::std::nothrow) QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);
+
+	horizontalLayout_00->addItem(tHorizontalSpacer);
+
+	QPushButton *tImage = new (::std::nothrow) QPushButton(tab);
+	tImage->setObjectName(QString("tab_%1_tImage").arg(idx));
+
+	horizontalLayout_00->addWidget(tImage);
+
+	verticalLayout_00->addLayout(horizontalLayout_00);
+
+	QHBoxLayout *horizontalLayout_01 = new (::std::nothrow) QHBoxLayout();
+	horizontalLayout_01->setObjectName(QString("tab_%1_horizontalLayout_01").arg(idx));
+	QLabel *label_00 = new (::std::nothrow) QLabel(tab);
+	label_00->setObjectName(QString("tab_%1_label_00").arg(idx));
+	label_00->setMinimumSize(QSize(15, 0));
+	label_00->setFont(font1);
+
+	horizontalLayout_01->addWidget(label_00);
+
+	QLabel *tDetail = new (::std::nothrow) QLabel(tab);
+	tDetail->setObjectName(QString("tab_%1_tDetail").arg(idx));
+	tDetail->setFont(font1);
+
+	horizontalLayout_01->addWidget(tDetail);
+
+	QSpacerItem *horizontalSpacer_00 = new (::std::nothrow) QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);
+
+	horizontalLayout_01->addItem(horizontalSpacer_00);
+
+	verticalLayout_00->addLayout(horizontalLayout_01);
+
+	timestampOverview(utcTstTime, tstValid,
+	    (NULL != entry)
+	        ? QByteArray::fromRawData((const char *)entry->der.data, entry->der.size)
+	        : QByteArray(),
+	    *tImage, *tStatus, *tDetail);
+
+	return tab;
+}
+
+/*!
+ * @brief Generate time string from UTC time.
+ *
+ * @param[in] utc_time UTC time.
+ * @return Time string.
+ */
+static
+QString utcToDisplay(qint64 utc_time)
+{
+	QDateTime tstTime;
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 8, 0))
+	tstTime = QDateTime::fromSecsSinceEpoch(utc_time);
+#else /* < Qt-5.8 */
+	tstTime = QDateTime::fromMSecsSinceEpoch(utc_time * 1000);
+#endif /* >= Qt-5.8 */
+	return tstTime.toString(Utility::dateTimeDisplayFormat) + " " +
+	    tstTime.timeZone().abbreviation(tstTime);
+}
+
+void DlgSignatureDetail::validateMessageTimestamp(const TstEntries &timeStamps)
+{
+	/* Always show envelope timestamp tab. */
+	{
+		const qint64 tst_utc_time = timeStamps.envelopeTstTime();
+		const bool tst_valid = timeStamps.envelopeTstValid();
+		const struct der_data_list_entry *tst_entry = timeStamps.envelopeTst();
+
+		timestampOverview(tst_utc_time, tst_valid,
+		    (NULL != tst_entry)
+		        ? QByteArray::fromRawData((const char *)tst_entry->der.data, tst_entry->der.size)
+		        : QByteArray(),
+		    *m_ui->tImage, *m_ui->tStatus, *m_ui->tDetail);
+		m_ui->tabWidget->setTabToolTip(0, /* Fist tab. */
+		    tr("%1\nTime stamp stored in the envelope.")
+		        .arg((tst_utc_time >= 0) ? utcToDisplay(tst_utc_time) : QString()));
+	}
+
+	int num_tabs = 1;
+	{
+		const qint64 tst_utc_time = timeStamps.signatureTstTime();
+		const bool tst_valid = timeStamps.signatureTstValid();
+		const struct der_data_list_entry *tst_entry = timeStamps.signatureTst();
+
+		if (NULL != tst_entry) {
+			QWidget *tab = newTab(tst_utc_time, tst_valid, tst_entry, num_tabs);
+			if (Q_NULLPTR != tab) {
+				m_ui->tabWidget->addTab(tab, QString());
+				int tabIdx = m_ui->tabWidget->indexOf(tab);
+				m_ui->tabWidget->setTabText(tabIdx,
+				    tr("Signature"));
+				m_ui->tabWidget->setTabToolTip(tabIdx,
+				    tr("%1\nSignature time stamp stored in the CMS container.")
+				        .arg((tst_utc_time >= 0) ? utcToDisplay(tst_utc_time) : QString()));
+
+				++num_tabs;
+			}
+		}
+	}
+
+	QList<qint64> keys = timeStamps.sortedArchiveTstTimes();
+	int archive_idx = 0;
+	for (const qint64 tst_utc_time : keys) {
+		const bool tst_valid = timeStamps.archiveTstValid(tst_utc_time);
+		const struct der_data_list_entry *tst_entry = timeStamps.archiveTst(tst_utc_time);
+
+		if (NULL != tst_entry) {
+			QWidget *tab = newTab(tst_utc_time, tst_valid, tst_entry, num_tabs);
+			if (Q_NULLPTR != tab) {
+				m_ui->tabWidget->addTab(tab, QString());
+				int tabIdx = m_ui->tabWidget->indexOf(tab);
+				m_ui->tabWidget->setTabText(tabIdx,
+				    tr("Archive %1").arg(archive_idx + 1));
+				m_ui->tabWidget->setTabToolTip(tabIdx,
+				    tr("%1\nArchive time stamp number %2 in the CMS container.")
+				        .arg((tst_utc_time >= 0) ? utcToDisplay(tst_utc_time) : QString())
+				        .arg(archive_idx + 1));
+
+				++num_tabs;
+				++archive_idx;
+			}
+		}
+	}
+
+	/* Select last tab. */
+	m_ui->tabWidget->setCurrentIndex(num_tabs - 1);
 }
 
 void DlgSignatureDetail::setIcons(void)
 {
 	const QIcon ico(
-	    IconContainer::construcIcon(IconContainer::ICON_WARNING));
+	    IconContainer::construcIcon(IconContainer::ICON_APP_WARNING));
 	m_ui->tImage->setIcon(ico);
 	m_ui->cImage->setIcon(ico);
 	m_ui->mSignatureImage->setIcon(ico);
