@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2024 CZ.NIC
+ * Copyright (C) 2014-2025 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@
 #include <QIntValidator>
 #include <QUrl>
 
+#include "src/datovka_shared/compat/compiler.h" /* macroStdMove */
 #include "src/datovka_shared/log/log.h"
 #include "src/dimensions/dimensions.h"
 #include "src/global.h"
@@ -118,57 +119,63 @@ bool DlgTagSettings::editTagContSettings(TagContainerSettings &tagContSet,
 		bool changedConnection = false;
 		bool changedProfile = false;
 
+		/*
+		 * Collect new settings and store them at once to minimise
+		 * the number of emitted signals.
+		 */
+		TagContainerSettings newTagContSet = tagContSet;
+
 		/* Backend. */
 		{
 			const enum TagContainer::Backend newBackend = selectedBackend(
 			    dlg.m_ui->localTagDbRadioButton, dlg.m_ui->tagClientRadioButton);
-			if (tagContSet.backend() != newBackend) {
+			if (newTagContSet.backend() != newBackend) {
 				changedBackend = true;
-				tagContSet.setBackend(newBackend);
+				newTagContSet.setBackend(newBackend);
 			}
 		}
 
 		/* Connection. */
 		{
 			const QString hostname = dlg.m_ui->tagSrvrNameLineEdit->text();
-			if (tagContSet.clientHostname() != hostname) {
+			if (newTagContSet.clientHostname() != hostname) {
 				changedConnection = true;
-				tagContSet.setClientHostname(hostname);
+				newTagContSet.setClientHostname(hostname);
 			}
 		}
 		{
 			bool iOk = false;
 			const int port = dlg.m_ui->tagSrvrPortLineEdit->text().toInt(&iOk);
 			if (iOk) {
-				if (tagContSet.clientPort() != port) {
+				if (newTagContSet.clientPort() != port) {
 					changedConnection = true;
-					tagContSet.setClientPort(port);
+					newTagContSet.setClientPort(port);
 				}
 			} else {
 				/* TODO -- Handle bad input. */
-				tagContSet.setClientPort(-1);
+				newTagContSet.setClientPort(-1);
 			}
 		}
 		{
 			const QString certPath = dlg.m_ui->tagSrvrCertLineEdit->text();
-			if (tagContSet.certPath() != certPath) {
+			if (newTagContSet.certPath() != certPath) {
 				changedConnection = true;
-				tagContSet.setCertPath(certPath);
+				newTagContSet.setCertPath(certPath);
 			}
 		}
 		{
 			const QString username = dlg.m_ui->usernameLineEdit->text();
-			if (tagContSet.username() != username) {
+			if (newTagContSet.username() != username) {
 				changedConnection = true;
-				tagContSet.setUsername(username);
+				newTagContSet.setUsername(username);
 			}
 		}
 		{
 			const QString pwd = dlg.m_ui->pwdLineEdit->text();
-			if (tagContSet.password() != pwd) {
+			if (newTagContSet.password() != pwd) {
 				changedConnection = true;
-				tagContSet.setPassword(pwd);
-				tagContSet.setPwdCode(QByteArray());
+				newTagContSet.setPassword(pwd);
+				newTagContSet.setPwdCode(QByteArray());
 			}
 		}
 
@@ -183,21 +190,24 @@ bool DlgTagSettings::editTagContSettings(TagContainerSettings &tagContSet,
 				const QString profileName = dlg.m_profileModel.data(
 				    index).toString();
 
-				if (tagContSet.profileId() != profileId) {
+				if (newTagContSet.profileId() != profileId) {
 					changedProfile = true;
-					tagContSet.setProfileId(profileId);
+					newTagContSet.setProfileId(profileId);
 				}
-				if (tagContSet.profileName() != profileName) {
+				if (newTagContSet.profileName() != profileName) {
 					/* Just store the new name. */
-					tagContSet.setProfileName(profileName);
+					newTagContSet.setProfileName(profileName);
 				}
 			} else {
 				/* TODO -- No profiles available. */
-				tagContSet.setProfileId(-1);
-				tagContSet.setProfileName(QString());
+				newTagContSet.setProfileId(-1);
+				newTagContSet.setProfileName(QString());
 			}
 		}
 
+		/* This emits a signal that the content has changed. */
+		tagContSet = macroStdMove(newTagContSet);
+
 		if (changedBackend || changedConnection) {
 			applyCompleteTagContainerSettings(tagContSet, tagCont);
 		} else if (changedProfile) {
@@ -580,7 +590,7 @@ bool sendFullMatchToTgt(const QHash<Json::TagEntry, qint64> &fullSrcToTgtMatch,
 		/* Construct new assignment. */
 		for (const Json::TagEntry &se : fullSrcToTgtMatch.keys()) {
 			qint64 tgtTagId = fullSrcToTgtMatch.value(se);
-			for (const Json::TagMsgId &msgId : localTagIdToMsgIds.value(se.id())) {
+			for (const Json::MsgId1 &msgId : localTagIdToMsgIds.value(se.id())) {
 				tagAssignmentCommand[msgId].append(tgtTagId);
 			}
 		}
@@ -827,8 +837,6 @@ void DlgTagSettings::watchServerTagsDeleted(const Json::Int64StringList &ids)
 
 void DlgTagSettings::initDialogue(const TagContainerSettings &tagContSet)
 {
-	setIcons();
-
 	/* Server connection tab. */
 	m_ui->tagSrvrPortLineEdit->setValidator(
 	    new (::std::nothrow) QIntValidator(0, 65535, m_ui->tagSrvrPortLineEdit));
@@ -953,15 +961,3 @@ void DlgTagSettings::initDialogue(const TagContainerSettings &tagContSet)
 
 	m_ui->updateServerTagButton->setEnabled(false);
 }
-
-void DlgTagSettings::setIcons(void)
-{
-	m_ui->updateLocalTagButton->setIcon(
-	    IconContainer::construcIcon(IconContainer::ICON_PENCIL));
-
-	m_ui->toServerButton->setIcon(
-	    IconContainer::construcIcon(IconContainer::ICON_RIGHT));
-
-	m_ui->updateServerTagButton->setIcon(
-	    IconContainer::construcIcon(IconContainer::ICON_PENCIL));
-}
