@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2024 CZ.NIC
+ * Copyright (C) 2014-2025 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,9 +31,10 @@
 #include "src/gui/dlg_tag.h"
 #include "src/gui/dlg_tags.h"
 #include "src/gui/icon_container.h"
+#include "src/gui/styles.h"
 #include "src/io/tag_container.h"
+#include "src/json/message_id.h"
 #include "src/json/tag_assignment_hash.h"
-#include "src/json/tag_message_id.h"
 #include "src/settings/prefs_specific.h"
 #include "ui_dlg_tags.h"
 
@@ -58,7 +59,7 @@ DlgTags::DlgTags(const AcntId &acntId, TagContainer *tagCont,
 	m_ui->setupUi(this);
 	/* Tab order is defined in UI file. */
 
-	initDlg(m_msgIdList);
+	initDlg(!m_msgIdList.isEmpty());
 	loadModels();
 	//selectAllAssingedTagsFromMsgs();
 }
@@ -68,15 +69,15 @@ DlgTags::~DlgTags(void)
 	delete m_ui;
 }
 
-enum DlgTags::ReturnCode DlgTags::editAvailable(const AcntId &acntId,
-    TagContainer *tagCont, QWidget *parent)
+enum DlgTags::ReturnCode DlgTags::editAvailable(TagContainer *tagCont,
+    QWidget *parent)
 {
 	if (Q_UNLIKELY(tagCont == Q_NULLPTR)) {
 		Q_ASSERT(0);
 		return NO_ACTION;
 	}
 
-	DlgTags dlg(acntId, tagCont, QList<qint64>(), parent);
+	DlgTags dlg(AcntId(), tagCont, QList<qint64>(), parent);
 
 	const QSize dfltSize = dlg.size();
 	{
@@ -274,7 +275,7 @@ void DlgTags::assignSelectedTagsToMsgs(void)
 
 	{
 		Json::TagAssignmentCommand tagAssignmentCommand;
-		for (const Json::TagMsgId &msgId : m_msgTagItemsMap.keys()) {
+		for (const Json::MsgId1 &msgId : m_msgTagItemsMap.keys()) {
 			tagAssignmentCommand[msgId] = tagIds;
 		}
 		m_tagContPtr->assignTagsToMsgs(tagAssignmentCommand);
@@ -306,7 +307,7 @@ void DlgTags::removeSelectedTagsFromMsgs(void)
 
 	{
 		Json::TagAssignmentCommand tagAssignmentCommand;
-		for (const Json::TagMsgId &msgId : m_msgTagItemsMap.keys()) {
+		for (const Json::MsgId1 &msgId : m_msgTagItemsMap.keys()) {
 			tagAssignmentCommand[msgId] = tagIds;
 		}
 		m_tagContPtr->removeTagsFromMsgs(tagAssignmentCommand);
@@ -433,14 +434,14 @@ QList<TagItem> itemSetToList(const QSet<TagItem> &set)
  * @param[in] tagsIntersection Intersection of all tags.
  */
 static
-void computeTagSets(const QHash<Json::TagMsgId, TagItemList> &msgTagItemsMap,
+void computeTagSets(const QHash<Json::MsgId1, TagItemList> &msgTagItemsMap,
     QSet<TagItem> &tagsUnion, QSet<TagItem> &tagsIntersection)
 {
 	tagsUnion.clear();
 	tagsIntersection.clear();
 
 	bool firstMsg = true;
-	QHash<Json::TagMsgId, TagItemList>::const_iterator it;
+	QHash<Json::MsgId1, TagItemList>::const_iterator it;
 	for (it = msgTagItemsMap.constBegin(); it != msgTagItemsMap.constEnd(); ++it) {
 		const QSet<TagItem> msgTagSet = itemListToSet(*it);
 
@@ -463,8 +464,8 @@ void DlgTags::watchTagAssignmentChanged(const Json::TagAssignmentList &assignmen
 			continue;
 		}
 
-		QHash<Json::TagMsgId, TagItemList>::iterator it =
-		    m_msgTagItemsMap.find(Json::TagMsgId(
+		QHash<Json::MsgId1, TagItemList>::iterator it =
+		    m_msgTagItemsMap.find(Json::MsgId1(
 		        assignment.testEnv(), assignment.dmId()));
 		if (it == m_msgTagItemsMap.end()) {
 			continue;
@@ -505,24 +506,35 @@ void DlgTags::watchTagContReset(void)
 	loadModels();
 }
 
-void DlgTags::fillTagsToListViews(const Json::TagMsgIdList &msgIdList)
+void DlgTags::loadAssignments(void)
 {
-	Q_ASSERT(m_acntId.isValid());
+	if (Q_UNLIKELY((!m_acntId.isValid()) || (m_msgIdList.isEmpty()))) {
+		/*
+		 * Can be called without account and empty message list.
+		 * Clearing assignments if done so.
+		 */
+		m_assignedTagsModel.setTagList(QList<TagItem>());
+		m_msgTagItemsMap.clear();
+		return;
+	}
 
 	/* This should also disable all related buttons. */
 	m_ui->availableTagsView->clearSelection();
 	m_ui->assignedTagsView->clearSelection();
 
-	/* Get all available tags. */
-	Json::TagEntryList availableTags;
-	m_tagContPtr->getAllTags(availableTags);
-	m_availableTagsModel.setTagList(availableTags);
+	Json::MsgId1List jsonMsgIdList;
+	for (qint64 msgId : m_msgIdList) {
+		jsonMsgIdList.append(Json::MsgId1(
+		    m_acntId.testing() ? Isds::Type::BOOL_TRUE : Isds::Type::BOOL_FALSE,
+		    msgId));
+	}
 
 	/* Fill map of tag item lists. */
+	m_msgTagItemsMap.clear();
 	{
 		Json::TagAssignmentHash assignments;
-		if (m_tagContPtr->getMessageTags(msgIdList, assignments)) {
-			for (const Json::TagMsgId &msgId : assignments.keys()) {
+		if (m_tagContPtr->getMessageTags(jsonMsgIdList, assignments)) {
+			for (const Json::MsgId1 &msgId : assignments.keys()) {
 				m_msgTagItemsMap[msgId] = assignments[msgId];
 			}
 		}
@@ -540,7 +552,7 @@ void DlgTags::fillTagsToListViews(const Json::TagMsgIdList &msgIdList)
 	}
 }
 
-void DlgTags::initDlg(const QList<qint64> &msgIdList)
+void DlgTags::initDlg(bool enableTagAssignment)
 {
 	setIcons();
 
@@ -571,7 +583,7 @@ void DlgTags::initDlg(const QList<qint64> &msgIdList)
 	databaseConnectActions();
 
 	/* Message tag assignment is going to be edited. */
-	if (!msgIdList.isEmpty()) {
+	if (enableTagAssignment) {
 		connect(m_ui->assignButton, SIGNAL(clicked()), this,
 		    SLOT(assignSelectedTagsToMsgs()));
 		connect(m_ui->removeButton, SIGNAL(clicked()), this,
@@ -584,23 +596,22 @@ void DlgTags::initDlg(const QList<qint64> &msgIdList)
 	m_ui->updateTagButton->setEnabled(false);
 
 	m_ui->assignButton->setEnabled(false);
-	m_ui->assignButton->setVisible(!msgIdList.isEmpty());
+	m_ui->assignButton->setVisible(enableTagAssignment);
 	m_ui->removeButton->setEnabled(false);
-	m_ui->removeButton->setVisible(!msgIdList.isEmpty());
+	m_ui->removeButton->setVisible(enableTagAssignment);
 
-	m_ui->assignedGroup->setEnabled(!msgIdList.isEmpty());
-	m_ui->assignedGroup->setVisible(!msgIdList.isEmpty());
+	m_ui->assignedGroup->setEnabled(enableTagAssignment);
+	m_ui->assignedGroup->setVisible(enableTagAssignment);
 }
 
 void DlgTags::loadModels(void)
 {
-	Json::TagMsgIdList jsonMsgIdList;
-	for (qint64 msgId : m_msgIdList) {
-		jsonMsgIdList.append(Json::TagMsgId(
-		    m_acntId.testing() ? Isds::Type::BOOL_TRUE : Isds::Type::BOOL_FALSE,
-		    msgId));
-	}
-	fillTagsToListViews(jsonMsgIdList);
+	/* Get all available tags. */
+	Json::TagEntryList availableTags;
+	m_tagContPtr->getAllTags(availableTags);
+	m_availableTagsModel.setTagList(availableTags);
+
+	loadAssignments();
 }
 
 void DlgTags::clearModels(void)
@@ -634,7 +645,7 @@ void DlgTags::selectAllAssingedTagsFromMsgs(void)
 	for (int i = 0; i < rows; ++i) {
 		const QModelIndex idx = m_ui->availableTagsView->model()->index(i, 0);
 		const qint64 id = getTagIdFromIndex(idx);
-		for (const Json::TagMsgId &msgId : m_msgTagItemsMap.keys()) {
+		for (const Json::MsgId1 &msgId : m_msgTagItemsMap.keys()) {
 			const TagItemList tags = m_msgTagItemsMap[msgId];
 			for (const TagItem &tag : tags) {
 				if (tag.id() == id) {
@@ -648,21 +659,13 @@ void DlgTags::selectAllAssingedTagsFromMsgs(void)
 
 void DlgTags::setIcons(void)
 {
-	m_ui->addTagButton->setIcon(
-	    IconContainer::construcIcon(IconContainer::ICON_PLUS));
-
-	m_ui->deleteTagButton->setIcon(
-	    IconContainer::construcIcon(IconContainer::ICON_DELETE));
-
-	m_ui->updateTagButton->setIcon(
-	    IconContainer::construcIcon(IconContainer::ICON_PENCIL));
-
-	m_ui->assignButton->setIcon(
-	    IconContainer::construcIcon(IconContainer::ICON_RIGHT));
-
-	m_ui->removeButton->setIcon(
-	    IconContainer::construcIcon(IconContainer::ICON_LEFT));
-
-	m_ui->removeAllButton->setIcon(
-	    IconContainer::construcIcon(IconContainer::ICON_LEFT));
+	if (GuiStyles::intToAppUiTheme(PrefsSpecific::appUiTheme(*GlobInstcs::prefsPtr)) == GuiStyles::NO_THEME) {
+		m_ui->assignButton->setIcon(IconContainer::construcIcon(IconContainer::ICON_APP_ARROW_RIGHT));
+		m_ui->removeButton->setIcon(IconContainer::construcIcon(IconContainer::ICON_APP_ARROW_LEFT));
+		m_ui->removeAllButton->setIcon(IconContainer::construcIcon(IconContainer::ICON_APP_ARROW_LEFT));
+	} else {
+		m_ui->assignButton->setIcon(IconContainer::construcIcon(IconContainer::ICON_APP_ARROW_RIGHT_WHITE));
+		m_ui->removeButton->setIcon(IconContainer::construcIcon(IconContainer::ICON_APP_ARROW_LEFT_WHITE));
+		m_ui->removeAllButton->setIcon(IconContainer::construcIcon(IconContainer::ICON_APP_ARROW_LEFT_WHITE));
+	}
 }
