@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2024 CZ.NIC
+ * Copyright (C) 2014-2025 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #ifndef WIN32
 #include <clocale>
 #endif /* !WIN32 */
+#include <QCommandLineParser>
 #include <QCoreApplication>
 #include <QDir>
 #include <QFile>
@@ -60,11 +61,18 @@
 #  include "src/io/tag_container.h"
 #  include "src/io/tag_db.h"
 #endif /* defined(ENABLE_TAGS) */
+#if defined(ENABLE_TIMESTAMP_TRACKING)
+#  include "src/io/timestamp_db.h"
+#endif /* defined(ENABLE_TIMESTAMP_TRACKING) */
 #include "src/isds/qt_signal_slot.h"
+#include "src/json/db_info.h"
 #include "src/settings/accounts.h"
 #include "src/settings/preferences.h"
 #include "src/settings/prefs_specific.h"
 #include "src/settings/proxy.h"
+#if defined(Q_OS_WIN)
+#  include "src/settings/registry.h"
+#endif /* Q_OS_WIN */
 #include "src/settings/tag_container_settings.h"
 #include "src/single/single_instance.h"
 #include "src/worker/message_emitter.h"
@@ -86,19 +94,76 @@ void setDefaultLocale(void)
 #endif /* !WIN32 */
 }
 
+int preferencesSetUpRegistry(INIPreferences &iniPrefs)
+{
+#if defined(Q_OS_WIN) && !defined(PORTABLE_APPLICATION)
+	/*
+	 * Portable application doesn't load directory settings from registry.
+	 */
+	QString str;
+
+	if (RegPreferences::haveEntry(RegPreferences::LOC_POL, RegPreferences::ENTR_CONF_BASEDIR)) {
+		str = RegPreferences::confBaseDir(RegPreferences::LOC_POL);
+	} else if (RegPreferences::haveEntry(RegPreferences::LOC_SYS, RegPreferences::ENTR_CONF_BASEDIR)) {
+		str = RegPreferences::confBaseDir(RegPreferences::LOC_SYS);
+	} else if (RegPreferences::haveEntry(RegPreferences::LOC_USR, RegPreferences::ENTR_CONF_BASEDIR)) {
+		str = RegPreferences::confBaseDir(RegPreferences::LOC_USR);
+	}
+	if (!str.isEmpty()) {
+		iniPrefs._confBaseDir = macroStdMove(str);
+		str.clear();
+	}
+
+	if (RegPreferences::haveEntry(RegPreferences::LOC_POL, RegPreferences::ENTR_CONF_SUBDIR)) {
+		str = RegPreferences::confSubdir(RegPreferences::LOC_POL);
+	} else if (RegPreferences::haveEntry(RegPreferences::LOC_SYS, RegPreferences::ENTR_CONF_SUBDIR)) {
+		str = RegPreferences::confSubdir(RegPreferences::LOC_SYS);
+	} else if (RegPreferences::haveEntry(RegPreferences::LOC_USR, RegPreferences::ENTR_CONF_SUBDIR)) {
+		str = RegPreferences::confSubdir(RegPreferences::LOC_USR);
+	}
+	if (!str.isEmpty()) {
+		iniPrefs._confSubdir = macroStdMove(str);
+		str.clear();
+	}
+
+	if (RegPreferences::haveEntry(RegPreferences::LOC_POL, RegPreferences::ENTR_CONF_DIR)) {
+		str = RegPreferences::confDir(RegPreferences::LOC_POL);
+	} else if (RegPreferences::haveEntry(RegPreferences::LOC_SYS, RegPreferences::ENTR_CONF_DIR)) {
+		str = RegPreferences::confDir(RegPreferences::LOC_SYS);
+	} else if (RegPreferences::haveEntry(RegPreferences::LOC_USR, RegPreferences::ENTR_CONF_DIR)) {
+		str = RegPreferences::confDir(RegPreferences::LOC_USR);
+	}
+	if (!str.isEmpty()) {
+		iniPrefs._confDir = macroStdMove(str);
+		str.clear();
+	}
+
+#else /* !Q_OS_WIN || PORTABLE_APPLICATION */
+	Q_UNUSED(iniPrefs);
+#endif /* Q_OS_WIN && !PORTABLE_APPLICATION */
+
+	return 0;
+}
+
 int preferencesSetUp(const QCommandLineParser &parser, INIPreferences &iniPrefs,
     LogDevice &log)
 {
 	int logFileId = -1;
 
+	if (parser.isSet(CONF_BASEDIR_OPT)) {
+		iniPrefs._confBaseDir = parser.value(CONF_BASEDIR_OPT);
+	}
 	if (parser.isSet(CONF_SUBDIR_OPT)) {
-		iniPrefs.confSubdir = parser.value(CONF_SUBDIR_OPT);
+		iniPrefs._confSubdir = parser.value(CONF_SUBDIR_OPT);
+	}
+	if (parser.isSet(CONF_DIR_OPT)) {
+		iniPrefs._confDir = parser.value(CONF_DIR_OPT);
 	}
 	if (parser.isSet(LOAD_CONF_OPT)) {
-		iniPrefs.loadFromConf = parser.value(LOAD_CONF_OPT);
+		iniPrefs._loadFromConf = parser.value(LOAD_CONF_OPT);
 	}
 	if (parser.isSet(SAVE_CONF_OPT)) {
-		iniPrefs.saveToConf = parser.value(SAVE_CONF_OPT);
+		iniPrefs._saveToConf = parser.value(SAVE_CONF_OPT);
 	}
 	if (parser.isSet(LOG_FILE)) {
 		QString logFileName = parser.value(LOG_FILE);
@@ -147,6 +212,46 @@ int preferencesSetUp(const QCommandLineParser &parser, INIPreferences &iniPrefs,
 	return 0;
 }
 
+int preferencesOk(const INIPreferences &iniPrefs, QString &errorMessage)
+{
+	/*
+	 * There is no need to call tr() or use QT_TRANSLATE_NOOP() as the
+	 * localisation isn't loaded when this function is called.
+	 */
+
+	if (Q_UNLIKELY((!iniPrefs._confBaseDir.isEmpty())
+	        && (!QDir::isAbsolutePath(iniPrefs._confBaseDir)))) {
+		errorMessage = QString("%1: Configuration base directory cannot be set as a relative path '%2'.")
+		    .arg(CONF_BASEDIR_OPT)
+		    .arg(iniPrefs._confBaseDir);
+		return -1;
+	}
+	if (Q_UNLIKELY((!iniPrefs._confSubdir.isEmpty())
+	        && QDir::isAbsolutePath(iniPrefs._confSubdir))) {
+		errorMessage = QString("%1: Configuration subdirectory cannot be set as an absolute path '%2'.")
+		    .arg(CONF_SUBDIR_OPT)
+		    .arg(iniPrefs._confSubdir);
+		return -1;
+	}
+	if (Q_UNLIKELY((!iniPrefs._confDir.isEmpty())
+	        && (!QDir::isAbsolutePath(iniPrefs._confDir)))) {
+		errorMessage = QString("%1: Configuration directory cannot be set as a relative path '%2'.")
+		    .arg(CONF_DIR_OPT)
+		    .arg(iniPrefs._confDir);
+		return -1;
+	}
+	if (Q_UNLIKELY((!iniPrefs._confDir.isEmpty())
+	        && ((!iniPrefs._confBaseDir.isEmpty()) || (!iniPrefs._confSubdir.isEmpty())))) {
+		errorMessage = QString("%1: Configuration directory cannot be set when '%2' or '%3' is set.")
+		    .arg(CONF_DIR_OPT)
+		    .arg(CONF_BASEDIR_OPT)
+		    .arg(CONF_SUBDIR_OPT);
+		return -1;
+	}
+
+	return 0;
+}
+
 void downloadCRL(void)
 {
 	/* Start downloading the CRL files. */
@@ -183,20 +288,16 @@ void downloadCRL(void)
 	}
 }
 
-void loadLocalisation(const Prefs &prefs)
+/*!
+ * @brief Loads application localisation depending on \a language.
+ *
+ * @param[in] language Language string.
+ */
+static
+void loadLocalisation(const QString &language)
 {
 	static QTranslator qtTranslator, appTranslator;
 
-	QString language;
-	{
-		QString val;
-		if (prefs.strVal("translation.language", val)) {
-			if (!val.isEmpty()) {
-				language = macroStdMove(val);
-			}
-		}
-	}
-
 	/* Check for application localisation location. */
 	QString localisationDir;
 	QString localisationFile;
@@ -245,6 +346,26 @@ void loadLocalisation(const Prefs &prefs)
 	QCoreApplication::installTranslator(&qtTranslator);
 }
 
+void loadLocalisationSystem(void)
+{
+	loadLocalisation(Localisation::langSystem);
+}
+
+void loadLocalisationPrefs(const Prefs &prefs)
+{
+	QString language;
+	{
+		QString val;
+		if (prefs.strVal("translation.language", val)) {
+			if (!val.isEmpty()) {
+				language = macroStdMove(val);
+			}
+		}
+	}
+
+	loadLocalisation(language);
+}
+
 void logAppVersion(void)
 {
 	/* Crude architecture bit width check. */
@@ -456,6 +577,40 @@ void deallocGlobPrefs(void)
 	}
 }
 
+#if defined(ENABLE_TIMESTAMP_TRACKING)
+int allocGlobTimestamps(const INIPreferences &iniPrefs)
+{
+	SQLiteDb::OpenFlags flags = SQLiteDb::NO_OPTIONS;
+
+	GlobInstcs::timestampDbPtr = new (::std::nothrow) TimestampDb("timestampDb", false);
+	if (Q_UNLIKELY(Q_NULLPTR == GlobInstcs::timestampDbPtr)) {
+		logErrorNL("%s", "Cannot allocate timestamp db.");
+		goto fail;
+	}
+	/* Open timestamp database. */
+	flags = SQLiteDb::CREATE_MISSING;
+	if (Q_UNLIKELY(!GlobInstcs::timestampDbPtr->openDb(iniPrefs.timestampDbPath(), flags))) {
+		logErrorNL("Error opening timestamp db '%s'.",
+		    iniPrefs.timestampDbPath().toUtf8().constData());
+		goto fail;
+	}
+
+	return 0;
+
+fail:
+	deallocGlobTimestamps();
+	return -1;
+}
+
+void deallocGlobTimestamps(void)
+{
+	if (Q_NULLPTR != GlobInstcs::timestampDbPtr) {
+		delete GlobInstcs::timestampDbPtr;
+		GlobInstcs::timestampDbPtr = Q_NULLPTR;
+	}
+}
+#endif /* defined(ENABLE_TIMESTAMP_TRACKING) */
+
 #if defined(ENABLE_TAGS)
 void createBackupOfOldFormatData(const INIPreferences &iniPrefs)
 {
@@ -470,7 +625,7 @@ void createBackupOfOldFormatData(const INIPreferences &iniPrefs)
 		if (Q_UNLIKELY(!tagDb.openDb(tagDbPath, SQLiteDb::NO_OPTIONS))) {
 			return;
 		}
-		Json::TagDbInfo info;
+		Json::DbInfo info;
 		tagDb.getDbInfo(info);
 		backUp = info.formatVersionMajor() < 1;
 	}
@@ -488,9 +643,12 @@ void createBackupOfOldFormatData(const INIPreferences &iniPrefs)
 }
 #endif /* defined(ENABLE_TAGS) */
 
-int allocGlobContainers(const Prefs &prefs, const INIPreferences &iniPrefs)
+enum InitialisationError allocGlobContainers(const Prefs &prefs,
+    const INIPreferences &iniPrefs, QString *problemPathPtr)
 {
 	SQLiteDb::OpenFlags flags = SQLiteDb::NO_OPTIONS;
+	QString problemPath;
+	enum InitialisationError retCode = SUCCESS;
 
 	/*
 	 * These objects cannot be globally accessible static objects.
@@ -503,12 +661,14 @@ int allocGlobContainers(const Prefs &prefs, const INIPreferences &iniPrefs)
 	GlobInstcs::isdsSessionsPtr = new (::std::nothrow) IsdsSessions;
 	if (Q_UNLIKELY(Q_NULLPTR == GlobInstcs::isdsSessionsPtr)) {
 		logErrorNL("%s", "Cannot allocate session container.");
+		retCode = ERR_ALLOC_RESOURCES;
 		goto fail;
 	}
 
 	GlobInstcs::accntDbPtr = new (::std::nothrow) AccountDb("accountDb", false);
 	if (Q_UNLIKELY(Q_NULLPTR == GlobInstcs::accntDbPtr)) {
 		logErrorNL("%s", "Cannot allocate account db.");
+		retCode = ERR_ALLOC_RESOURCES;
 		goto fail;
 	}
 	/* Open accounts database. */
@@ -518,6 +678,8 @@ int allocGlobContainers(const Prefs &prefs, const INIPreferences &iniPrefs)
 	if (Q_UNLIKELY(!GlobInstcs::accntDbPtr->openDb(iniPrefs.acntDbPath(), flags))) {
 		logErrorNL("Error opening account db '%s'.",
 		    iniPrefs.acntDbPath().toUtf8().constData());
+		problemPath = iniPrefs.acntDbPath();
+		retCode = ERR_OPEN_FILES;
 		goto fail;
 	}
 
@@ -526,6 +688,7 @@ int allocGlobContainers(const Prefs &prefs, const INIPreferences &iniPrefs)
 	    PrefsSpecific::messageDbOnDisk(prefs), "GLOBALDBS");
 	if (Q_UNLIKELY(Q_NULLPTR == GlobInstcs::msgDbsPtr)) {
 		logErrorNL("%s", "Cannot allocate message db container.");
+		retCode = ERR_ALLOC_RESOURCES;
 		goto fail;
 	}
 
@@ -533,16 +696,19 @@ int allocGlobContainers(const Prefs &prefs, const INIPreferences &iniPrefs)
 	GlobInstcs::tagContSetPtr = new (::std::nothrow) TagContainerSettings();
 	if (Q_UNLIKELY(Q_NULLPTR == GlobInstcs::tagContSetPtr)) {
 		logErrorNL("%s", "Cannot allocate tag container settings.");
+		retCode = ERR_ALLOC_RESOURCES;
 		goto fail;
 	}
 	GlobInstcs::tagContPtr = new (::std::nothrow) TagContainer(TagContainer::BACK_DB);
 	if (Q_UNLIKELY(Q_NULLPTR == GlobInstcs::tagContPtr)) {
 		logErrorNL("%s", "Cannot allocate tag container.");
+		retCode = ERR_ALLOC_RESOURCES;
 		goto fail;
 	}
 	/* Check tag database backend presence. */
 	if (Q_UNLIKELY(Q_NULLPTR == GlobInstcs::tagContPtr->backendDb())) {
 		logErrorNL("%s", "Cannot allocate tag db.");
+		retCode = ERR_ALLOC_RESOURCES;
 		goto fail;
 	}
 	/* Open tags database. */
@@ -550,11 +716,14 @@ int allocGlobContainers(const Prefs &prefs, const INIPreferences &iniPrefs)
 	if (Q_UNLIKELY(!GlobInstcs::tagContPtr->backendDb()->openDb(iniPrefs.tagDbPath(), flags))) {
 		logErrorNL("Error opening tag db '%s'.",
 		    iniPrefs.tagDbPath().toUtf8().constData());
+		problemPath = iniPrefs.tagDbPath();
+		retCode = ERR_OPEN_FILES;
 		goto fail;
 	}
 	/* Check tag client backend presence. */
 	if (Q_UNLIKELY(Q_NULLPTR == GlobInstcs::tagContPtr->backendClient())) {
 		logErrorNL("%s", "Cannot allocate tag client.");
+		retCode = ERR_ALLOC_RESOURCES;
 		goto fail;
 	}
 #endif /* defined(ENABLE_TAGS) */
@@ -563,6 +732,7 @@ int allocGlobContainers(const Prefs &prefs, const INIPreferences &iniPrefs)
 	    RecordsManagementDb("recordsManagementDb", false);
 	if (Q_UNLIKELY(Q_NULLPTR == GlobInstcs::recMgmtDbPtr)) {
 		logErrorNL("%s", "Cannot allocate records management db.");
+		retCode = ERR_ALLOC_RESOURCES;
 		goto fail;
 	}
 	/* Open records management database. */
@@ -570,6 +740,8 @@ int allocGlobContainers(const Prefs &prefs, const INIPreferences &iniPrefs)
 	if (Q_UNLIKELY(!GlobInstcs::recMgmtDbPtr->openDb(iniPrefs.recMgmtDbPath(), flags))) {
 		logErrorNL("Error opening records management db '%s'.",
 		    iniPrefs.recMgmtDbPath().toUtf8().constData());
+		problemPath = iniPrefs.recMgmtDbPath();
+		retCode = ERR_OPEN_FILES;
 		goto fail;
 	}
 
@@ -577,6 +749,7 @@ int allocGlobContainers(const Prefs &prefs, const INIPreferences &iniPrefs)
 	    AcntContainer::CT_REGULAR);
 	if (Q_UNLIKELY(Q_NULLPTR == GlobInstcs::acntMapPtr)) {
 		logErrorNL("%s", "Cannot allocate account container.");
+		retCode = ERR_ALLOC_RESOURCES;
 		goto fail;
 	}
 
@@ -584,14 +757,18 @@ int allocGlobContainers(const Prefs &prefs, const INIPreferences &iniPrefs)
 	    AcntContainer::CT_SHADOW);
 	if (Q_UNLIKELY(Q_NULLPTR == GlobInstcs::shadowAcntsPtr)) {
 		logErrorNL("%s", "Cannot allocate shadow account container.");
+		retCode = ERR_ALLOC_RESOURCES;
 		goto fail;
 	}
 
-	return 0;
+	return SUCCESS;
 
 fail:
 	deallocGlobContainers();
-	return -1;
+	if (Q_NULLPTR != problemPathPtr) {
+		*problemPathPtr = macroStdMove(problemPath);
+	}
+	return retCode;
 }
 
 void deallocGlobContainers(void)
