@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2024 CZ.NIC
+ * Copyright (C) 2014-2025 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -71,10 +71,10 @@ DbEntry AccountDb::accountEntry(const QString &key) const
 	}
 
 	queryStr = "SELECT ";
-	for (int i = 0; i < (accntinfTbl.knownAttrs.size() - 1); ++i) {
-		queryStr += accntinfTbl.knownAttrs[i].first + ", ";
+	for (int i = 0; i < (AccountDbTables::accntinfTbl.knownAttrs.size() - 1); ++i) {
+		queryStr += AccountDbTables::accntinfTbl.knownAttrs[i].first + ", ";
 	}
-	queryStr += accntinfTbl.knownAttrs.last().first + " ";
+	queryStr += AccountDbTables::accntinfTbl.knownAttrs.last().first + " ";
 	queryStr += "FROM account_info WHERE key = :key";
 
 	if (Q_UNLIKELY(!query.prepare(queryStr))) {
@@ -85,11 +85,11 @@ DbEntry AccountDb::accountEntry(const QString &key) const
 	query.bindValue(":key", key);
 	if (query.exec() && query.isActive() && query.first()) {
 		QSqlRecord rec = query.record();
-		for (int i = 0; i < accntinfTbl.knownAttrs.size(); ++i) {
+		for (int i = 0; i < AccountDbTables::accntinfTbl.knownAttrs.size(); ++i) {
 			QVariant value = query.value(rec.indexOf(
-			    accntinfTbl.knownAttrs[i].first));
+			    AccountDbTables::accntinfTbl.knownAttrs[i].first));
 			if (!value.isNull() && value.isValid()) {
-				aEntry.setValue(accntinfTbl.knownAttrs[i].first,
+				aEntry.setValue(AccountDbTables::accntinfTbl.knownAttrs[i].first,
 				    value);
 			}
 		}
@@ -115,10 +115,10 @@ DbEntry AccountDb::userEntry(const QString &key) const
 	}
 
 	queryStr = "SELECT ";
-	for (int i = 0; i < (userinfTbl.knownAttrs.size() - 1); ++i) {
-		queryStr += userinfTbl.knownAttrs[i].first + ", ";
+	for (int i = 0; i < (AccountDbTables::userinfTbl.knownAttrs.size() - 1); ++i) {
+		queryStr += AccountDbTables::userinfTbl.knownAttrs[i].first + ", ";
 	}
-	queryStr += userinfTbl.knownAttrs.last().first + " ";
+	queryStr += AccountDbTables::userinfTbl.knownAttrs.last().first + " ";
 	queryStr += "FROM user_info WHERE key = :key";
 
 	if (Q_UNLIKELY(!query.prepare(queryStr))) {
@@ -129,11 +129,11 @@ DbEntry AccountDb::userEntry(const QString &key) const
 	query.bindValue(":key", key);
 	if (query.exec() && query.isActive() && query.first()) {
 		QSqlRecord rec = query.record();
-		for (int i = 0; i < userinfTbl.knownAttrs.size(); ++i) {
+		for (int i = 0; i < AccountDbTables::userinfTbl.knownAttrs.size(); ++i) {
 			QVariant value = query.value(rec.indexOf(
-			    userinfTbl.knownAttrs[i].first));
+			    AccountDbTables::userinfTbl.knownAttrs[i].first));
 			if (!value.isNull() && value.isValid()) {
-				uEntry.setValue(userinfTbl.knownAttrs[i].first,
+				uEntry.setValue(AccountDbTables::userinfTbl.knownAttrs[i].first,
 				    value);
 			}
 		}
@@ -787,12 +787,133 @@ fail:
 	return Isds::DTInfoOutput();
 }
 
+Isds::DbOwnerInfoExt2 AccountDb::getOwnerInfo2(const QString &key) const
+{
+	QMutexLocker locker(&m_lock);
+	QSqlQuery query(m_db);
+	QString queryStr;
+	Isds::AddressExt2 address;
+	Isds::BirthInfo biInfo;
+	Isds::DbOwnerInfoExt2 dbOwnerInfo;
+	Isds::PersonName2 personName;
+
+	if (Q_UNLIKELY(!m_db.isOpen())) {
+		logErrorNL("%s", "Account database seems not to be open.");
+		goto fail;
+	}
+
+	queryStr = "SELECT "
+	    "dbID, dbType, ic, pnFirstName, pnMiddleName, pnLastName, "
+	    "pnLastNameAtBirth, firmName, biDate, biCity, biCounty, biState, "
+	    "adCity, adStreet, adNumberInStreet, adNumberInMunicipality, "
+	    "adZipCode, adState, nationality, identifier, registryCode, "
+	    "dbState, dbEffectiveOVM, dbOpenAddressing "
+	    "FROM account_info WHERE key = :key";
+
+	if (Q_UNLIKELY(!query.prepare(queryStr))) {
+		logErrorNL("Cannot prepare SQL query: %s.",
+		    query.lastError().text().toUtf8().constData());
+		goto fail;
+	}
+	query.bindValue(":key", key);
+	if (query.exec() && query.isActive() &&
+	    query.first() && query.isValid()) {
+		dbOwnerInfo.setDbID(query.value(0).toString());
+		dbOwnerInfo.setDbType(Isds::strVariant2DbType(query.value(1)));
+		dbOwnerInfo.setIc(query.value(2).toString());
+		personName.setGivenNames(query.value(3).toString());
+		personName.setLastName(query.value(5).toString());
+		dbOwnerInfo.setPersonName(macroStdMove(personName));
+		dbOwnerInfo.setFirmName(query.value(7).toString());
+		biInfo.setDate(dateFromDbFormat(query.value(8).toString()));
+		biInfo.setCity(query.value(9).toString());
+		biInfo.setCounty(query.value(10).toString());
+		biInfo.setState(query.value(11).toString());
+		dbOwnerInfo.setBirthInfo(macroStdMove(biInfo));
+		address.setCity(query.value(12).toString());
+		address.setStreet(query.value(13).toString());
+		address.setNumberInStreet(query.value(14).toString());
+		address.setNumberInMunicipality(query.value(15).toString());
+		address.setZipCode(query.value(16).toString());
+		address.setState(query.value(17).toString());
+		dbOwnerInfo.setAddress(macroStdMove(address));
+		dbOwnerInfo.setNationality(query.value(18).toString());
+		dbOwnerInfo.setDbState(Isds::variant2DbState(query.value(21)));
+		dbOwnerInfo.setDbOpenAddressing(
+		    Isds::variant2NilBool(query.value(23)));
+		return dbOwnerInfo;
+	} else {
+		logErrorNL("Cannot execute SQL query and/or read SQL data: "
+		    "%s.", query.lastError().text().toUtf8().constData());
+	}
+fail:
+	return Isds::DbOwnerInfoExt2();
+}
+
+Isds::DbUserInfoExt2 AccountDb::getUserInfo2(const QString &key) const
+{
+	QMutexLocker locker(&m_lock);
+	QSqlQuery query(m_db);
+	QString queryStr;
+	Isds::AddressExt2 address;
+	Isds::DbUserInfoExt2 dbUserInfo;
+	Isds::PersonName2 personName;
+
+	if (Q_UNLIKELY(!m_db.isOpen())) {
+		logErrorNL("%s", "Account database seems not to be open.");
+		goto fail;
+	}
+
+	queryStr = "SELECT "
+	    "firmName, ic, pnFirstName, pnMiddleName, pnLastName, biDate, "
+	    "adCity, adStreet, adNumberInStreet, adNumberInMunicipality, "
+	    "adZipCode, adState, caCity, caStreet, caZipCode, caState, "
+	    "userType, userPrivils FROM user_info WHERE key = :key";
+
+	if (Q_UNLIKELY(!query.prepare(queryStr))) {
+		logErrorNL("Cannot prepare SQL query: %s.",
+		    query.lastError().text().toUtf8().constData());
+		goto fail;
+	}
+	query.bindValue(":key", key);
+	if (query.exec() && query.isActive() &&
+	    query.first() && query.isValid()) {
+		dbUserInfo.setFirmName(query.value(0).toString());
+		dbUserInfo.setIc(query.value(1).toString());
+		personName.setGivenNames(query.value(2).toString());
+		personName.setLastName(query.value(4).toString());
+		dbUserInfo.setPersonName(macroStdMove(personName));
+		dbUserInfo.setBiDate(dateFromDbFormat(query.value(5).toString()));
+		address.setCity(query.value(6).toString());
+		address.setStreet(query.value(7).toString());
+		address.setNumberInStreet(query.value(8).toString());
+		address.setNumberInMunicipality(query.value(9).toString());
+		address.setZipCode(query.value(10).toString());
+		address.setState(query.value(11).toString());
+		dbUserInfo.setAddress(macroStdMove(address));
+		dbUserInfo.setCaCity(query.value(12).toString());
+		dbUserInfo.setCaStreet(query.value(13).toString());
+		dbUserInfo.setCaZipCode(query.value(14).toString());
+		dbUserInfo.setCaState(query.value(15).toString());
+		dbUserInfo.setUserType(
+		    Isds::str2UserType(query.value(16).toString()));
+		dbUserInfo.setUserPrivils(
+		    Isds::long2Privileges(query.value(17).toLongLong()));
+		return dbUserInfo;
+	} else {
+		logErrorNL("Cannot execute SQL query and/or read SQL data: "
+		    "%s.", query.lastError().text().toUtf8().constData());
+	}
+fail:
+	return Isds::DbUserInfoExt2();
+}
+
 QList<class SQLiteTbl *> AccountDb::listOfTables(void) const
 {
 	QList<class SQLiteTbl *> tables;
-	tables.append(&accntinfTbl);
-	tables.append(&userinfTbl);
-	tables.append(&pwdexpdtTbl);
-	tables.append(&dtinfTbl);
+	tables.append(&AccountDbTables::accntinfTbl);
+	tables.append(&AccountDbTables::userinfTbl);
+	tables.append(&AccountDbTables::pwdexpdtTbl);
+	tables.append(&AccountDbTables::dtinfTbl);
 	return tables;
 }
