@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2024 CZ.NIC
+ * Copyright (C) 2014-2025 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,13 +21,17 @@
  * the two.
  */
 
-#include <QFileDialog>
+#ifdef QT_WIDGETS_LIB
+#  include <QFileDialog>
+#endif /* QT_WIDGETS_LIB */
 #include <QPrinter>
 #include <QTextDocument>
 
 #include "src/datovka_shared/compat/compiler.h" /* macroStdMove */
+#include "src/datovka_shared/html/html_export.h"
 #include "src/datovka_shared/log/log.h"
 #include "src/datovka_shared/settings/prefs.h"
+#include "src/datovka_shared/utility/pdf_printer.h"
 #include "src/global.h"
 #include "src/io/export_file_name.h"
 #include "src/io/exports.h"
@@ -99,7 +103,174 @@ QString Exports::attachmentSavePathWithFileName(const MessageDbSet &dbSet,
 	return targetPath + QDir::separator() + fileName;
 }
 
-enum Exports::ExportError Exports::exportAs(QWidget *parent,
+#define PDF_FONT_FILE_NAME ":/font/NotoSerif-Regular.ttf"
+
+enum Exports::ExportError Exports::exportAs(const MessageDbSet &dbSet,
+    enum ExportFileType fileType, const QString &targetPath,
+    QString fileNameFormat, const QString &userName,
+    const QString &accountName, const QString &dbId, const MsgId &msgId,
+    QString &errStr)
+{
+	debugFuncCall();
+
+	if (Q_UNLIKELY(userName.isEmpty())) {
+		Q_ASSERT(0);
+		return EXP_ERROR;
+	}
+	if (Q_UNLIKELY(msgId.dmId() < 0)) {
+		Q_ASSERT(0);
+		return EXP_ERROR;
+	}
+
+	QString fileTypeStr;
+	QString fileSufix;
+	QByteArray data;
+	bool ret = false;
+	QString msgID = QString::number(msgId.dmId());
+
+	MessageDb *messageDb = dbSet.constAccessMessageDb(msgId.deliveryTime());
+	if (Q_UNLIKELY(Q_NULLPTR == messageDb)) {
+		Q_ASSERT(0);
+		errStr = tr("Cannot access message database for username '%1'.")
+		    .arg(userName);
+		return EXP_DB_ERROR;
+	}
+
+	QString fileName;
+	{
+		QString dfltFileNameformat;
+
+		/* What will we export? */
+		switch (fileType) {
+		case ZFO_MESSAGE:
+			fileTypeStr = tr("message");
+			fileSufix = ".zfo";
+			dfltFileNameformat = DEFAULT_MESSAGE_FILENAME_FORMAT;
+			data = messageDb->getCompleteMessageRaw(msgId.dmId());
+			break;
+		case ZFO_DELIVERY:
+			fileTypeStr = tr("acceptance info");
+			fileSufix = ".zfo";
+			dfltFileNameformat = DEFAULT_ACCEPTANCE_INFO_FILENAME_FORMAT;
+			data = messageDb->getDeliveryInfoRaw(msgId.dmId());
+			break;
+		case ZFO_DELIV_ATTACH:
+			fileTypeStr = tr("acceptance info");
+			fileSufix = ".zfo";
+			dfltFileNameformat = DEFAULT_ATTACHMENT_ACCEPTANCE_FORMAT;
+			data = messageDb->getDeliveryInfoRaw(msgId.dmId());
+			break;
+		case PDF_DELIVERY:
+			fileTypeStr = tr("acceptance info");
+			fileSufix = ".pdf";
+			dfltFileNameformat = DEFAULT_ACCEPTANCE_INFO_FILENAME_FORMAT;
+			data = messageDb->getDeliveryInfoRaw(msgId.dmId());
+			break;
+		case PDF_DELIV_ATTACH:
+			fileTypeStr = tr("acceptance info");
+			fileSufix = ".pdf";
+			dfltFileNameformat = DEFAULT_ATTACHMENT_ACCEPTANCE_FORMAT;
+			data = messageDb->getDeliveryInfoRaw(msgId.dmId());
+			break;
+		case PDF_ENVELOPE:
+			fileTypeStr = tr("message envelope");
+			fileSufix = ".pdf";
+			dfltFileNameformat = DEFAULT_MESSAGE_FILENAME_FORMAT;
+			data = "n/a";
+			break;
+		default:
+			Q_ASSERT(0);
+			errStr = tr("Export file type of message '%1' was not specified!")
+			    .arg(msgID);
+			return EXP_ERROR;
+			break;
+		}
+
+		if (fileNameFormat.isEmpty()) {
+			fileNameFormat = macroStdMove(dfltFileNameformat);
+		}
+
+		/* Is complete message in database? */
+		if (Q_UNLIKELY(data.isEmpty())) {
+			errStr = tr("Complete message '%1' missing!").arg(msgID);
+			return EXP_NOT_MSG_DATA;
+		}
+
+		errStr = tr("Export of %1 '%2' to %3 was not successful!")
+		    .arg(fileTypeStr).arg(msgID).arg(fileSufix);
+
+		const bool prohibitDirSep = false;
+
+		const Isds::Envelope envelope = messageDb->getMessageEnvelope(msgId.dmId());
+		/*
+		 * Create new file name with format string. Allow subdirectories
+		 * if the user should not be asked after location.
+		 */
+		fileName = fileSubPathFromFormat(macroStdMove(fileNameFormat),
+		    prohibitDirSep, msgId.dmId(), dbId, userName, accountName,
+		    QString(), envelope,
+		    dmTypePrefix(messageDb, msgId.dmId()), fileSufix);
+	}
+	fileName = targetPath + QDir::separator() + fileName;
+
+	if (Q_UNLIKELY(fileName.isEmpty())) {
+		Q_ASSERT(0);
+		return EXP_ERROR;
+	}
+
+	createDirStructureRecursive(fileName);
+
+	if (Q_UNLIKELY(fileName.isEmpty())) {
+		Q_ASSERT(0);
+		return EXP_ERROR;
+	}
+
+	// save file to disk
+	switch (fileType) {
+	case ZFO_DELIV_ATTACH:
+	case ZFO_DELIVERY:
+	case ZFO_MESSAGE:
+		ret = writeZFO(fileName, data);
+		break;
+	case PDF_DELIVERY:
+	case PDF_DELIV_ATTACH:
+		ret = Utility::printPDFWithFont(fileName,
+		    Html::Export::htmlInfoPdf(Html::Export::TYPE_DELIVERY_INFO,
+		    messageDb->getMessageEnvelope(msgId.dmId()),
+		    QList<Isds::Document>(),
+		    messageDb->messageAuthorJsonStr(msgId.dmId())),
+		    Utility::HTML_TEXT, PDF_FONT_FILE_NAME);
+		break;
+	case PDF_ENVELOPE:
+		ret = Utility::printPDFWithFont(fileName,
+		    Html::Export::htmlInfoPdf(Html::Export::TYPE_ENVELOPE_INFO,
+		    messageDb->getMessageEnvelope(msgId.dmId()),
+		    messageDb->getMessageAttachments(msgId.dmId()),
+		    messageDb->messageAuthorJsonStr(msgId.dmId())),
+		    Utility::HTML_TEXT, PDF_FONT_FILE_NAME);
+		break;
+	default:
+		Q_ASSERT(0);
+		return EXP_ERROR;
+		break;
+	}
+
+	/* Test if file was exported. TODO - show error dialogue? */
+	ret = QFileInfo(fileName).exists();
+
+	if (ret) {
+		errStr = tr("Export of %1 '%2' to %3 was successful.")
+		    .arg(fileTypeStr).arg(msgID).arg(fileSufix);
+		return EXP_SUCCESS;
+	} else {
+		errStr = tr("Export of %1 '%2' to %3 wasn't successful. Cannot write the file. File name may be too long.")
+		    .arg(fileTypeStr).arg(msgID).arg(fileSufix);
+		return EXP_WRITE_FILE_ERROR;
+	}
+}
+
+#ifdef QT_WIDGETS_LIB
+enum Exports::ExportError Exports::exportAsGUI(QWidget *parent,
     const MessageDbSet &dbSet, enum ExportFileType fileType,
     const QString &targetPath, const QString &attachFileName,
     const QString &userName, const QString &accountName, const QString &dbId,
@@ -132,7 +303,7 @@ enum Exports::ExportError Exports::exportAs(QWidget *parent,
 
 	QString fileName;
 	{
-		QString fileNameformat;
+		QString fileNameFormat;
 
 		/* What will we export? */
 		switch (fileType) {
@@ -140,14 +311,14 @@ enum Exports::ExportError Exports::exportAs(QWidget *parent,
 			fileTypeStr = tr("message");
 			fileSufix = ".zfo";
 			GlobInstcs::prefsPtr->strVal(
-			    "file_system.format.filename.message", fileNameformat);
+			    "file_system.format.filename.message", fileNameFormat);
 			data = messageDb->getCompleteMessageRaw(msgId.dmId());
 			break;
 		case ZFO_DELIVERY:
 			fileTypeStr = tr("acceptance info");
 			fileSufix = ".zfo";
 			GlobInstcs::prefsPtr->strVal(
-			    "file_system.format.filename.acceptance_info", fileNameformat);
+			    "file_system.format.filename.acceptance_info", fileNameFormat);
 			data = messageDb->getDeliveryInfoRaw(msgId.dmId());
 			break;
 		case ZFO_DELIV_ATTACH:
@@ -155,14 +326,14 @@ enum Exports::ExportError Exports::exportAs(QWidget *parent,
 			fileSufix = ".zfo";
 			GlobInstcs::prefsPtr->strVal(
 			    "file_system.format.filename.every_attachment_acceptance_info",
-			    fileNameformat);
+			    fileNameFormat);
 			data = messageDb->getDeliveryInfoRaw(msgId.dmId());
 			break;
 		case PDF_DELIVERY:
 			fileTypeStr = tr("acceptance info");
 			fileSufix = ".pdf";
 			GlobInstcs::prefsPtr->strVal(
-			    "file_system.format.filename.acceptance_info", fileNameformat);
+			    "file_system.format.filename.acceptance_info", fileNameFormat);
 			data = messageDb->getDeliveryInfoRaw(msgId.dmId());
 			break;
 		case PDF_DELIV_ATTACH:
@@ -170,14 +341,14 @@ enum Exports::ExportError Exports::exportAs(QWidget *parent,
 			fileSufix = ".pdf";
 			GlobInstcs::prefsPtr->strVal(
 			    "file_system.format.filename.every_attachment_acceptance_info",
-			    fileNameformat);
+			    fileNameFormat);
 			data = messageDb->getDeliveryInfoRaw(msgId.dmId());
 			break;
 		case PDF_ENVELOPE:
 			fileTypeStr = tr("message envelope");
 			fileSufix = ".pdf";
 			GlobInstcs::prefsPtr->strVal(
-			    "file_system.format.filename.message", fileNameformat);
+			    "file_system.format.filename.message", fileNameFormat);
 			data = "n/a";
 			break;
 		default:
@@ -204,7 +375,7 @@ enum Exports::ExportError Exports::exportAs(QWidget *parent,
 		 * Create new file name with format string. Allow subdirectories
 		 * if the user should not be asked after location.
 		 */
-		fileName = fileSubPathFromFormat(macroStdMove(fileNameformat),
+		fileName = fileSubPathFromFormat(macroStdMove(fileNameFormat),
 		    prohibitDirSep, msgId.dmId(), dbId, userName, accountName,
 		    attachFileName, envelope,
 		    dmTypePrefix(messageDb, msgId.dmId()), fileSufix);
@@ -243,12 +414,20 @@ enum Exports::ExportError Exports::exportAs(QWidget *parent,
 		break;
 	case PDF_DELIVERY:
 	case PDF_DELIV_ATTACH:
-		ret = printPDF(fileName,
-		    messageDb->deliveryInfoHtmlToPdf(msgId.dmId()));
+		ret = Utility::printPDFWithFont(fileName,
+		    Html::Export::htmlInfoPdf(Html::Export::TYPE_DELIVERY_INFO,
+		    messageDb->getMessageEnvelope(msgId.dmId()),
+		    QList<Isds::Document>(),
+		    messageDb->messageAuthorJsonStr(msgId.dmId())),
+		    Utility::HTML_TEXT, PDF_FONT_FILE_NAME);
 		break;
 	case PDF_ENVELOPE:
-		ret = printPDF(fileName,
-		    messageDb->envelopeInfoHtmlToPdf(msgId.dmId(), ""));
+		ret = Utility::printPDFWithFont(fileName,
+		    Html::Export::htmlInfoPdf(Html::Export::TYPE_ENVELOPE_INFO,
+		    messageDb->getMessageEnvelope(msgId.dmId()),
+		    messageDb->getMessageAttachments(msgId.dmId()),
+		    messageDb->messageAuthorJsonStr(msgId.dmId())),
+		    Utility::HTML_TEXT, PDF_FONT_FILE_NAME);
 		break;
 	default:
 		Q_ASSERT(0);
@@ -271,6 +450,7 @@ enum Exports::ExportError Exports::exportAs(QWidget *parent,
 		return EXP_WRITE_FILE_ERROR;
 	}
 }
+#endif /* QT_WIDGETS_LIB */
 
 /*!
  * @brief Split the file name into the base name and full suffix.
@@ -395,8 +575,6 @@ enum Exports::ExportError Exports::exportEnvAndAttachments(
 		return EXP_DB_ERROR;
 	}
 
-	Isds::Envelope envelope = messageDb->getMessageEnvelope(msgId.dmId());
-
 	/* Get list of attachment documents. */
 	QList<Isds::Document> attachList =
 	    messageDb->getMessageAttachments(msgId.dmId());
@@ -405,8 +583,6 @@ enum Exports::ExportError Exports::exportEnvAndAttachments(
 		return EXP_NOT_MSG_DATA;
 	}
 
-	messageDb->getMessageType(msgId.dmId());
-
 	/*
 	 * Treat all FS as case-insensitive. This will make the life a little
 	 * bit easier. Also for the user, as they may copy the generated files
@@ -443,6 +619,8 @@ enum Exports::ExportError Exports::exportEnvAndAttachments(
 		}
 	}
 
+	Isds::Envelope envelope = messageDb->getMessageEnvelope(msgId.dmId());
+
 	QString fileName;
 	{
 		QString msgFilenameFmt;
@@ -465,19 +643,21 @@ enum Exports::ExportError Exports::exportEnvAndAttachments(
 	createDirStructureRecursive(fileName);
 
 	/* Write file. */
-	if (printPDF(fileName,
-	    messageDb->descriptionHtml(msgId.dmId()) +
-	    messageDb->fileListHtmlToPdf(msgId.dmId())) && attachWriteSuccess) {
-		errStr = tr("All attachments of message '%1' and the envelope PDF have been successfully saved to target directory.")
-		    .arg(msgID);
+	if (Utility::printPDFWithFont(fileName,
+	    Html::Export::htmlInfoPdf(Html::Export::TYPE_ENVELOPE_INFO,
+	    envelope,
+	    messageDb->getMessageAttachments(msgId.dmId()),
+	    messageDb->messageAuthorJsonStr(msgId.dmId())),
+	    Utility::HTML_TEXT, PDF_FONT_FILE_NAME) && attachWriteSuccess) {
+		errStr = tr("All attachments of message '%1' and the envelope PDF have been successfully saved to target directory.").arg(msgID);
 		return EXP_SUCCESS;
 	} else {
-		errStr = tr("The envelope PDF of message '%1' hasn't been saved.")
-		    .arg(msgID);
+		errStr = tr("The envelope PDF of message '%1' hasn't been saved.").arg(msgID);
 		return EXP_WRITE_FILE_ERROR;
 	}
 }
 
+#ifdef QT_WIDGETS_LIB
 enum Exports::ExportError Exports::saveAttachmentsWithExports(
     const MessageDbSet &dbSet, const QString &targetPath,
     const QString &userName, const QString &accountName, const QString &dbId,
@@ -563,13 +743,13 @@ enum Exports::ExportError Exports::saveAttachmentsWithExports(
 
 		if (accInfoForEveryFile) {
 			if (exportAccInfoZfo) {
-				exportAs(Q_NULLPTR, dbSet, Exports::ZFO_DELIV_ATTACH,
+				exportAsGUI(Q_NULLPTR, dbSet, Exports::ZFO_DELIV_ATTACH,
 				    targetPath, attach.fileDescr(), userName,
 				    accountName, dbId, msgId, false, lastPath,
 				    errStr);
 			}
 			if (exportAccInfoPdf) {
-				exportAs(Q_NULLPTR, dbSet, Exports::PDF_DELIV_ATTACH,
+				exportAsGUI(Q_NULLPTR, dbSet, Exports::PDF_DELIV_ATTACH,
 				    targetPath, attach.fileDescr(), userName,
 				    accountName, dbId, msgId, false, lastPath,
 				    errStr);
@@ -578,25 +758,25 @@ enum Exports::ExportError Exports::saveAttachmentsWithExports(
 	}
 
 	if (exportMsgZfo) {
-		exportAs(Q_NULLPTR, dbSet, Exports::ZFO_MESSAGE,
+		exportAsGUI(Q_NULLPTR, dbSet, Exports::ZFO_MESSAGE,
 		    targetPath, QString(), userName, accountName, dbId, msgId,
 		    false, lastPath, errStr);
 	}
 
 	if (exportEnvelPdf) {
-		exportAs(Q_NULLPTR, dbSet, Exports::PDF_ENVELOPE,
+		exportAsGUI(Q_NULLPTR, dbSet, Exports::PDF_ENVELOPE,
 		    targetPath, QString(), userName, accountName, dbId, msgId,
 		    false, lastPath, errStr);
 	}
 
 	if (!accInfoForEveryFile) {
 		if (exportAccInfoZfo) {
-			exportAs(Q_NULLPTR, dbSet, Exports::ZFO_DELIVERY,
+			exportAsGUI(Q_NULLPTR, dbSet, Exports::ZFO_DELIVERY,
 			    targetPath, QString(), userName, accountName, dbId,
 			    msgId, false, lastPath, errStr);
 		}
 		if (exportAccInfoPdf) {
-			exportAs(Q_NULLPTR, dbSet, Exports::PDF_DELIVERY,
+			exportAsGUI(Q_NULLPTR, dbSet, Exports::PDF_DELIVERY,
 			    targetPath, QString(), userName, accountName, dbId,
 			    msgId, false, lastPath, errStr);
 		}
@@ -612,6 +792,7 @@ enum Exports::ExportError Exports::saveAttachmentsWithExports(
 		return EXP_WRITE_FILE_ERROR;
 	}
 }
+#endif /* QT_WIDGETS_LIB */
 
 bool Exports::printPDF(const QString &fileName, const QString &data)
 {
