@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2024 CZ.NIC
+ * Copyright (C) 2014-2025 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,7 +21,9 @@
  * the two.
  */
 
+#include <QByteArray>
 #include <QCoreApplication>
+#include <QDateTime>
 #include <QDir>
 #include <QDirIterator>
 #include <QFile>
@@ -29,12 +31,14 @@
 #include <QLibraryInfo>
 #include <QRegularExpression>
 #include <QStandardPaths>
+#include <QStringBuilder>
 #include <QTemporaryDir>
 #include <QTemporaryFile>
 #include <QTextStream>
 
 #include "src/datovka_shared/log/log.h"
 #include "src/datovka_shared/settings/prefs.h"
+#include "src/datovka_shared/utility/date_time.h"
 #include "src/global.h"
 #include "src/io/filesystem.h"
 
@@ -215,7 +219,7 @@ fail:
 
 QString nonconflictingFileName(QString filePath)
 {
-	if (filePath.isEmpty()) {
+	if (Q_UNLIKELY(filePath.isEmpty())) {
 		return QString();
 	}
 
@@ -243,21 +247,21 @@ QString nonconflictingFileName(QString filePath)
 enum WriteFileState writeFile(const QString &filePath, const QByteArray &data,
     bool deleteOnError)
 {
-	if (filePath.isEmpty()) {
+	if (Q_UNLIKELY(filePath.isEmpty())) {
 		Q_ASSERT(0);
 		return WF_ERROR;
 	}
 
 	QFile fout(filePath);
-	if (!fout.open(QIODevice::WriteOnly)) {
+	if (Q_UNLIKELY(!fout.open(QIODevice::WriteOnly))) {
 		return WF_CANNOT_CREATE;
 	}
 
-	int written = fout.write(data);
+	qint64 written = fout.write(data);
 	bool flushed = fout.flush();
 	fout.close();
 
-	if ((written != data.size()) || !flushed) {
+	if (Q_UNLIKELY((written != data.size()) || !flushed)) {
 		if (deleteOnError) {
 			QFile::remove(filePath);
 		}
@@ -312,7 +316,7 @@ bool checkDirectoryParent(const QString &parent, const QString &child)
 }
 
 /*!
- * @brief Return astandard location. If the location could not be aquired then
+ * @brief Return standard location. If the location could not be acquired then
  *     temporary location is returned.
  *
  * @param[in] location Location identifier.
@@ -392,7 +396,7 @@ QString createTemporarySubdir(const QString &nameTemplate)
 QString writeTemporaryFile(const QString &fileName, const QByteArray &data,
     bool deleteOnError)
 {
-	if (fileName.isEmpty()) {
+	if (Q_UNLIKELY(fileName.isEmpty())) {
 		Q_ASSERT(0);
 		return QString();
 	}
@@ -402,7 +406,7 @@ QString writeTemporaryFile(const QString &fileName, const QByteArray &data,
 
 	/* StandardLocation::writableLocation(QStandardPaths::TempLocation) ? */
 	QTemporaryFile fout(tempDirPath(true, true) + QDir::separator() + nameCopy);
-	if (!fout.open()) {
+	if (Q_UNLIKELY(!fout.open())) {
 		return QString();
 	}
 	fout.setAutoRemove(false);
@@ -410,11 +414,11 @@ QString writeTemporaryFile(const QString &fileName, const QByteArray &data,
 	/* Get whole path. */
 	QString fullName = fout.fileName();
 
-	int written = fout.write(data);
+	qint64 written = fout.write(data);
 	bool flushed = fout.flush();
 	fout.close();
 
-	if ((written != data.size()) || !flushed) {
+	if (Q_UNLIKELY((written != data.size()) || !flushed)) {
 		if (deleteOnError) {
 			QFile::remove(fullName);
 		}
@@ -424,7 +428,7 @@ QString writeTemporaryFile(const QString &fileName, const QByteArray &data,
 	return fullName;
 }
 
-QString confDirPath(const QString &confSubdir)
+QString confDfltBaseDirPath(const QString &confSubdir)
 {
 #define WIN_PREFIX "AppData/Roaming"
 
@@ -444,7 +448,6 @@ QString confDirPath(const QString &confSubdir)
 	}
 #  endif /* Q_OS_OSX */
 
-	dirPath += QDir::separator() + confSubdir;
 	return dirPath;
 
 #else /* !PORTABLE_APPLICATION */
@@ -466,13 +469,13 @@ QString confDirPath(const QString &confSubdir)
 			    "You don't have permissions to write into '%s'.",
 			    dir.toUtf8().constData());
 		}
-		return dir;
+		return homeDir.path();
 	} else if (homeDir.exists() && QFileInfo(homeDir.path()).isWritable()) {
 		/*
 		 * Default configuration subdirectory in home directory is not
 		 * present but home is writeable.
 		 */
-		return homeDir.path() + QDir::separator() + confSubdir;
+		return homeDir.path();
 	} else {
 		/*
 		 * There is no previous configuration and home directory cannot
@@ -492,15 +495,14 @@ QString confDirPath(const QString &confSubdir)
 		 * Still using confSubdir in order to let the user change
 		 * the configuration via command line.
 		 */
-		return QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) +
-		    QDir::separator() + confSubdir;
+		return QStandardPaths::writableLocation(QStandardPaths::AppDataLocation);
 #  else /* < Qt-5.4 */
 #    warning "Compiling against version < Qt-5.4 which does not provide QStandardPaths::AppDataLocation."
 		/*
 		 * Falling back to default location which is inaccessible.
 		 * If you want to evade this branch then use Qt-5.4 and newer.
 		 */
-		return homeDir.path() + QDir::separator() + confSubdir;
+		return homeDir.path();
 #  endif /* >= Qt-5.4 */
 	}
 
@@ -509,63 +511,139 @@ QString confDirPath(const QString &confSubdir)
 #undef WIN_PREFIX
 }
 
-enum WriteFileState confFileFixBackSlashes(const QString &filePath)
+QString confDirPath(const QString &confBaseDir, const QString &confSubdir)
 {
-	QString fileContent;
-	QFile file(filePath);
-
-	if (file.open(QIODevice::ReadOnly | QIODevice::Text)) {
-		QTextStream in(&file);
-		fileContent = in.readAll();
-		file.close();
+	if (confBaseDir.isEmpty()) {
+		return confDfltBaseDirPath(confSubdir) + QDir::separator() + confSubdir;
 	} else {
-		logErrorNL("Cannot open file '%s' for reading.",
-		    filePath.toUtf8().constData());
-		return WF_CANNOT_READ;
+		return confBaseDir + QDir::separator() + confSubdir;
+	}
+}
+
+QString auxConfFilePath(const QString &filePath)
+{
+	return filePath % QStringLiteral(".aux.")
+	    % QDateTime::currentDateTimeUtc().toString(
+	        Utility::dateTimeFileSuffixFormat);
+}
+
+enum WriteFileState confFileFixBackSlashes(const QString &filePath,
+    bool useAux)
+{
+	QString fileOldContent;
+	QString fileNewContent;
+
+	{
+		QFile file(filePath);
+		if (file.open(QIODevice::ReadOnly | QIODevice::Text)) {
+			QTextStream in(&file);
+			fileOldContent = in.readAll();
+			file.close();
+		} else {
+			logErrorNL("Cannot open file '%s' for reading.",
+			    filePath.toUtf8().constData());
+			return WF_CANNOT_READ;
+		}
 	}
 
-	fileContent.replace(QString("\\"), QString("/"));
+	fileNewContent = fileOldContent;
+	fileNewContent.replace(QString("\\"), QString("/"));
+	if (fileOldContent == fileNewContent) {
+		/* No data changed, no changes to write. */
+		return WF_SUCCESS;
+	}
 
-	if (file.open(QIODevice::WriteOnly | QIODevice::Text)) {
-		QTextStream in(&file);
-		file.reset();
-		in << fileContent;
-		file.close();
-	} else {
-		logErrorNL("Cannot write file '%s'.",
-		    filePath.toUtf8().constData());
-		return WF_CANNOT_WRITE_WHOLE;
+	const QString tmpFilePath =
+	    useAux ? auxConfFilePath(filePath) : filePath;
+
+	{
+		QFile file(tmpFilePath);
+		if (file.open(QIODevice::WriteOnly | QIODevice::Text)) {
+			QTextStream in(&file);
+			file.reset();
+			in << fileNewContent;
+			file.close();
+		} else {
+			logErrorNL("Cannot write file '%s'.",
+			    tmpFilePath.toUtf8().constData());
+			return WF_CANNOT_WRITE_WHOLE;
+		}
+	}
+
+	if (useAux) {
+		if (Q_UNLIKELY(!QFile::remove(filePath))) {
+			logErrorNL("Cannot remove file '%s'.",
+			    filePath.toUtf8().constData());
+			return WF_CANNOT_WRITE_WHOLE;
+		}
+
+		if (Q_UNLIKELY(!QFile::rename(tmpFilePath, filePath))) {
+			logErrorNL("Cannot rename file '%s' to '%s'.",
+			    tmpFilePath.toUtf8().constData(),
+			    filePath.toUtf8().constData());
+			return WF_CANNOT_WRITE_WHOLE;
+		}
 	}
 
 	return WF_SUCCESS;
 }
 
-enum WriteFileState confFileRemovePwdQuotes(const QString &filePath)
+enum WriteFileState confFileRemovePwdQuotes(const QString &filePath,
+    bool useAux)
 {
-	QString fileContent;
-	QFile file(filePath);
+	QString fileOldContent;
+	QString fileNewContent;
 
-	if (file.open(QIODevice::ReadOnly | QIODevice::Text)) {
-		QTextStream in(&file);
-		fileContent = in.readAll();
-		file.close();
-	} else {
-		logErrorNL("Cannot open file '%s' for reading.",
-		    filePath.toUtf8().constData());
-		return WF_CANNOT_READ;
+	{
+		QFile file(filePath);
+		if (file.open(QIODevice::ReadOnly | QIODevice::Text)) {
+			QTextStream in(&file);
+			fileOldContent = in.readAll();
+			file.close();
+		} else {
+			logErrorNL("Cannot open file '%s' for reading.",
+			    filePath.toUtf8().constData());
+			return WF_CANNOT_READ;
+		}
 	}
 
-	fileContent.replace(QString("\""), QString(""));
+	fileNewContent = fileOldContent;
+	fileNewContent.replace(QString("\""), QString(""));
+	if (fileOldContent == fileNewContent) {
+		/* No data changed, no changes to write. */
+		return WF_SUCCESS;
+	}
 
-	if (file.open(QIODevice::WriteOnly | QIODevice::Text)) {
-		QTextStream in(&file);
-		file.reset();
-		in << fileContent;
-		file.close();
-	} else {
-		logErrorNL("Cannot write file '%s'.",
-		    filePath.toUtf8().constData());
-		return WF_CANNOT_WRITE_WHOLE;
+	const QString tmpFilePath =
+	    useAux ? auxConfFilePath(filePath) : filePath;
+
+	{
+		QFile file(tmpFilePath);
+		if (file.open(QIODevice::WriteOnly | QIODevice::Text)) {
+			QTextStream in(&file);
+			file.reset();
+			in << fileNewContent;
+			file.close();
+		} else {
+			logErrorNL("Cannot write file '%s'.",
+			    tmpFilePath.toUtf8().constData());
+			return WF_CANNOT_WRITE_WHOLE;
+		}
+	}
+
+	if (useAux) {
+		if (Q_UNLIKELY(!QFile::remove(filePath))) {
+			logErrorNL("Cannot remove file '%s'.",
+			    filePath.toUtf8().constData());
+			return WF_CANNOT_WRITE_WHOLE;
+		}
+
+		if (Q_UNLIKELY(!QFile::rename(tmpFilePath, filePath))) {
+			logErrorNL("Cannot rename file '%s' to '%s'.",
+			    tmpFilePath.toUtf8().constData(),
+			    filePath.toUtf8().constData());
+			return WF_CANNOT_WRITE_WHOLE;
+		}
 	}
 
 	return WF_SUCCESS;
