@@ -139,12 +139,21 @@ QString writeTemporaryFile(const QString &fileName, const QByteArray &data,
     bool deleteOnError = false);
 
 /*!
- * @brief Returns path to configuration directory.
+ * @brief Returns path to default configuration base directory without \a confSubdir.
  *
  * @param[in] confSubdir Configuration directory name (e.g. ".dsgui").
+ * @return Full path to configuration base directory.
+ */
+QString confDfltBaseDirPath(const QString &confSubdir);
+
+/*!
+ * @brief Returns path to configuration directory.
+ *
+ * @param[in] confBaseDir Configuration base directory name. Ignored when empty.
+ * @param[in] confSubdir Configuration subdirectory name (e.g. ".dsgui").
  * @return Full path to configuration directory.
  */
-QString confDirPath(const QString &confSubdir);
+QString confDirPath(const QString &confBaseDir, const QString &confSubdir);
 
 /*!
  * @brief Generates an auxiliary file name.
