@@ -358,14 +358,6 @@ public:
 	 */
 	QDateTime messageAcceptanceTime(qint64 dmId);
 
-	/*!
-	 * @brief Get Html info about message signature and timestamp.
-	 *
-	 * @param[in] dmId Message identifier.
-	 * @return Html info about message signature and timestamp.
-	 */
-	QString verifySignatureHtml(qint64 dmId);
-
 	/*!
 	 * @brief Return attachments related to given message.
 	 *
@@ -674,6 +666,14 @@ public:
 	 */
 	QByteArray messageAuthorJsonStr(qint64 dmId);
 
+	/*!
+	 * @brief Copy some message data into new single database.
+	 *
+	 * @param[in] dstMsgDbPath Path to destination message database.
+	 * @return True on success.
+	 */
+	bool copyMessageDataToSingleDb(const QString &dstMsgDbPath);
+
 protected: /* These function are used from within a database container. */
 
 	/*!
@@ -949,6 +949,16 @@ protected: /* These function are used from within a database container. */
 	 */
 	SoughtMsg msgsGetMsgDataFromId(const qint64 dmId);
 
+	/*!
+	 * @brief Get message envelope data from identifiers.
+	 *
+	 * @paran[in] msgIds Message identifierd.
+	 * @param[out] soughtMsgEntries Found entries assigned to sought keys.
+	 * @return True on success.
+	 */
+	bool msgsGetMsgDataFromIds(const QList<MsgId> &msgIds,
+	    QHash<MsgId, SoughtMsg> &soughtMsgEntries);
+
 	/*!
 	 * @brief Test if imported message is relevant to account db.
 	 *
@@ -1057,18 +1067,6 @@ private:
 	 */
 	QJsonDocument getMessageCustomData(qint64 dmId);
 
-	/*!
-	 * @brief Check whether message signature was valid at given date
-	 *     (local time).
-	 *
-	 * @param[in] dmId                  Message id.
-	 * @param[in] dateTime              Local time identifier.
-	 * @param[in] ignoreMissingCrlCheck Ignore CRL check if set to true.
-	 * @return True if date check succeeds.
-	 */
-	bool msgCertValidAtDate(qint64 dmId, const QDateTime &dateTime,
-	    bool ignoreMissingCrlCheck = false);
-
 	friend class MessageDbSet;
 	friend class MessageDbSingle;
 };
