@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2024 CZ.NIC
+ * Copyright (C) 2014-2025 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@
 #include <QPair>
 #include <QRegularExpression>
 #include <QSqlDatabase>
+#include <QStringBuilder>
 
 #include "src/datovka_shared/log/log.h"
 #include "src/io/message_db_set.h"
@@ -55,6 +56,8 @@ MessageDbSet::~MessageDbSet(void)
 		MessageDb *db = i.value();
 		db->disconnect(SIGNAL(opened(QString)),
 		    this, SLOT(watchOpened(QString)));
+		db->disconnect(SIGNAL(openingFailed(QString)),
+		    this, SLOT(watchOpeningFailed(QString)));
 		db->disconnect(SIGNAL(changedLocallyRead(QList<MsgId>, bool)),
 		    this, SLOT(watchChangedLocallyRead(QList<MsgId>, bool)));
 		db->disconnect(SIGNAL(changedProcessState(QList<MsgId>, enum MessageDb::MessageProcessState)),
@@ -68,7 +71,8 @@ MessageDbSet::~MessageDbSet(void)
 }
 
 bool MessageDbSet::openLocation(const QString &newLocDir,
-   enum Organisation organisation, enum CreationManner manner)
+    enum Organisation organisation, enum CreationManner creationManner,
+    enum AbandonManner abandonManner)
 {
 	QStringList matchingFiles;
 
@@ -116,7 +120,7 @@ bool MessageDbSet::openLocation(const QString &newLocDir,
 	matchingFiles = existingDbFileNamesInLocation(newLocDir,
 	    m_primaryKey, m_testing, organisation, true);
 
-	if (Q_UNLIKELY((manner == CM_MUST_EXIST) && matchingFiles.isEmpty())) {
+	if (Q_UNLIKELY((creationManner == CM_MUST_EXIST) && matchingFiles.isEmpty())) {
 		logErrorNL("No matching database file for '%s' in '%s'.",
 		    m_primaryKey.toUtf8().constData(),
 		    newLocDir.toUtf8().constData());
@@ -137,7 +141,7 @@ bool MessageDbSet::openLocation(const QString &newLocDir,
 	/* There may be no matching files. */
 
 	/* Check primary keys. */
-	foreach (const QString &fileName, matchingFiles) {
+	for (const QString &fileName : matchingFiles) {
 		QString secondaryKey = secondaryKeyFromFileName(
 		    fileName, organisation);
 
@@ -148,13 +152,14 @@ bool MessageDbSet::openLocation(const QString &newLocDir,
 		}
 	}
 
+	const QString oldLocDir = m_locDir;
 	m_locDir = newLocDir;
 	m_organisation = organisation;
 
 	MessageDb *db = Q_NULLPTR;
 	if (matchingFiles.size()) {
 		/* Load files that have been found. */
-		foreach (const QString &fileName, matchingFiles) {
+		for (const QString &fileName : matchingFiles) {
 			QString secondaryKey = secondaryKeyFromFileName(
 			    fileName, organisation);
 			Q_ASSERT(!secondaryKey.isNull());
@@ -166,7 +171,7 @@ bool MessageDbSet::openLocation(const QString &newLocDir,
 				/* TODO -- How can be this operation aborted? */
 			}
 		}
-	} else if (manner == CM_CREATE_EMPTY_CURRENT) {
+	} else if (creationManner == CM_CREATE_EMPTY_CURRENT) {
 		/* Create empty file matching current date. */
 		QString secKey = secondaryKey(QDateTime::currentDateTime());
 		Q_ASSERT(!secKey.isNull());
@@ -181,6 +186,11 @@ bool MessageDbSet::openLocation(const QString &newLocDir,
 		}
 	}
 
+	if (abandonManner == AM_DELETE) {
+		deleteRelatedInLocation(oldLocDir,
+		    m_primaryKey, m_testing, DO_UNKNOWN);
+	}
+
 	return true;
 }
 
@@ -330,7 +340,8 @@ bool MessageDbSet::moveToLocation(const QString &newLocDir)
 }
 
 bool MessageDbSet::reopenLocation(const QString &newLocDir,
-    enum Organisation organisation, enum CreationManner manner)
+    enum Organisation organisation, enum CreationManner creationManner,
+    enum AbandonManner abandonManner)
 {
 	if (Q_UNLIKELY(m_organisation == DO_UNKNOWN)) {
 		return false;
@@ -362,17 +373,18 @@ bool MessageDbSet::reopenLocation(const QString &newLocDir,
 	/* Remove all possible database files from new location. */
 	QStringList impedingFiles = existingDbFileNamesInLocation(newLocDir,
 	    m_primaryKey, m_testing, DO_UNKNOWN, false);
-	foreach (const QString &fileName, impedingFiles) {
+	for (const QString &fileName : impedingFiles) {
 		const QString name = newLocDir + QDir::separator() + fileName;
 		QFile::remove(name);
 		QDir(MessageDb::_assocFileDirPath(name)).removeRecursively();
 	}
 
+	const QString oldLocDir = m_locDir;
 	m_locDir = newLocDir;
 	m_organisation = organisation;
 
 	MessageDb *db = Q_NULLPTR;
-	if (manner == CM_CREATE_EMPTY_CURRENT) {
+	if (creationManner == CM_CREATE_EMPTY_CURRENT) {
 		/* Create empty file matching current date. */
 		QString secKey = secondaryKey(QDateTime::currentDateTime());
 		Q_ASSERT(!secKey.isNull());
@@ -387,6 +399,11 @@ bool MessageDbSet::reopenLocation(const QString &newLocDir,
 		}
 	}
 
+	if (abandonManner == AM_DELETE) {
+		deleteRelatedInLocation(oldLocDir,
+		    m_primaryKey, m_testing, DO_UNKNOWN);
+	}
+
 	return true;
 }
 
@@ -462,6 +479,59 @@ QString MessageDbSet::secondaryKey(const QDateTime &time) const
 	}
 }
 
+/*!
+ * @brief Returns only years fitting the supplied date range.
+ *
+ * @param[in] fromTime Start time, ignored when invalid.
+ * @param[in] toTime End time, ignored when converted.
+ * @return List of year keys or empty list on error. Returns all
+ *     years when both limits are invalid.
+ */
+static
+QStringList yearsInRange(const QStringList &years, const QDateTime &fromTime,
+    const QDateTime &toTime)
+{
+	QStringList matchingYears;
+	const bool fromValid = fromTime.isValid();
+	const int fromYear = fromValid ? fromTime.toString("yyyy").toInt() : -1;
+	const bool toValid = toTime.isValid();
+	const int toYear = toValid ? toTime.toString("yyyy").toInt() : -1;
+
+	for (const QString &yearStr : years) {
+		bool iOk = false;
+		const int year = yearStr.toInt(&iOk);
+		if (Q_UNLIKELY(!iOk)) {
+			continue;
+		}
+		if ((fromValid && (year < fromYear))
+		    || (toValid && (year > toYear))) {
+			continue;
+		}
+
+		matchingYears.append(
+		    QStringLiteral("%1").arg(year, 4, 10, QLatin1Char('0')));
+	}
+
+	return matchingYears;
+}
+
+QStringList MessageDbSet::secondaryKeys(const QDateTime &fromTime,
+    const QDateTime &toTime) const
+{
+	switch (m_organisation) {
+	case DO_SINGLE_FILE:
+		return {secondaryKeySingleFile(fromTime)};
+		break;
+	case DO_YEARLY:
+		return yearsInRange(this->keys(), fromTime, toTime);
+		break;
+	case DO_UNKNOWN:
+	default:
+		Q_ASSERT(0);
+		return QStringList();
+	}
+}
+
 MessageDb *MessageDbSet::constAccessMessageDb(
     const QDateTime &deliveryTime) const
 {
@@ -619,6 +689,41 @@ bool MessageDbSet::changePrimaryKey(const QString &newPrimaryKey)
 	return true;
 }
 
+/*!
+ * @brief Using explicit enumeration values to prevent accidental usage of
+ *     wrong values.
+ */
+enum DbStorageType {
+	DST_FILE, /* Database file. */
+	DST_DIR /* Accopmanying directory. */
+};
+
+/*!
+ * @brief Constructs a database file/directory name.
+ *
+ * @param[in] primaryKey Usually user name.
+ * @param[in] testing True if testing account.
+ * @param[in] storageType Name for file or related directory.
+ * @teturn File or directory name.
+ */
+static inline
+QString singleFileDbName(const QString &primaryKey, bool testing,
+   enum DbStorageType storageType)
+{
+	static const QString underscores(DB_UNDERSCORES);
+	static const QString markerProd(DB_MARKER_PROD);
+	static const QString markerTest(DB_MARKER_TEST);
+	static const QString dbSuff(DB_SUFFIX);
+
+	if (DST_FILE == storageType) {
+		return primaryKey % underscores
+		    % (testing ? markerTest : markerProd) % dbSuff;
+	} else {
+		return primaryKey % underscores
+		    % (testing ? markerTest : markerProd);
+	}
+}
+
 /*!
  * @brief Returns true if file name matches single file naming conventions.
  *
@@ -631,10 +736,7 @@ static
 bool fileNameMatchesSingleFile(const QString &fileName,
     const QString &primaryKey, bool testing)
 {
-	QString constructed(
-	    primaryKey + "___" + (testing ? "1" : "0") + DB_SUFFIX);
-
-	return fileName == constructed;
+	return fileName == singleFileDbName(primaryKey, testing, DST_FILE);
 }
 
 /*!
@@ -649,10 +751,66 @@ static
 bool dirNameMatchesSingleFile(const QString &dirName,
     const QString &primaryKey, bool testing)
 {
-	QString constructed(
-	    primaryKey + "___" + (testing ? "1" : "0"));
+	return dirName == singleFileDbName(primaryKey, testing, DST_DIR);
+}
 
-	return dirName == constructed;
+/*!
+ * @brief Constructs a yearly-organised database file/directory name matching
+ *     expression.
+ *
+ * @param[in] primaryKey Usually user name.
+ * @param[in] testing True if testing account.
+ * @param[in] storageType Name for file or related directory.
+ * @teturn File or directory matching regular expression.
+ */
+static inline
+QRegularExpression yearlyFileDbRe(const QString &primaryKey, bool testing,
+   enum DbStorageType storageType)
+{
+	static const QString lineStart("^");
+	static const QString yearSection("_" YEARLY_SEC_KEY_RE DB_UNDERSCORES);
+	static const QString markerProd(DB_MARKER_PROD);
+	static const QString markerTest(DB_MARKER_TEST);
+	static const QString dbSuff(DB_SUFFIX);
+	static const QString lineEnd("$");
+
+	if (DST_FILE == storageType) {
+		return QRegularExpression(lineStart % primaryKey % yearSection
+		    % (testing ? markerTest : markerProd) % dbSuff % lineEnd);
+	} else {
+		return QRegularExpression(lineStart % primaryKey % yearSection
+		    % (testing ? markerTest : markerProd) % lineEnd);
+	}
+}
+
+/*!
+ * @brief Constructs an invalid database file/directory name.
+ *
+ * @param[in] primaryKey Usually user name.
+ * @param[in] testing True if testing account.
+ * @param[in] storageType Name for file or related directory.
+ * @teturn File or directory name.
+ */
+static inline
+QString invalidFileDbName(const QString &primaryKey, bool testing,
+   enum DbStorageType storageType)
+{
+	static const QString lineStart("^");
+	static const QString underscore("_");
+	static const QString invalid(YEARLY_SEC_KEY_INVALID);
+	static const QString underscores(DB_UNDERSCORES);
+	static const QString markerProd(DB_MARKER_PROD);
+	static const QString markerTest(DB_MARKER_TEST);
+	static const QString dbSuff(DB_SUFFIX);
+	static const QString lineEnd("$");
+
+	if (DST_FILE == storageType) {
+		return primaryKey % underscore % invalid % underscores
+		    % (testing ? markerTest : markerProd) % dbSuff;
+	} else {
+		return primaryKey % underscore % invalid % underscores
+		    % (testing ? markerTest : markerProd);
+	}
 }
 
 /*!
@@ -667,13 +825,8 @@ static
 bool fileNameMatchesYearly(const QString &fileName, const QString &primaryKey,
     bool testing)
 {
-	const QRegularExpression re("^" + primaryKey + "_" YEARLY_SEC_KEY_RE
-	    "___" + (testing ? "1" : "0") + DB_SUFFIX "$");
-
-	QString invFileName(primaryKey + "_" + YEARLY_SEC_KEY_INVALID +
-	    "___" + (testing ? "1" : "0") + DB_SUFFIX);
-
-	return re.match(fileName).hasMatch() || (fileName == invFileName);
+	return yearlyFileDbRe(primaryKey, testing, DST_FILE).match(fileName).hasMatch()
+	    || (fileName == invalidFileDbName(primaryKey, testing, DST_FILE));
 }
 
 /*!
@@ -688,13 +841,8 @@ static
 bool dirNameMatchesYearly(const QString &dirName, const QString &primaryKey,
     bool testing)
 {
-	const QRegularExpression re("^" + primaryKey + "_" YEARLY_SEC_KEY_RE
-	    "___" + (testing ? "1" : "0") + "$");
-
-	QString invDirName(primaryKey + "_" + YEARLY_SEC_KEY_INVALID +
-	    "___" + (testing ? "1" : "0"));
-
-	return re.match(dirName).hasMatch() || (dirName == invDirName);
+	return yearlyFileDbRe(primaryKey, testing, DST_DIR).match(dirName).hasMatch()
+	    || (dirName == invalidFileDbName(primaryKey, testing, DST_DIR));
 }
 
 QStringList MessageDbSet::fileNamesAvailable(const QString &locDir,
@@ -913,7 +1061,7 @@ static
 QString fileNameSecondaryKeySingleFile(const QString &fileName)
 {
 	static const QRegularExpression re(
-	    QString("^") + PRIMARY_KEY_RE "___" "[01]" DB_SUFFIX "$");
+	    "^" PRIMARY_KEY_RE DB_UNDERSCORES "[01]" DB_SUFFIX "$");
 
 	return re.match(fileName).hasMatch() ? SINGLE_FILE_SEC_KEY : QString();
 }
@@ -928,14 +1076,14 @@ static
 QString fileNameSecondaryKeyYearly(const QString &fileName)
 {
 	static const QRegularExpression reInv(QString("^") + PRIMARY_KEY_RE
-	    "_" + YEARLY_SEC_KEY_INVALID + "___" "[01]" DB_SUFFIX "$");
+	    "_" + YEARLY_SEC_KEY_INVALID + DB_UNDERSCORES "[01]" DB_SUFFIX "$");
 
 	if (reInv.match(fileName).hasMatch()) {
 		return YEARLY_SEC_KEY_INVALID;
 	}
 
-	static const QRegularExpression reValid(QString("^") + PRIMARY_KEY_RE
-	    "_" YEARLY_SEC_KEY_RE "___" "[01]" DB_SUFFIX "$");
+	static const QRegularExpression reValid("^" PRIMARY_KEY_RE
+	    "_" YEARLY_SEC_KEY_RE DB_UNDERSCORES "[01]" DB_SUFFIX "$");
 
 	if (reValid.match(fileName).hasMatch()) {
 		return fileName.section('_', 1, 1);
@@ -1071,7 +1219,7 @@ QString MessageDbSet::constructDbFileName(const QString &locDir,
 	}
 
 	return locDir + QDir::separator() +
-	    key + QString("___") + (testing ? "1" : "0") + DB_SUFFIX;
+	    key + QString(DB_UNDERSCORES) + (testing ? "1" : "0") + DB_SUFFIX;
 }
 
 bool MessageDbSet::isValidDbFileName(const QString &fileName,
@@ -1083,7 +1231,7 @@ bool MessageDbSet::isValidDbFileName(const QString &fileName,
 	dbYear.clear();
 	errMsg.clear();
 
-	if (fileName.contains("___")) {
+	if (fileName.contains(DB_UNDERSCORES)) {
 		// get username from filename
 		fileNameParts = fileName.split("_");
 		if (fileNameParts.isEmpty() || fileNameParts.count() <= 1) {
@@ -1124,7 +1272,7 @@ bool MessageDbSet::isValidDbFileName(const QString &fileName,
 			        .arg(fileName);
 			return ret;
 		}
-		fileNameParts = fileNameParts[0].split("___");
+		fileNameParts = fileNameParts[0].split(DB_UNDERSCORES);
 		if (fileNameParts.isEmpty()) {
 			errMsg = tr(
 			    "File '%1' does not contain a valid database filename.")
@@ -1157,28 +1305,33 @@ void MessageDbSet::watchOpened(const QString &fileName)
 {
 	Q_UNUSED(fileName);
 
-	emit opened(m_primaryKey, m_testing);
+	Q_EMIT opened(m_primaryKey, m_testing);
+}
+
+void MessageDbSet::watchOpeningFailed(const QString &fileName)
+{
+	Q_EMIT openingFailed(m_primaryKey, m_testing, fileName);
 }
 
 void MessageDbSet::watchChangedLocallyRead(const QList<MsgId> &msgIds, bool read)
 {
-	emit changedLocallyRead(m_primaryKey, m_testing, msgIds, read);
+	Q_EMIT changedLocallyRead(m_primaryKey, m_testing, msgIds, read);
 }
 
 void MessageDbSet::watchChangedProcessState(const QList<MsgId> &msgIds,
     enum MessageDb::MessageProcessState state)
 {
-	emit changedProcessState(m_primaryKey, m_testing, msgIds, state);
+	Q_EMIT changedProcessState(m_primaryKey, m_testing, msgIds, state);
 }
 
 void MessageDbSet::watchMessageInserted(const MsgId &msgId, int direction)
 {
-	emit messageInserted(m_primaryKey, m_testing, msgId, direction);
+	Q_EMIT messageInserted(m_primaryKey, m_testing, msgId, direction);
 }
 
 void MessageDbSet::watchMessageDataDeleted(const MsgId &msgId)
 {
-	emit messageDataDeleted(m_primaryKey, m_testing, msgId);
+	Q_EMIT messageDataDeleted(m_primaryKey, m_testing, msgId);
 }
 
 MessageDb *MessageDbSet::_accessMessageDb(const QString &secondaryKey,
@@ -1265,6 +1418,8 @@ MessageDb *MessageDbSet::_accessMessageDb(const QString &secondaryKey,
 
 	connect(db, SIGNAL(opened(QString)),
 	    this, SLOT(watchOpened(QString)));
+	connect(db, SIGNAL(openingFailed(QString)),
+	    this, SLOT(watchOpeningFailed(QString)));
 	connect(db, SIGNAL(changedLocallyRead(QList<MsgId>, bool)),
 	    this, SLOT(watchChangedLocallyRead(QList<MsgId>, bool)));
 	connect(db, SIGNAL(changedProcessState(QList<MsgId>, enum MessageDb::MessageProcessState)),
@@ -1275,10 +1430,48 @@ MessageDb *MessageDbSet::_accessMessageDb(const QString &secondaryKey,
 	    this, SLOT(watchMessageDataDeleted(MsgId)));
 
 	this->insert(secondaryKey, db);
-	emit madeAvailable(m_primaryKey, m_testing);
+	Q_EMIT madeAvailable(m_primaryKey, m_testing);
 	return db;
 }
 
+void MessageDbSet::deleteRelatedInLocation(const QString &locDir,
+    const QString &primaryKey, bool testing, enum Organisation organisation)
+{
+	QStringList deletedFiles;
+	QStringList deletedDirs;
+
+	/* Only existing database file names. */
+	const QStringList abandonedFileNames = existingDbFileNamesInLocation(
+	    locDir, primaryKey, testing, organisation, false);
+
+	/* Get full paths to files and existing directories. */
+	for (const QString &fileName : abandonedFileNames) {
+		QString filePath = locDir % QDir::separator() % fileName;
+		QString dirPath = MessageDb::_assocFileDirPath(filePath);
+
+		deletedFiles.append(filePath);
+		if (QFileInfo(dirPath).isDir()) {
+			deletedDirs.append(dirPath);
+		}
+	}
+
+	/* Delete files. */
+	for (const QString &filePath : deletedFiles) {
+		if (Q_UNLIKELY(!QFile::remove(filePath))) {
+			logErrorNL("Failed to delete database file '%s'.",
+			    filePath.toUtf8().constData());
+		}
+	}
+
+	/* Delete directories. */
+	for (const QString &dirPath : deletedDirs) {
+		if (Q_UNLIKELY(!QDir(dirPath).removeRecursively())) {
+			logErrorNL("Failed to delete database directory '%s'.",
+			    dirPath.toUtf8().constData());
+		}
+	}
+}
+
 enum MessageDbSet::Error MessageDbSet::checkGivenDbFile(const QString &filePath,
     AccessFlags flags)
 {
