@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2024 CZ.NIC
+ * Copyright (C) 2014-2025 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 
 #include <QDateTime>
 #include <QMap>
+#include <QHash>
 #include <QObject>
 #include <QString>
 #include <QStringList>
@@ -32,6 +33,9 @@
 #include "src/identifiers/message_id.h"
 #include "src/io/message_db.h"
 
+#define DB_UNDERSCORES "___"
+#define DB_MARKER_PROD "0" /* Production environment data. */
+#define DB_MARKER_TEST "1" /* Testing environment data. */
 #define DB_SUFFIX ".db"
 #define PRIMARY_KEY_RE "[^_]+"
 #define SINGLE_FILE_SEC_KEY ""
@@ -98,7 +102,15 @@ public:
 	};
 
 	/*!
-	 *  When gathering database file sizes.
+	 * What to do with the abandoned files.
+	 */
+	enum AbandonManner {
+		AM_LEAVE, /*!< Leave abandoned files. */
+		AM_DELETE /*!< Delete abandoned files. */
+	};
+
+	/*!
+	 * When gathering database file sizes.
 	 */
 	enum SizeComputation {
 		SC_SUM, /*!< Compute the sum of all available database files. */
@@ -113,18 +125,20 @@ public:
 	~MessageDbSet(void);
 
 	/*!
-	 * @brief Opens a location. Old file(s) is(are) left untouched.
+	 * @brief Opens a location.
 	 *     Files in new location are opened if any exist.
 	 *
 	 * @note The new location must exist.
 	 *
-	 * @param[in] newLocDir    New location directory.
+	 * @param[in] newLocDir New location directory.
 	 * @paran[in] organisation Organisation type.
-	 * @param[in] manner       How to treat files when opening database.
+	 * @param[in] creationManner How to treat files when opening database.
+	 * @param[in] abandonManner How to treat closed files.
 	 * @reurn True if database was opened.
 	 */
 	bool openLocation(const QString &newLocDir,
-	   enum Organisation organisation, enum CreationManner manner);
+	   enum Organisation organisation, enum CreationManner creationManner,
+	   enum AbandonManner abandonManner);
 
 	/*!
 	 * @brief Creates a copy of the current database into a given new
@@ -144,21 +158,23 @@ public:
 	bool moveToLocation(const QString &newLocDir);
 
 	/*!
-	 * @brief Re-open a new empty database file. The old file(s) is(are)
-	 *     left untouched. The new location is emptied.
+	 * @brief Re-open a new empty database file.
+	 *     The new location is emptied.
 	 *
 	 * @note The CM_MUST_EXIST value of `manner` parameter is ignored.
 	 *
-	 * @param[in] newLocDir    New location directory.
+	 * @param[in] newLocDir New location directory.
 	 * @paran[in] organisation Organisation type.
-	 * @param[in] manner       How to treat files when opening database.
+	 * @param[in] creationManner How to treat files when opening database.
+	 * @param[in] abandonManner How to treat closed files.
 	 * @return True if database was re-opened.
 	 */
 	bool reopenLocation(const QString &newLocDir,
-	    enum Organisation organisation, enum CreationManner manner);
+	    enum Organisation organisation, enum CreationManner creationManner,
+	    enum AbandonManner abandonManner);
 
 	/*!
-	 * @brief Delete associated message db files in location.
+	 * @brief Delete associated message db files in currently used location.
 	 *
 	 * @return True on success.
 	 */
@@ -168,11 +184,23 @@ public:
 	 * @brief Computes the secondary key from supplied date according to
 	 *     container settings.
 	 *
-	 * @param time Time to be converted.
+	 * @param[in] time Time to be converted.
 	 * @return Secondary key or null string on error.
 	 */
 	QString secondaryKey(const QDateTime &time) const;
 
+	/*!
+	 * @brief Computes the secondary key fitting the supplied date range
+	 *     according to container settings.
+	 *
+	 * @param[in] fromTime Start time, ignored when invalid.
+	 * @param[in] toTime End time, ignored when converted.
+	 * @return List of secondary keys or empty list on error. Returns all
+	 *     secondary keys when both limits are invalid.
+	 */
+	QStringList secondaryKeys(const QDateTime &fromTime,
+	    const QDateTime &toTime) const;
+
 	/*!
 	 * @brief Access already existent message database.
 	 *
@@ -382,7 +410,7 @@ public:
 	bool isValidDbFileName(const QString &fileName, QString &dbUserName,
 	    QString &dbYear, bool &dbTestingFlag, QString &errMsg);
 
-signals:
+Q_SIGNALS:
 	/*!
 	 * @brief Emitted when a database file has been added into the container.
 	 *
@@ -401,6 +429,16 @@ signals:
 	 */
 	void opened(const QString &primaryKey, bool testing);
 
+	/*!
+	 * @brief Emitted when the attempt to open a database file fails.
+	 *
+	 * @param[in] primaryKey Primary key, usually the username.
+	 * @param[in] testing Whether the database is related to a testing account.
+	 * @param[in] fileName Database file name.
+	 */
+	void openingFailed(const QString &primaryKey, bool testing,
+	    const QString &fileName);
+
 	/*!
 	 * @brief Emitted when the locally read flag changes.
 	 *
@@ -445,7 +483,7 @@ signals:
 	void messageDataDeleted(const QString &primaryKey, bool testing,
 	    const MsgId &msgId);
 
-private slots:
+private Q_SLOTS:
 	/*!
 	 * @brief This slot must be connected to every database created inside
 	 *     this container.
@@ -454,6 +492,14 @@ private slots:
 	 */
 	void watchOpened(const QString &fileName);
 
+	/*!
+	 * @brief This slot must be connected to every database created inside
+	 *     this container.
+	 *
+	 * @param[in] fileName Database file name.
+	 */
+	void watchOpeningFailed(const QString &fileName);
+
 	/*!
 	 * @brief This slot must be connected to every database created inside
 	 *     this container.
@@ -515,6 +561,24 @@ private:
 	 */
 	MessageDb *_accessMessageDb(const QString &secondaryKey, bool create);
 
+	/*!
+	 * @brief Deletes the related database file/directories stored in the
+	 *     location.
+	 *
+	 * @note If DO_UNKNOWN is passed via `organisation` then all possible
+	 *     matching files and directories are deleted.
+	 *
+	 * @param[in] locDir Directory where the database should reside.
+	 * @param[in] primaryKey Primary key.
+	 * @param[in] testing True if a testing account.
+	 * @paran[in] organisation Organisation type.
+	 * @return List of database files in the location.
+	 */
+	static
+	void deleteRelatedInLocation(const QString &locDir,
+	    const QString &primaryKey, bool testing,
+	    enum Organisation organisation);
+
 	/*!
 	 * @brief Test given file.
 	 *
@@ -551,9 +615,12 @@ public: /* Database function that have been delegated to the container. */
 	/*!
 	 * @brief Return sum of sizes of all managed databases;
 	 *
+	 * @param[in] forceAccess Whether to connect all databases before
+	 *                        querying their size.
+	 *
 	 * @return Size in bytes or -1 on error.
 	 */
-	qint64 dbSize(void);
+	qint64 dbSize(bool forceAccess = false);
 
 	/*!
 	 * @brief Returns the size of the underlying database files.
@@ -737,7 +804,7 @@ public: /* Database function that have been delegated to the container. */
 	 * @brief Return contacts from message db.
 	 *
 	 * @return List of vectors containing recipientId, recipientName,
-	 *     recipentAddress.
+	 *     recipientAddress.
 	 */
 	QList<MessageDb::ContactEntry> uniqueContacts(void) const;
 
@@ -791,11 +858,13 @@ public: /* Database function that have been delegated to the container. */
 	 * @param[in] attachPhrase Phrase to search in attachment names.
 	 * @param[in] logicalAnd Set to true if found messages should match all criteria,
 	 *                       set to false if found messages should match any criteria.
+	 * @param[in] fromDate Start date. Use invalid value to ignore this limit.
+	 * @param[in] toDate Stop date. Use invalid value to ignore this limit.
 	 * @return Found message data.
 	 */
 	QList<MessageDb::SoughtMsg> msgsSearch(const Isds::Envelope &envel,
 	    enum MessageDirection msgDirect, const QString &attachPhrase,
-	    bool logicalAnd) const;
+	    bool logicalAnd, const QDate &fromDate, const QDate &toDate) const;
 
 	/*!
 	 * @brief Get message envelope data from id.
@@ -804,6 +873,9 @@ public: /* Database function that have been delegated to the container. */
 	 */
 	MessageDb::SoughtMsg msgsGetMsgDataFromId(const qint64 msgId) const;
 
+	bool msgsGetMsgDataFromIds(const QList<MsgId> &msgIds,
+	    QHash<MsgId, MessageDb::SoughtMsg> &soughtMsgEntries) const;
+
 private:
 	/*!
 	 * @brief Return list of seconday keys that may be involed in last 90
@@ -892,9 +964,12 @@ private:
 	inline QList<MsgId> _sf_msgsDateInterval(const QDate &fromDate, const QDate &toDate, enum MessageDb::MessageType messageType) const;
 	inline QList<MsgId> _yrly_msgsDateInterval(const QDate &fromDate, const QDate &toDate, enum MessageDb::MessageType messageType) const;
 
-	inline QList<MessageDb::SoughtMsg> _sf_msgsSearch(const Isds::Envelope &envel, enum MessageDirection msgDirect, const QString &attachPhrase, bool logicalAnd) const;
-	inline QList<MessageDb::SoughtMsg> _yrly_msgsSearch(const Isds::Envelope &envel, enum MessageDirection msgDirect, const QString &attachPhrase, bool logicalAnd) const;
+	inline QList<MessageDb::SoughtMsg> _sf_msgsSearch(const Isds::Envelope &envel, enum MessageDirection msgDirect, const QString &attachPhrase, bool logicalAnd, const QDate &fromDate, const QDate &toDate) const;
+	inline QList<MessageDb::SoughtMsg> _yrly_msgsSearch(const Isds::Envelope &envel, enum MessageDirection msgDirect, const QString &attachPhrase, bool logicalAnd, const QDate &fromDate, const QDate &toDate) const;
 
 	inline MessageDb::SoughtMsg _sf_msgsGetMsgDataFromId(const qint64 msgId) const;
 	inline MessageDb::SoughtMsg _yrly_msgsGetMsgDataFromId(const qint64 msgId) const;
+
+	inline bool _sf_msgsGetMsgDataFromIds(const QList<MsgId> &msgIds, QHash<MsgId, MessageDb::SoughtMsg> &soughtMsgEntries) const;
+	inline bool _yrly_msgsGetMsgDataFromIds(const QList<MsgId> &msgIds, QHash<MsgId, MessageDb::SoughtMsg> &soughtMsgEntries) const;
 };
