@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2024 CZ.NIC
+ * Copyright (C) 2014-2025 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -87,7 +87,7 @@ bool MessageDbSet::backup(const QString &dirName)
 	return ret;
 }
 
-qint64 MessageDbSet::dbSize(void)
+qint64 MessageDbSet::dbSize(bool forceAccess)
 {
 	qint64 sum = 0;
 
@@ -98,6 +98,9 @@ qint64 MessageDbSet::dbSize(void)
 			Q_ASSERT(0);
 			return -1;
 		}
+		if (forceAccess) {
+			db->accessDb();
+		}
 		qint64 size = db->dbSize();
 		if (Q_UNLIKELY(size < 0)) {
 			return -1;
@@ -1640,32 +1643,41 @@ QList<MsgId> MessageDbSet::msgsDateInterval(const QDate &fromDate,
 
 QList<MessageDb::SoughtMsg> MessageDbSet::_sf_msgsSearch(
     const Isds::Envelope &envel, enum MessageDirection msgDirect,
-    const QString &attachPhrase, bool logicalAnd) const
+    const QString &attachPhrase, bool logicalAnd, const QDate &fromDate,
+    const QDate &toDate) const
 {
 	if (Q_UNLIKELY(this->size() == 0)) {
 		return QList<MessageDb::SoughtMsg>();
 	}
 	Q_ASSERT(this->size() == 1);
 	return this->first()->msgsSearch(envel, msgDirect, attachPhrase,
-	    logicalAnd);
+	    logicalAnd, fromDate, toDate);
 }
 
 QList<MessageDb::SoughtMsg> MessageDbSet::_yrly_msgsSearch(
     const Isds::Envelope &envel, enum MessageDirection msgDirect,
-    const QString &attachPhrase, bool logicalAnd) const
+    const QString &attachPhrase, bool logicalAnd, const QDate &fromDate,
+    const QDate &toDate) const
 {
 	QList<MessageDb::SoughtMsg> msgs;
 
-	for (QMap<QString, MessageDb *>::const_iterator i = this->begin();
-	     i != this->end(); ++i) {
-		MessageDb *db = i.value();
+	const QStringList secKeys = secondaryKeys(
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
+	    fromDate.startOfDay(), toDate.startOfDay()
+#else /* < Qt-5.14.0 */
+	    QDateTime(fromDate), QDateTime(toDate)
+#endif /* >= Qt-5.14.0 */
+	    );
+
+	for (const QString &secKey : secKeys) {
+		MessageDb *db = this->value(secKey, Q_NULLPTR);
 		if (Q_UNLIKELY(Q_NULLPTR == db)) {
 			Q_ASSERT(0);
 			return QList<MessageDb::SoughtMsg>();
 		}
 
 		msgs.append(db->msgsSearch(envel, msgDirect, attachPhrase,
-		    logicalAnd));
+		    logicalAnd, fromDate, toDate));
 	}
 
 	return msgs;
@@ -1673,16 +1685,17 @@ QList<MessageDb::SoughtMsg> MessageDbSet::_yrly_msgsSearch(
 
 QList<MessageDb::SoughtMsg> MessageDbSet::msgsSearch(
     const Isds::Envelope &envel, enum MessageDirection msgDirect,
-    const QString &attachPhrase, bool logicalAnd) const
+    const QString &attachPhrase, bool logicalAnd, const QDate &fromDate,
+    const QDate &toDate) const
 {
 	switch (m_organisation) {
 	case DO_SINGLE_FILE:
 		return _sf_msgsSearch(envel, msgDirect, attachPhrase,
-		    logicalAnd);
+		    logicalAnd, fromDate, toDate);
 		break;
 	case DO_YEARLY:
 		return _yrly_msgsSearch(envel, msgDirect, attachPhrase,
-		    logicalAnd);
+		    logicalAnd, fromDate, toDate);
 		break;
 	default:
 		Q_ASSERT(0);
@@ -1740,3 +1753,74 @@ MessageDb::SoughtMsg MessageDbSet::msgsGetMsgDataFromId(
 
 	return MessageDb::SoughtMsg();
 }
+
+bool MessageDbSet::_sf_msgsGetMsgDataFromIds(const QList<MsgId> &msgIds,
+    QHash<MsgId, MessageDb::SoughtMsg> &soughtMsgEntries) const
+{
+	if (Q_UNLIKELY(this->size() == 0)) {
+		soughtMsgEntries.clear();
+		return false;
+	}
+	Q_ASSERT(this->size() == 1);
+	return this->first()->msgsGetMsgDataFromIds(msgIds, soughtMsgEntries);
+}
+
+bool MessageDbSet::_yrly_msgsGetMsgDataFromIds(const QList<MsgId> &msgIds,
+    QHash<MsgId, MessageDb::SoughtMsg> &soughtMsgEntries) const
+{
+	/* Sort identifiers acorind to secondary keys. */
+	QHash<QString, QList<MsgId> > queriedIds;
+	for (const MsgId &msgId : msgIds) {
+		queriedIds[secondaryKey(msgId.deliveryTime())].append(msgId);
+	}
+
+	QHash<MsgId, MessageDb::SoughtMsg> collectedEntries;
+
+	for (const QString &secKey : queriedIds.keys()) {
+		MessageDb *db = this->value(secKey, Q_NULLPTR);
+		if (Q_UNLIKELY(Q_NULLPTR == db)) {
+			soughtMsgEntries.clear();
+			return false;
+		}
+
+		QHash<MsgId, MessageDb::SoughtMsg> foundEntries;
+		if (db->msgsGetMsgDataFromIds(queriedIds[secKey], foundEntries)) {
+
+			/* Append found entries into collected. */
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 15, 0))
+			collectedEntries.insert(foundEntries);
+#else /* < Qt-5.15.0 */
+			for (QHash<MsgId, MessageDb::SoughtMsg>::const_iterator i = foundEntries.cbegin();
+			     i != foundEntries.cend(); ++i) {
+				collectedEntries.insert(i.key(), i.value());
+			}
+#endif /* >= Qt-5.15.0 */
+
+		} else {
+			soughtMsgEntries.clear();
+			return false;
+		}
+	}
+
+	soughtMsgEntries = macroStdMove(collectedEntries);
+	return true;
+}
+
+bool MessageDbSet::msgsGetMsgDataFromIds(const QList<MsgId> &msgIds,
+    QHash<MsgId, MessageDb::SoughtMsg> &soughtMsgEntries) const
+{
+	switch (m_organisation) {
+	case DO_SINGLE_FILE:
+		return _sf_msgsGetMsgDataFromIds(msgIds, soughtMsgEntries);
+		break;
+	case DO_YEARLY:
+		return _yrly_msgsGetMsgDataFromIds(msgIds, soughtMsgEntries);
+		break;
+	default:
+		Q_ASSERT(0);
+		break;
+	}
+
+	soughtMsgEntries.clear();
+	return false;
+}
