@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2024 CZ.NIC
+ * Copyright (C) 2014-2025 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -942,6 +942,7 @@ enum Isds::Type::CreditEventType libisdsCreditEventType2CreditEventType(
 	case ISDS_CREDIT_MESSAGE_SENT: cet = Isds::Type::CET_MESSAGE_SENT; break;
 	case ISDS_CREDIT_STORAGE_SET: cet = Isds::Type::CET_STORAGE_SET; break;
 	case ISDS_CREDIT_EXPIRED: cet = Isds::Type::CET_EXPIRED; break;
+	case ISDS_CREDIT_DELETED_MESSAGE_RECOVERED: cet = Isds::Type::CET_DELETED_MESSAGE_RECOVERED; break;
 	default:
 		Q_ASSERT(0);
 		iOk = false;
@@ -1045,6 +1046,28 @@ Isds::CreditEventStorageSet libisds2CreditEventStorageSet(
 	return creditEventStorageSet;
 }
 
+static
+Isds::CreditEventDeletedMessageRecovered libisds2DeletedMessageRecovered(
+    const struct isds_credit_event_deleted_message_recovered *icedmr,
+    bool *ok = Q_NULLPTR)
+{
+	if (Q_UNLIKELY(icedmr == Q_NULLPTR)) {
+		if (ok != Q_NULLPTR) {
+			*ok = true;
+		}
+		Isds::CreditEventDeletedMessageRecovered();
+	}
+
+	Isds::CreditEventDeletedMessageRecovered creditEventDeletedMessageRecovered;
+
+	creditEventDeletedMessageRecovered.setInitiator(Isds::fromCStr(icedmr->initiator));
+
+	if (ok != Q_NULLPTR) {
+		*ok = true;
+	}
+	return creditEventDeletedMessageRecovered;
+}
+
 Isds::CreditEvent Isds::libisds2creditEvent(const struct isds_credit_event *ice,
     bool *ok)
 {
@@ -1088,6 +1111,10 @@ Isds::CreditEvent Isds::libisds2creditEvent(const struct isds_credit_event *ice,
 	case Type::CET_EXPIRED:
 		creditEvent.setExpired();
 		break;
+	case Isds::Type::CET_DELETED_MESSAGE_RECOVERED:
+		creditEvent.setDeletedMessageRecovered(
+		    libisds2DeletedMessageRecovered(&ice->details.deleted_message_recovered, &iOk));
+		break;
 	default:
 		Q_ASSERT(0);
 		goto fail;
