@@ -69,6 +69,7 @@ static const QString keyAssociatedDirectories("associated_directories");
 
 static const QString typeBackup("backup");
 static const QString typeTransfer("transfer");
+static const QString typeDesktopToMobile("desktop_to_mobile");
 
 static const QString appName("datovka");
 
@@ -2312,6 +2313,8 @@ Json::Backup Json::Backup::fromJsonVal(const QJsonValue &jsonVal, bool *ok)
 					bu.setType(Backup::BUT_BACKUP);
 				} else if (val == typeTransfer) {
 					bu.setType(Backup::BUT_TRANSFER);
+				} else if (val == typeDesktopToMobile) {
+					bu.setType(Backup::BUT_DESKTOP_TO_MOBILE);
 				} else if (val.isEmpty()) {
 					bu.setType(Backup::BUT_UNKNOWN);
 				} else {
@@ -2423,6 +2426,9 @@ bool Json::Backup::toJsonVal(QJsonValue &jsonVal) const
 		case BUT_TRANSFER:
 			val = typeTransfer;
 			break;
+		case BUT_DESKTOP_TO_MOBILE:
+			val = typeDesktopToMobile;
+			break;
 		default:
 			Q_ASSERT(0);
 			break;
