@@ -26,13 +26,13 @@
 #include <utility> /* ::std::swap */
 
 #include "src/datovka_shared/json/helper.h"
+#include "src/json/message_id.h"
 #include "src/json/tag_assignment.h"
 #include "src/json/tag_entry.h"
-#include "src/json/tag_message_id.h"
 
 #define DLFT_TEST_ENV Isds::Type::BOOL_NULL
 #define nullInt -1
-static const Json::TagMsgIdList nullTagMsgIds;
+static const Json::MsgId1List nullMsgId1s;
 static const Json::TagEntryList nullEntries;
 
 /*!
@@ -307,7 +307,7 @@ public:
 	}
 
 	qint64 m_tagId; /*!< Tag identifier. */
-	TagMsgIdList m_tagMsgIds; /*!< List of message identifiers. */
+	MsgId1List m_tagMsgIds; /*!< List of message identifiers. */
 };
 
 Json::TagIdToMsgIdList::TagIdToMsgIdList(void)
@@ -345,7 +345,7 @@ Json::TagIdToMsgIdList::~TagIdToMsgIdList(void)
 }
 
 Json::TagIdToMsgIdList::TagIdToMsgIdList(qint64 tagId,
-    const TagMsgIdList &tagMsgIds)
+    const MsgId1List &tagMsgIds)
     : Object(),
     d_ptr(new (::std::nothrow) TagIdToMsgIdListPrivate)
 {
@@ -355,7 +355,7 @@ Json::TagIdToMsgIdList::TagIdToMsgIdList(qint64 tagId,
 }
 
 #ifdef Q_COMPILER_RVALUE_REFS
-Json::TagIdToMsgIdList::TagIdToMsgIdList(qint64 tagId, TagMsgIdList &&tagMsgIds)
+Json::TagIdToMsgIdList::TagIdToMsgIdList(qint64 tagId, MsgId1List &&tagMsgIds)
     : Object(),
     d_ptr(new (::std::nothrow) TagIdToMsgIdListPrivate)
 {
@@ -449,17 +449,17 @@ void Json::TagIdToMsgIdList::setTagId(qint64 i)
 	d->m_tagId = i;
 }
 
-const Json::TagMsgIdList &Json::TagIdToMsgIdList::tagMsgIds(void) const
+const Json::MsgId1List &Json::TagIdToMsgIdList::tagMsgIds(void) const
 {
 	Q_D(const TagIdToMsgIdList);
 	if (Q_UNLIKELY(d == Q_NULLPTR)) {
-		return nullTagMsgIds;
+		return nullMsgId1s;
 	}
 
 	return d->m_tagMsgIds;
 }
 
-void Json::TagIdToMsgIdList::setTagMsgIds(const TagMsgIdList &l)
+void Json::TagIdToMsgIdList::setMsgId1s(const MsgId1List &l)
 {
 	ensureTagIdToMsgIdListPrivate();
 	Q_D(TagIdToMsgIdList);
@@ -467,7 +467,7 @@ void Json::TagIdToMsgIdList::setTagMsgIds(const TagMsgIdList &l)
 }
 
 #ifdef Q_COMPILER_RVALUE_REFS
-void Json::TagIdToMsgIdList::setTagMsgIds(TagMsgIdList &&l)
+void Json::TagIdToMsgIdList::setMsgId1s(MsgId1List &&l)
 {
 	ensureTagIdToMsgIdListPrivate();
 	Q_D(TagIdToMsgIdList);
@@ -518,7 +518,7 @@ Json::TagIdToMsgIdList Json::TagIdToMsgIdList::fromJsonVal(
 				goto fail;
 			}
 			iOk = false;
-			tmil.setTagMsgIds(TagMsgIdList::fromJsonVal(arr, &iOk));
+			tmil.setMsgId1s(MsgId1List::fromJsonVal(arr, &iOk));
 			if (Q_UNLIKELY(!iOk)) {
 				goto fail;
 			}
