@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2024 CZ.NIC
+ * Copyright (C) 2014-2025 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,6 +25,8 @@
 #include <QApplication>
 #include <QCommandLineParser>
 #include <QDateTime>
+#include <QDir>
+#include <QMessageBox>
 #include <QSettings>
 #include <QSplashScreen>
 #include <QStyleFactory>
@@ -44,6 +46,7 @@
 #include "src/datovka_shared/log/memory_log.h"
 #include "src/datovka_shared/settings/pin.h"
 #include "src/datovka_shared/worker/pool.h"
+#include "src/db_repair/gui/mw_db_repair.h"
 #include "src/global.h"
 #include "src/gui/datovka.h"
 #include "src/gui/dlg_pin_input.h"
@@ -52,6 +55,7 @@
 #include "src/gui/dlg_view_zfo.h"
 #include "src/gui/dlg_update_portable.h"
 #include "src/initialisation.h"
+#include "src/initialisation_gui.h" /* loadAppUiTheme() */
 #include "src/io/filesystem.h"
 #include "src/settings/accounts.h"
 #include "src/settings/convert_to_prefs.h"
@@ -158,6 +162,8 @@ int main(int argc, char *argv[])
 	/* Process command-line arguments. */
 	parser.process(app);
 
+	preferencesSetUpRegistry(*GlobInstcs::iniPrefsPtr);
+
 	if (Q_UNLIKELY(0 != preferencesSetUp(parser, *GlobInstcs::iniPrefsPtr,
 	        *GlobInstcs::logPtr))) {
 		logErrorNL("%s",
@@ -165,6 +171,17 @@ int main(int argc, char *argv[])
 		return EXIT_FAILURE;
 	}
 
+	{
+		/* No localisation is loaded here. */
+		QString errorMessage;
+		if (Q_UNLIKELY(0 != preferencesOk(*GlobInstcs::iniPrefsPtr, errorMessage))) {
+			QMessageBox::critical(Q_NULLPTR,
+			    "Bad Command Line Argument",
+			    errorMessage);
+			return EXIT_FAILURE;
+		}
+	}
+
 	const QStringList cmdLineFileNames(parser.positionalArguments());
 	const QStringList srvcArgs(
 	    CLIParser::CLIServiceArgs(parser.optionNames()));
@@ -220,7 +237,50 @@ int main(int argc, char *argv[])
 	logInfo("Starting at %lld.%03lld .\n", start / 1000, start % 1000);
 
 	/* Create configuration file if file is missing. */
-	GlobInstcs::iniPrefsPtr->ensureConfPresence();
+	{
+		QString problemPath;
+		const enum INIPreferences::ConfPresence presenceCode =
+		    GlobInstcs::iniPrefsPtr->ensureConfPresence(&problemPath);
+		if (Q_UNLIKELY((runMode == RM_GUI)
+		        && (presenceCode != INIPreferences::ACCESSIBLE))) {
+			loadLocalisationSystem();
+			QString errorMessage =
+			    INIPreferences::confPresenceErrorMessage(
+			        presenceCode, problemPath);
+			errorMessage += "\n\n";
+			errorMessage +=
+			    QObject::tr("Please check that you own the permissions to read and write the entire content of the directory '%1'.")
+			        .arg(GlobInstcs::iniPrefsPtr->confDir());
+			QMessageBox::critical(Q_NULLPTR,
+			    QObject::tr("Cannot Access Configuration"),
+			    errorMessage);
+			return EXIT_FAILURE;
+		}
+	}
+	/*
+	 * Check whether there are permissions to modify content of
+	 * configuration directory.
+	 */
+	{
+		QStringList problemPaths;
+		const enum INIPreferences::ConfPresence presenceCode =
+		    GlobInstcs::iniPrefsPtr->checkConfiguration(problemPaths);
+		if (Q_UNLIKELY((runMode == RM_GUI)
+		        && (presenceCode != INIPreferences::ACCESSIBLE))) {
+			loadLocalisationSystem();
+			QString errorMessage =
+			    INIPreferences::confPresenceErrorMessage2(
+			        presenceCode, problemPaths);
+			errorMessage += "\n\n";
+			errorMessage +=
+			    QObject::tr("Please check that you own the permissions to read and write the entire content of the directory '%1'.")
+			        .arg(GlobInstcs::iniPrefsPtr->confDir());
+			QMessageBox::critical(Q_NULLPTR,
+			    QObject::tr("Cannot Access Configuration"),
+			    errorMessage);
+			return EXIT_FAILURE;
+		}
+	}
 
 	logAppVersion();
 	logTimeZone();
@@ -233,12 +293,14 @@ int main(int argc, char *argv[])
 		/* OSX/macOS contains own SSL library. Just log a warning. */
 		logWarningNL("%s", "Cryptographic library mismatch.");
 #else /* !defined(Q_OS_MACOS) */
+		/*
+		 * Qt-6 seems to be compiled against Windows SSL implementation.
+		 * Just log an error and continue.
+		 *
+		 * https://stackoverflow.com/questions/77116935/the-backend-cert-only-does-not-support-qsslkey
+		 */
 		logErrorNL("%s", "Cryptographic library mismatch.");
 #endif /* defined(Q_OS_MACOS) */
-#if defined(Q_OS_WIN)
-		/* Exit only on windows. */
-		return EXIT_FAILURE;
-#endif /* defined(Q_OS_WIN) */
 		break;
 	case 0:
 		break;
@@ -270,11 +332,35 @@ int main(int argc, char *argv[])
 		    GlobInstcs::iniPrefsPtr->saveConfPath().toUtf8().constData());
 
 		/* Change "\" to "/" */
-		confFileFixBackSlashes(GlobInstcs::iniPrefsPtr->loadConfPath());
+		confFileFixBackSlashes(GlobInstcs::iniPrefsPtr->loadConfPath(),
+		    true);
+	}
+
+	if (!SQLiteDb::dbDriverSupport()) {
+		logError("Cannot load database driver '%s'.\n",
+		    SQLiteDb::dbDriverType.toUtf8().constData());
+		loadLocalisationSystem();
+		QMessageBox::critical(Q_NULLPTR,
+		    QObject::tr("Missing SQLite Database Driver"),
+		    QObject::tr("Missing SQLite database driver module for the Qt framework."));
+		return EXIT_FAILURE;
 	}
 
 	/* Initialise database-based preferences. */
 	if (Q_UNLIKELY(0 != allocGlobPrefs(*GlobInstcs::iniPrefsPtr))) {
+		loadLocalisationSystem();
+		QString errorMessage =
+		    QObject::tr("Cannot open preferences database file '%1'.")
+		        .arg(GlobInstcs::iniPrefsPtr->prefsDbPath());
+		errorMessage += "\n\n";
+		errorMessage +=
+		    QObject::tr("You may move, rename or delete this file. It doesn't contain any message data.");
+		errorMessage += "\n\n";
+		errorMessage +=
+		    QObject::tr("If you remove the file then all application settings will be reset to default values.");
+		QMessageBox::critical(Q_NULLPTR,
+		    QObject::tr("Cannot Access Preferences"),
+		    errorMessage);
 		return EXIT_FAILURE;
 	}
 
@@ -282,9 +368,28 @@ int main(int argc, char *argv[])
 	PrefsDefaults::setDefaults(*GlobInstcs::prefsPtr);
 	/* Load preferences from database. */
 	if (Q_UNLIKELY(0 != loadPreferencesDatabase())) {
+		loadLocalisationSystem();
+		QString errorMessage =
+		    QObject::tr("Cannot apply content from preferences database file '%1'.")
+		        .arg(GlobInstcs::iniPrefsPtr->prefsDbPath());
+		errorMessage += "\n\n";
+		errorMessage +=
+		    QObject::tr("You may try moving, renaming or deleting this file. It doesn't contain any message data.");
+		errorMessage += "\n\n";
+		errorMessage +=
+		    QObject::tr("If you remove the file then all application settings will be reset to default values.");
+		QMessageBox::critical(Q_NULLPTR,
+		    QObject::tr("Cannot Apply Preferences"),
+		    errorMessage);
 		return EXIT_FAILURE;
 	}
 
+	/*
+	 * Load and set application UI theme from preferences.
+	 * Dath theme is detected here.
+	 */
+	loadAppUiTheme(*GlobInstcs::prefsPtr);
+
 	/* Convert INI preferences into database and remove them from INI file. */
 	{
 		QSettings settings(GlobInstcs::iniPrefsPtr->loadConfPath(),
@@ -304,20 +409,13 @@ int main(int argc, char *argv[])
 	}
 
 	/* Load proxy settings. */
-	GlobInstcs::proxSetPtr->loadFromSettings(*GlobInstcs::prefsPtr);
+	GlobInstcs::proxSetPtr->loadFromPrefs(*GlobInstcs::prefsPtr);
 	/* Apply settings onto the environment because of libcurl. */
 	GlobInstcs::proxSetPtr->setProxyEnvVars();
 
 	/* Start downloading the CRL files. */
 	downloadCRL();
 
-	if (!SQLiteDb::dbDriverSupport()) {
-		logError("Cannot load database driver '%s'.\n",
-		    SQLiteDb::dbDriverType.toUtf8().constData());
-		/* TODO -- throw a dialog notifying the user. */
-		return EXIT_FAILURE;
-	}
-
 #if defined(Q_OS_OSX)
 	/* Set style. */
 	{
@@ -337,7 +435,7 @@ int main(int argc, char *argv[])
 	logDebugLv0NL("App path: '%s'.",
 	    app.applicationDirPath().toUtf8().constData());
 
-	loadLocalisation(*GlobInstcs::prefsPtr);
+	loadLocalisationPrefs(*GlobInstcs::prefsPtr);
 
 	/* Ask for PIN. */
 	{
@@ -367,6 +465,7 @@ int main(int argc, char *argv[])
 				pinOk = DlgPinInput::queryPin(
 				    *GlobInstcs::pinSetPtr);
 				splash->show();
+				app.processEvents();
 			} else if (RM_CLI == runMode) {
 				pinOk = CLIPin::queryPin(
 				    *GlobInstcs::pinSetPtr, 2);
@@ -381,9 +480,9 @@ int main(int argc, char *argv[])
 
 	/* set splash action text */
 	if (runMode == RM_GUI) {
-		Qt::Alignment align = Qt::AlignCenter;
-		splash->showMessage(QObject::tr("Application is loading..."),
-		align, Qt::white);
+		splash->showMessage(QObject::tr("Application is loading data from\n%1")
+		    .arg(QDir::toNativeSeparators(GlobInstcs::iniPrefsPtr->confDir())),
+		    Qt::AlignCenter, Qt::white);
 		app.processEvents();
 	}
 
@@ -393,12 +492,57 @@ int main(int argc, char *argv[])
 	/* Backup tag database if it is in old format. */
 	createBackupOfOldFormatData(*GlobInstcs::iniPrefsPtr);
 
-	/* Initialise database-based containers. */
-	if (Q_UNLIKELY(0 != allocGlobContainers(*GlobInstcs::prefsPtr,
-	        *GlobInstcs::iniPrefsPtr))) {
+	if (Q_UNLIKELY(0 != allocGlobTimestamps(*GlobInstcs::iniPrefsPtr))) {
+		loadLocalisationSystem();
+		QString errorMessage =
+		    QObject::tr("Cannot open timestamp database file '%1'.")
+		        .arg(GlobInstcs::iniPrefsPtr->timestampDbPath());
+		errorMessage += "\n\n";
+		errorMessage +=
+		    QObject::tr("You may move, rename or delete this file. It doesn't contain any message data.");
+		errorMessage += "\n\n";
+		errorMessage +=
+		    QObject::tr("If you remove the file then information about message timestamp expiration will be lost and will need to be generated again.");
+		QMessageBox::critical(Q_NULLPTR,
+		    QObject::tr("Cannot Access Timestamp Data"),
+		    errorMessage);
 		return EXIT_FAILURE;
 	}
 
+	/* Initialise database-based containers. */
+	{
+		QString problemPath;
+		const enum InitialisationError ret = allocGlobContainers(
+		    *GlobInstcs::prefsPtr, *GlobInstcs::iniPrefsPtr,
+		    &problemPath);
+
+		if (Q_UNLIKELY(SUCCESS != ret)) {
+			/* Localisation already loaded. */
+			QString errorMessage;
+			switch (ret) {
+			case ERR_ALLOC_RESOURCES:
+				errorMessage = QObject::tr("Cannot allocate resources.");
+				break;
+			case ERR_OPEN_FILES:
+				errorMessage = QObject::tr("Cannot read content of file '%1'.").arg(problemPath);
+				errorMessage += "\n\n";
+				errorMessage += QObject::tr("File content may be corrupted.");
+				errorMessage += "\n\n";
+				errorMessage += QObject::tr("You may try moving the affected file into a different location or try renaming it. This may solve the problem.");
+				break;
+			default:
+				break;
+			}
+			errorMessage += "\n\n";
+			errorMessage +=
+			    QObject::tr("You may try running the application with enabled logging and check the log for more detailed information.");
+			QMessageBox::critical(Q_NULLPTR,
+			    QObject::tr("Cannot Load Container Data"),
+			    errorMessage);
+			return EXIT_FAILURE;
+		}
+	}
+
 	int ret = EXIT_SUCCESS;
 
 	/* Start worker threads. */
@@ -410,6 +554,11 @@ int main(int argc, char *argv[])
 	//logInfoNL("%s", "Enabling high DPI scaling.");
 	//QGuiApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
 #endif /* >= Qt-5.6 */
+	/*
+	 * Enable icons in menus.
+	 * The default value is platform dependent and seems to be true in Qt-6.8 on macOS.
+	 */
+	QCoreApplication::setAttribute(Qt::AA_DontShowIconsInMenus, false);
 
 	if (runMode == RM_CLI) {
 		/* Parse account information. */
@@ -430,17 +579,17 @@ int main(int argc, char *argv[])
 			GlobInstcs::acntMapPtr->loadFromSettings(
 			    GlobInstcs::iniPrefsPtr->confDir(), settings);
 			GlobInstcs::acntMapPtr->decryptAllPwds(
-			    GlobInstcs::pinSetPtr->_pinVal);
+			    GlobInstcs::pinSetPtr->pinValue());
 			GlobInstcs::shadowAcntsPtr->loadFromSettings(
 			    GlobInstcs::iniPrefsPtr->confDir(), settings);
 			GlobInstcs::shadowAcntsPtr->decryptAllPwds(
-			    GlobInstcs::pinSetPtr->_pinVal);
+			    GlobInstcs::pinSetPtr->pinValue());
 		}
 		delete splash;
 		ret = CLIParser::runCLIService(srvcArgs, parser);
 	} else if (runMode == RM_ZFO) {
 		delete splash;
-		foreach (const QString &fileName, cmdLineFileNames) {
+		for (const QString &fileName : cmdLineFileNames) {
 			ret = showZfo(fileName);
 			if (0 != ret) {
 				break;
@@ -472,18 +621,31 @@ int main(int argc, char *argv[])
 	logInfo("%s\n", "All worker threads finished");
 
 	if (DlgRestore::restoreDescr != Q_NULLPTR) {
+		deallocGlobTimestamps();
 		deallocGlobContainers(); /* Databases cannot be loaded here. */
 		deallocGlobSettings(); /* Settings cannot be loaded here. */
 		DlgRestoreProgress::restore();
 	}
 	if (DlgUpdatePortable::updatePortableData != Q_NULLPTR) {
 		/* Stop everything except logging. */
+		deallocGlobTimestamps();
 		deallocGlobContainers();
 		deallocGlobPrefs();
 		deallocGlobSettings();
 		deallocGlobInfrastruct();
 		DlgUpdatePortable::updatePortable();
 	}
+	if (MWDbRepair::runDatabaseRepair) {
+		/* Stop everything except logging. */
+		deallocGlobTimestamps();
+		deallocGlobContainers();
+		deallocGlobPrefs();
+		deallocGlobSettings();
+		deallocGlobInfrastruct();
+		MWDbRepair mwDbrepair;
+		mwDbrepair.show();
+		app.exec();
+	}
 
 	stop = QDateTime::currentMSecsSinceEpoch();
 	diff = stop - start;
@@ -498,6 +660,7 @@ int main(int argc, char *argv[])
 	 */
 	//crypto_cleanup_threads();
 
+	deallocGlobTimestamps();
 	deallocGlobContainers();
 	deallocGlobPrefs();
 	deallocGlobSettings();
