@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2024 CZ.NIC
+ * Copyright (C) 2014-2025 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -87,7 +87,7 @@ static const AcntId nullAcntId;
 
 AccountModel::AccountModel(QObject *parent)
     : QAbstractItemModel(parent),
-    m_regularAccounts(Q_NULLPTR),
+    m_accountsPtr(Q_NULLPTR),
     m_numIdMapping(),
     m_countersMap()
 {
@@ -118,10 +118,10 @@ QModelIndex AccountModel::index(int row, int column,
 	quintptr internalId = 0;
 	if (nodeAccountTop == type) {
 		/* Set top node row and type. */
-		Q_ASSERT(m_regularAccounts != Q_NULLPTR);
-		Q_ASSERT((row >= 0) && (row < m_regularAccounts->acntIds().size()));
-		Q_ASSERT(m_regularAccounts->acntIds().at(row).isValid());
-		const int numId = m_numIdMapping.numId(m_regularAccounts->acntIds()[row]);
+		Q_ASSERT(m_accountsPtr != Q_NULLPTR);
+		Q_ASSERT((row >= 0) && (row < m_accountsPtr->acntIds().size()));
+		Q_ASSERT(m_accountsPtr->acntIds().at(row).isValid());
+		const int numId = m_numIdMapping.numId(m_accountsPtr->acntIds()[row]);
 		Q_ASSERT(numId >= 0);
 		Q_ASSERT(m_numIdMapping.acntId(numId).isValid());
 		internalId = internalIdCreate(numId, type);
@@ -153,11 +153,11 @@ QModelIndex AccountModel::parent(const QModelIndex &index) const
 
 	if (parentRow < 0) {
 		Q_ASSERT(nodeAccountTop == type);
-		Q_ASSERT(m_regularAccounts != Q_NULLPTR);
+		Q_ASSERT(m_accountsPtr != Q_NULLPTR);
 		/* Determine the row of the account top node. */
 		const int numId = internalIdNumId(internalId);
 		const AcntId acntId = m_numIdMapping.acntId(numId);
-		int row = m_regularAccounts->acntIds().indexOf(acntId);
+		int row = m_accountsPtr->acntIds().indexOf(acntId);
 		if (row >= 0) {
 			parentRow = row;
 		}
@@ -178,8 +178,8 @@ int AccountModel::rowCount(const QModelIndex &parent) const
 
 	if (!parent.isValid()) {
 		/* Root. */
-		if (m_regularAccounts != Q_NULLPTR) {
-			return m_regularAccounts->acntIds().size();
+		if (m_accountsPtr != Q_NULLPTR) {
+			return m_accountsPtr->acntIds().size();
 		} else {
 			return 0;
 		}
@@ -248,9 +248,9 @@ QVariant AccountModel::data(const QModelIndex &index, int role) const
 		switch (type) {
 		case nodeAccountTop:
 			if (!collapsed) {
-				return m_regularAccounts->acntData(aId).accountName();
+				return m_accountsPtr->acntData(aId).accountName();
 			} else {
-				QString label = m_regularAccounts->acntData(aId).accountName();
+				QString label = m_accountsPtr->acntData(aId).accountName();
 				Q_ASSERT(m_countersMap.constFind(aId) !=
 				    m_countersMap.constEnd());
 				const AccountCounters &cntrs(
@@ -333,25 +333,25 @@ QVariant AccountModel::data(const QModelIndex &index, int role) const
 			switch (type) {
 			case nodeAccountTop:
 				if (!collapsed) {
-					return inconContainer.icon(IconContainer::ICON_DATOVKA_MESSAGE_BLUE);
+					return inconContainer.icon(IconContainer::ICON_DATABOX);
 				} else {
 					const AccountCounters &cntrs(
 					    m_countersMap[aId]);
 					return inconContainer.icon(
 					    (cntrs.unreadRecentReceived > 0) ?
-					        IconContainer::ICON_DATOVKA_MESSAGE_BLUE_RED_DOT :
-					        IconContainer::ICON_DATOVKA_MESSAGE_BLUE);
+					        IconContainer::ICON_DATABOX_NEW_MESSAGE :
+					        IconContainer::ICON_DATABOX);
 				}
 				break;
 			case nodeRecentReceived:
 			case nodeReceived:
 			case nodeReceivedYear:
-				return inconContainer.icon(IconContainer::ICON_DATOVKA_MESSAGE_DOWNLOAD);
+				return inconContainer.icon(IconContainer::ICON_MESSAGE_RECEIVED);
 				break;
 			case nodeRecentSent:
 			case nodeSent:
 			case nodeSentYear:
-				return inconContainer.icon(IconContainer::ICON_DATOVKA_MESSAGE_REPLY);
+				return inconContainer.icon(IconContainer::ICON_MESSAGE_SENT);
 				break;
 			default:
 				return QVariant();
@@ -444,7 +444,7 @@ QVariant AccountModel::data(const QModelIndex &index, int role) const
 		switch (type) {
 		case nodeAccountTop:
 			return tr("data box") + QLatin1String(" ") +
-			    m_regularAccounts->acntData(aId).accountName();
+			    m_accountsPtr->acntData(aId).accountName();
 			break;
 		case nodeRecentReceived:
 			{
@@ -582,7 +582,7 @@ bool AccountModel::moveRows(const QModelIndex &sourceParent, int sourceRow,
 		return false;
 	}
 
-	return m_regularAccounts->moveAccounts(sourceRow, count,
+	return m_accountsPtr->moveAccounts(sourceRow, count,
 	    destinationChild);
 }
 
@@ -728,51 +728,51 @@ bool AccountModel::dropMimeData(const QMimeData *data, Qt::DropAction action,
 	return false; /* If false is returned then removeRows() won't be triggered. */
 }
 
-void AccountModel::loadContent(AccountsMap *regularAccounts)
+void AccountModel::loadContent(AccountsMap *accounts)
 {
-	if (m_regularAccounts != Q_NULLPTR) {
-		m_regularAccounts->disconnect(SIGNAL(accountAboutToBeAdded(AcntId, int)),
-		    this, SLOT(regularAccountAboutToBeAdded(AcntId, int)));
-		m_regularAccounts->disconnect(SIGNAL(accountAdded(AcntId, int)),
-		    this, SLOT(regularAccountAdded(AcntId, int)));
+	if (m_accountsPtr != Q_NULLPTR) {
+		m_accountsPtr->disconnect(SIGNAL(accountAboutToBeAdded(AcntId, int)),
+		    this, SLOT(watchAccountAboutToBeAdded(AcntId, int)));
+		m_accountsPtr->disconnect(SIGNAL(accountAdded(AcntId, int)),
+		    this, SLOT(watchAccountAdded(AcntId, int)));
 
-		m_regularAccounts->disconnect(SIGNAL(accountAboutToBeRemoved(AcntId, int)),
-		    this, SLOT(regularAccountAboutToBeRemoved(AcntId, int)));
-		m_regularAccounts->disconnect(SIGNAL(accountRemoved(AcntId, int)),
-		    this, SLOT(regularAccountRemoved(AcntId, int)));
+		m_accountsPtr->disconnect(SIGNAL(accountAboutToBeRemoved(AcntId, int)),
+		    this, SLOT(watchAccountAboutToBeRemoved(AcntId, int)));
+		m_accountsPtr->disconnect(SIGNAL(accountRemoved(AcntId, int)),
+		    this, SLOT(watchAccountRemoved(AcntId, int)));
 
-		m_regularAccounts->disconnect(SIGNAL(accountsAboutToBeMoved(int, int, int)),
-		    this, SLOT(regularAccountsAboutToBeMoved(int, int, int)));
-		m_regularAccounts->disconnect(SIGNAL(accountsMoved(int, int, int)),
-		    this, SLOT(regularAccountsMoved(int, int, int)));
+		m_accountsPtr->disconnect(SIGNAL(accountsAboutToBeMoved(int, int, int)),
+		    this, SLOT(watchAccountsAboutToBeMoved(int, int, int)));
+		m_accountsPtr->disconnect(SIGNAL(accountsMoved(int, int, int)),
+		    this, SLOT(watchAccountsMoved(int, int, int)));
 
-		m_regularAccounts->disconnect(SIGNAL(accountUsernameChanged(AcntId, AcntId, int)),
-		    this, SLOT(regularUsernameChanged(AcntId, AcntId, int)));
-		m_regularAccounts->disconnect(SIGNAL(accountDataChanged(AcntId)),
-		    this, SLOT(regularAcountChanged(AcntId)));
+		m_accountsPtr->disconnect(SIGNAL(accountUsernameChanged(AcntId, AcntId, int)),
+		    this, SLOT(watchUsernameChanged(AcntId, AcntId, int)));
+		m_accountsPtr->disconnect(SIGNAL(accountDataChanged(AcntId)),
+		    this, SLOT(watchAcountChanged(AcntId)));
 	}
 
 	const QModelIndex parent;
 	int rows = rowCount(parent);
 	if (rows > 0) {
 		beginRemoveRows(parent, 0, rows - 1);
-		m_regularAccounts = Q_NULLPTR;
+		m_accountsPtr = Q_NULLPTR;
 		m_numIdMapping.clear();
 		m_countersMap.clear();
 		endRemoveRows();
 	}
 
-	if (regularAccounts == Q_NULLPTR) {
+	if (accounts == Q_NULLPTR) {
 		return;
 	}
 
-	if (regularAccounts->keys().isEmpty()) {
-		m_regularAccounts = regularAccounts;
+	if (accounts->keys().isEmpty()) {
+		m_accountsPtr = accounts;
 	} else {
-		rows = regularAccounts->acntIds().size();
+		rows = accounts->acntIds().size();
 		beginInsertRows(parent, 0, rows - 1);
-		m_regularAccounts = regularAccounts;
-		foreach (const AcntId &acntId, m_regularAccounts->acntIds()) {
+		m_accountsPtr = accounts;
+		foreach (const AcntId &acntId, m_accountsPtr->acntIds()) {
 			/* Prepare mapping and counters. */
 			m_numIdMapping.addAcntId(acntId);
 			m_countersMap[acntId] = AccountCounters();
@@ -780,25 +780,25 @@ void AccountModel::loadContent(AccountsMap *regularAccounts)
 		endInsertRows();
 	}
 
-	connect(m_regularAccounts, SIGNAL(accountAboutToBeAdded(AcntId, int)),
-	    this, SLOT(regularAccountAboutToBeAdded(AcntId, int)));
-	connect(m_regularAccounts, SIGNAL(accountAdded(AcntId, int)),
-	    this, SLOT(regularAccountAdded(AcntId, int)));
+	connect(m_accountsPtr, SIGNAL(accountAboutToBeAdded(AcntId, int)),
+	    this, SLOT(watchAccountAboutToBeAdded(AcntId, int)));
+	connect(m_accountsPtr, SIGNAL(accountAdded(AcntId, int)),
+	    this, SLOT(watchAccountAdded(AcntId, int)));
 
-	connect(m_regularAccounts, SIGNAL(accountAboutToBeRemoved(AcntId, int)),
-	    this, SLOT(regularAccountAboutToBeRemoved(AcntId, int)));
-	connect(m_regularAccounts, SIGNAL(accountRemoved(AcntId, int)),
-	    this, SLOT(regularAccountRemoved(AcntId, int)));
+	connect(m_accountsPtr, SIGNAL(accountAboutToBeRemoved(AcntId, int)),
+	    this, SLOT(watchAccountAboutToBeRemoved(AcntId, int)));
+	connect(m_accountsPtr, SIGNAL(accountRemoved(AcntId, int)),
+	    this, SLOT(watchAccountRemoved(AcntId, int)));
 
-	connect(m_regularAccounts, SIGNAL(accountsAboutToBeMoved(int, int, int)),
-	    this, SLOT(regularAccountsAboutToBeMoved(int, int, int)));
-	connect(m_regularAccounts, SIGNAL(accountsMoved(int, int, int)),
-	    this, SLOT(regularAccountsMoved(int, int, int)));
+	connect(m_accountsPtr, SIGNAL(accountsAboutToBeMoved(int, int, int)),
+	    this, SLOT(watchAccountsAboutToBeMoved(int, int, int)));
+	connect(m_accountsPtr, SIGNAL(accountsMoved(int, int, int)),
+	    this, SLOT(watchAccountsMoved(int, int, int)));
 
-	connect(m_regularAccounts, SIGNAL(accountUsernameChanged(AcntId, AcntId, int)),
-	    this, SLOT(regularUsernameChanged(AcntId, AcntId, int)));
-	connect(m_regularAccounts, SIGNAL(accountDataChanged(AcntId)),
-	    this, SLOT(regularAcountChanged(AcntId)));
+	connect(m_accountsPtr, SIGNAL(accountUsernameChanged(AcntId, AcntId, int)),
+	    this, SLOT(watchUsernameChanged(AcntId, AcntId, int)));
+	connect(m_accountsPtr, SIGNAL(accountDataChanged(AcntId)),
+	    this, SLOT(watchAcountChanged(AcntId)));
 }
 
 const AcntId &AccountModel::acntId(const QModelIndex &index) const
@@ -907,9 +907,9 @@ bool AccountModel::updateRecentUnread(const AcntId &acntId,
 		/*
 		 * Recent received unread number can also be viewed in top node.
 		 */
-		emit dataChanged(topIndex, topIndex);
+		Q_EMIT dataChanged(topIndex, topIndex);
 	}
-	emit dataChanged(childIndex, childIndex);
+	Q_EMIT dataChanged(childIndex, childIndex);
 
 	return true;
 }
@@ -1043,7 +1043,7 @@ bool AccountModel::updateYearNodes(const AcntId &acntId,
 			/* Update unread, increment row. */
 			(*unreadGroups)[groupName] = yCounter;
 			QModelIndex childIndex(index(row, 0, childTopIndex));
-			emit dataChanged(childIndex, childIndex);
+			Q_EMIT dataChanged(childIndex, childIndex);
 		}
 		--row;
 	}
@@ -1121,7 +1121,7 @@ bool AccountModel::updateYear(const AcntId &acntId,
 	Q_ASSERT(childIndex.isValid());
 
 	(*unreadGroups)[year] = yCounter;
-	emit dataChanged(childIndex, childIndex);
+	Q_EMIT dataChanged(childIndex, childIndex);
 
 	return true;
 }
@@ -1144,12 +1144,12 @@ bool AccountModel::changePosition(const AcntId &acntId, int shunt)
 	}
 
 	int row = topAcntRow(acntId);
-	if (Q_UNLIKELY((row < 0) || (row >= m_regularAccounts->acntIds().size()))) {
+	if (Q_UNLIKELY((row < 0) || (row >= m_accountsPtr->acntIds().size()))) {
 		Q_ASSERT(0);
 		return false;
 	}
 	int newRow = row + shunt;
-	if ((newRow < 0) || (newRow >= m_regularAccounts->acntIds().size())) {
+	if ((newRow < 0) || (newRow >= m_accountsPtr->acntIds().size())) {
 		return false;
 	}
 
@@ -1163,12 +1163,47 @@ bool AccountModel::changePosition(const AcntId &acntId, int shunt)
 		destChild += 1;
 	}
 
-	m_regularAccounts->moveAccounts(row, 1, destChild);
+	m_accountsPtr->moveAccounts(row, 1, destChild);
 
 	return true;
 }
 
-void AccountModel::regularAccountAboutToBeAdded(const AcntId &acntId, int row)
+void AccountModel::reloadIcons(void)
+{
+	inconContainer.rebuild();
+
+	/* Collect all valid indexes. */
+	QList<QModelIndex> indexes;
+	{
+		int i = 0;
+		indexes.append(QModelIndex()); /* Top invalid node at index 0. */
+		do {
+			/* Don't use const reference here as the container may grow. */
+			const QModelIndex parent = indexes.at(i);
+
+			for (int row = 0; row < rowCount(parent); ++row) {
+				for (int col = 0; col < columnCount(parent); ++col) {
+					QModelIndex child = index(row, col, parent);
+					if (child.isValid()) {
+						indexes.append(child);
+					} else {
+						Q_ASSERT(0);
+					}
+				}
+			}
+
+			++i;
+		} while (i < indexes.size());
+	}
+
+	/* Inform about decoration change. */
+	for (int i = 1; i < indexes.size(); ++i) { /* Top invalid node at index 0. */
+		const QModelIndex &index = indexes.at(i);
+		Q_EMIT dataChanged(index, index, {Qt::DecorationRole});
+	}
+}
+
+void AccountModel::watchAccountAboutToBeAdded(const AcntId &acntId, int row)
 {
 	beginInsertRows(QModelIndex(), row, row);
 
@@ -1176,7 +1211,7 @@ void AccountModel::regularAccountAboutToBeAdded(const AcntId &acntId, int row)
 	m_countersMap[acntId] = AccountCounters();
 }
 
-void AccountModel::regularAccountAdded(const AcntId &acntId, int row)
+void AccountModel::watchAccountAdded(const AcntId &acntId, int row)
 {
 	Q_UNUSED(acntId);
 	Q_UNUSED(row);
@@ -1184,7 +1219,7 @@ void AccountModel::regularAccountAdded(const AcntId &acntId, int row)
 	endInsertRows();
 }
 
-void AccountModel::regularAccountAboutToBeRemoved(const AcntId &acntId, int row)
+void AccountModel::watchAccountAboutToBeRemoved(const AcntId &acntId, int row)
 {
 	beginRemoveRows(QModelIndex(), row, row);
 
@@ -1192,7 +1227,7 @@ void AccountModel::regularAccountAboutToBeRemoved(const AcntId &acntId, int row)
 	m_countersMap.remove(acntId);
 }
 
-void AccountModel::regularAccountRemoved(const AcntId &acntId, int row)
+void AccountModel::watchAccountRemoved(const AcntId &acntId, int row)
 {
 	Q_UNUSED(acntId);
 	Q_UNUSED(row);
@@ -1200,14 +1235,14 @@ void AccountModel::regularAccountRemoved(const AcntId &acntId, int row)
 	endRemoveRows();
 }
 
-void AccountModel::regularAccountsAboutToBeMoved(int srcFrst, int srcLst, int dst)
+void AccountModel::watchAccountsAboutToBeMoved(int srcFrst, int srcLst, int dst)
 {
 	const QModelIndex parent;
 
 	beginMoveRows(parent, srcFrst, srcLst, parent, dst);
 }
 
-void AccountModel::regularAccountsMoved(int srcFrst, int srcLst, int dst)
+void AccountModel::watchAccountsMoved(int srcFrst, int srcLst, int dst)
 {
 	Q_UNUSED(srcFrst);
 	Q_UNUSED(srcLst);
@@ -1216,7 +1251,7 @@ void AccountModel::regularAccountsMoved(int srcFrst, int srcLst, int dst)
 	endMoveRows();
 }
 
-void AccountModel::regularUsernameChanged(const AcntId &oldId,
+void AccountModel::watchUsernameChanged(const AcntId &oldId,
     const AcntId &newId, int index)
 {
 	Q_UNUSED(index);
@@ -1224,18 +1259,18 @@ void AccountModel::regularUsernameChanged(const AcntId &oldId,
 	m_numIdMapping.replaceAcntId(oldId, newId);
 	m_countersMap.insert(newId, m_countersMap.take(oldId));
 
-	QModelIndex topIndex(topAcntIndex(newId));
+	const QModelIndex topIndex(topAcntIndex(newId));
 	if (topIndex.isValid()) {
-		emit dataChanged(topIndex, topIndex);
+		Q_EMIT dataChanged(topIndex, topIndex);
 	}
 }
 
-void AccountModel::regularAcountChanged(const AcntId &acntId)
+void AccountModel::watchAcountChanged(const AcntId &acntId)
 {
-	QModelIndex topIndex(topAcntIndex(acntId));
+	const QModelIndex topIndex(topAcntIndex(acntId));
 
 	if (topIndex.isValid()) {
-		emit dataChanged(topIndex, topIndex);
+		Q_EMIT dataChanged(topIndex, topIndex);
 	}
 }
 
@@ -1389,13 +1424,13 @@ int AccountModel::topAcntRow(const AcntId &acntId) const
 {
 	int foundRow = -1;
 
-	if (Q_UNLIKELY(m_regularAccounts == Q_NULLPTR)) {
+	if (Q_UNLIKELY(m_accountsPtr == Q_NULLPTR)) {
 		Q_ASSERT(0);
 		return foundRow;
 	}
 
-	for (int row = 0; row < m_regularAccounts->acntIds().size(); ++row) {
-		if (acntId == m_regularAccounts->acntIds().at(row)) {
+	for (int row = 0; row < m_accountsPtr->acntIds().size(); ++row) {
+		if (acntId == m_accountsPtr->acntIds().at(row)) {
 			foundRow = row;
 			break;
 		}
