@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2024 CZ.NIC
+ * Copyright (C) 2014-2025 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -240,7 +240,7 @@ QVariant AttachmentTblModel::data(const QModelIndex &index, int role) const
 			{
 				const QString fPath(_data(index).toString());
 				return (fPath == LOCAL_DATABASE_STR) ?
-				    tr("local database") : fPath;
+				    tr("local database") : QDir::toNativeSeparators(fPath);
 			}
 			break;
 		default:
@@ -473,11 +473,11 @@ void AttachmentTblModel::setHeader(void)
 	for (i = 0; i < MessageDb::fileItemIds.size(); ++i) {
 		/* Description. */
 		setHeaderData(i, Qt::Horizontal,
-		    flsTbl.attrProps.value(MessageDb::fileItemIds[i]).desc,
+		    MessageDbTables::flsTbl.attrProps.value(MessageDb::fileItemIds[i]).desc,
 		    Qt::DisplayRole);
 		/* Data type. */
 		setHeaderData(i, Qt::Horizontal,
-		    flsTbl.attrProps.value(MessageDb::fileItemIds[i]).type,
+		    MessageDbTables::flsTbl.attrProps.value(MessageDb::fileItemIds[i]).type,
 		    ROLE_DB_ENTRY_TYPE);
 	}
 
@@ -651,7 +651,9 @@ int AttachmentTblModel::insertAttachmentFile(const QString &filePath, int row)
 	}
 
 	for (int i = 0; i < rowCount(); ++i) {
-		if (Q_UNLIKELY(_data(i, FPATH_COL).toString() == filePath)) {
+		if (Q_UNLIKELY(
+		        QDir::fromNativeSeparators(_data(i, FPATH_COL).toString()) ==
+		            QDir::fromNativeSeparators(filePath))) {
 			/* Already in table. */
 			logWarning("File '%s' already in table.\n",
 			    filePath.toUtf8().constData());
@@ -675,7 +677,7 @@ int AttachmentTblModel::insertAttachmentFile(const QString &filePath, int row)
 #else /* !defined HOLD_FILE_CONTENT */
 	rowVect[BINARY_SIZE_COL] = 0;
 #endif /* defined HOLD_FILE_CONTENT */
-	rowVect[FPATH_COL] = filePath;
+	rowVect[FPATH_COL] = QDir::fromNativeSeparators(filePath);
 
 	/* Check data duplicity. */
 	if (insertVector(rowVect, row, true)) {
@@ -685,6 +687,23 @@ int AttachmentTblModel::insertAttachmentFile(const QString &filePath, int row)
 	}
 }
 
+bool AttachmentTblModel::holdsAttachmentFile(const QString &filePath) const
+{
+	if (Q_UNLIKELY(filePath.isEmpty())) {
+		return false;
+	}
+
+	for (int row = 0; row < rowCount(); ++row) {
+		const QString modelPath = QDir::fromNativeSeparators(
+		    _data(row, FPATH_COL, Qt::DisplayRole).toString());
+		if (modelPath == QDir::fromNativeSeparators(filePath)) {
+			return true;
+		}
+	}
+
+	return false;
+}
+
 bool AttachmentTblModel::appendBinaryAttachment(const QByteArray &binaryContent,
     const QString &fName)
 {
@@ -859,7 +878,8 @@ bool AttachmentTblModel::nameAndContentPresent(const QVariant &binaryContent,
 	for (int row = 0; row < rowCount(); ++row) {
 		const QVector<QVariant> &rowEntry = m_data.at(row);
 		if ((rowEntry.at(FNAME_COL) == fName) &&
-		    (rowEntry.at(FPATH_COL) == fPath) &&
+		    (QDir::fromNativeSeparators(rowEntry.at(FPATH_COL).toString()) ==
+		         QDir::fromNativeSeparators(fPath.toString())) &&
 		    (rowEntry.at(BINARY_CONTENT_COL) == binaryContent)) {
 			return true;
 		}
