@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2024 CZ.NIC
+ * Copyright (C) 2014-2025 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,8 +38,9 @@
 #include "src/io/dbs.h"
 #include "src/io/message_db.h"
 #include "src/io/message_db_tables.h"
+#include "src/io/timestamp_db.h" /* TstValidityHash */
+#include "src/json/message_id.h"
 #include "src/json/tag_assignment_hash.h"
-#include "src/json/tag_message_id.h"
 #include "src/models/helper.h"
 #include "src/models/messages_model.h"
 
@@ -53,7 +54,7 @@ static IconContainer inconContainer; /* Local icon container. */
 static inline
 QIcon defaultRMIcon(void)
 {
-	return IconContainer::construcIcon(IconContainer::ICON_UP);
+	return IconContainer::construcIcon(IconContainer::ICON_APP_ARROW_UP);
 }
 
 DbMsgsTblModel::DbMsgsTblModel(enum DbMsgsTblModel::Type type, QObject *parent)
@@ -74,7 +75,9 @@ QVariant DbMsgsTblModel::data(const QModelIndex &index, int role) const
 {
 	/* Draw records management and tag information. */
 	/* TODO -- This is only a temporary solution. */
-	if (index.column() == (MAX_COLNUM + RECMGMT_NEG_COL)) {
+	if (index.column() == (MAX_COLNUM + TST_TRAC_NEG_COL)) {
+		return tstData(index, role);
+	} else if (index.column() == (MAX_COLNUM + RECMGMT_NEG_COL)) {
 		return recMgmtData(index, role);
 	} else if (index.column() == (MAX_COLNUM + TAGS_NEG_COL)) {
 		return tagsData(index, role);
@@ -116,22 +119,22 @@ QVariant DbMsgsTblModel::data(const QModelIndex &index, int role) const
 		case READLOC_COL:
 			/* Show icon for 'read locally'. */
 			if (_data(index).toBool()) {
-				return inconContainer.icon(IconContainer::ICON_GREY_BALL);
+				return inconContainer.icon(IconContainer::ICON_BALL_GREY);
 			} else {
-				return inconContainer.icon(IconContainer::ICON_GREEN_BALL);
+				return inconContainer.icon(IconContainer::ICON_BALL_GREEN);
 			}
 			break;
 		case PROCSNG_COL:
 			/* Show icon for 'process status'. */
 			switch (_data(index).toInt()) {
 			case MessageDb::UNSETTLED:
-				return inconContainer.icon(IconContainer::ICON_RED_BALL);
+				return inconContainer.icon(IconContainer::ICON_BALL_RED);
 				break;
 			case MessageDb::IN_PROGRESS:
-				return inconContainer.icon(IconContainer::ICON_YELLOW_BALL);
+				return inconContainer.icon(IconContainer::ICON_BALL_YELLOW);
 				break;
 			case MessageDb::SETTLED:
-				return inconContainer.icon(IconContainer::ICON_GREY_BALL);
+				return inconContainer.icon(IconContainer::ICON_BALL_GREY);
 				break;
 			default:
 				Q_ASSERT(0);
@@ -317,6 +320,12 @@ QVariant DbMsgsTblModel::data(const QModelIndex &index, int role) const
 			return _data(index,
 			    Qt::DisplayRole).toString().toLower();
 			break;
+		case DELIVERY_COL:
+		case ACCEPT_COL:
+			return ModelHelper::sortRank(
+			    _data(index, Qt::DisplayRole).toDateTime(),
+			    getDmId(index));
+			break;
 		default:
 			return _data(index, Qt::DisplayRole);
 			break;
@@ -356,7 +365,7 @@ QVariant DbMsgsTblModel::headerData(int section, Qt::Orientation orientation,
 	case Qt::DecorationRole:
 		switch (section) {
 		case PERSDELIV_COL: /* 'personal delivery' */
-			return inconContainer.icon(IconContainer::ICON_HAND_GREY);
+			return inconContainer.icon(IconContainer::ICON_HAND);
 			break;
 		case READLOC_COL: /* 'read locally' */
 			return inconContainer.icon(IconContainer::ICON_READCOL);
@@ -547,66 +556,66 @@ bool DbMsgsTblModel::setHeader(const QList<AppendedCol> &appendedCols)
 	/* Sets description and data type. */
 
 	setHeaderData(DMID_COL, Qt::Horizontal,
-	    msgsTbl.attrProps.value(MessageDb::rcvdItemIds[0]).desc,
+	    MessageDbTables::msgsTbl.attrProps.value(MessageDb::rcvdItemIds[0]).desc,
 	    Qt::DisplayRole);
 	setHeaderData(DMID_COL, Qt::Horizontal,
-	    msgsTbl.attrProps.value(MessageDb::rcvdItemIds[0]).type,
+	    MessageDbTables::msgsTbl.attrProps.value(MessageDb::rcvdItemIds[0]).type,
 	    ROLE_DB_ENTRY_TYPE);
 
 	setHeaderData(PERSDELIV_COL, Qt::Horizontal,
-	    msgsTbl.attrProps.value(MessageDb::rcvdItemIds[1]).desc,
+	    MessageDbTables::msgsTbl.attrProps.value(MessageDb::rcvdItemIds[1]).desc,
 	    Qt::DisplayRole);
 	setHeaderData(PERSDELIV_COL, Qt::Horizontal,
-	    msgsTbl.attrProps.value(MessageDb::rcvdItemIds[1]).type,
+	    MessageDbTables::msgsTbl.attrProps.value(MessageDb::rcvdItemIds[1]).type,
 	    ROLE_DB_ENTRY_TYPE);
 
 	setHeaderData(ANNOT_COL, Qt::Horizontal,
-	    msgsTbl.attrProps.value(MessageDb::rcvdItemIds[2]).desc,
+	    MessageDbTables::msgsTbl.attrProps.value(MessageDb::rcvdItemIds[2]).desc,
 	    Qt::DisplayRole);
 	setHeaderData(ANNOT_COL, Qt::Horizontal,
-	    msgsTbl.attrProps.value(MessageDb::rcvdItemIds[2]).type,
+	    MessageDbTables::msgsTbl.attrProps.value(MessageDb::rcvdItemIds[2]).type,
 	    ROLE_DB_ENTRY_TYPE);
 
 	setHeaderData(SENDER_COL, Qt::Horizontal,
-	    msgsTbl.attrProps.value(MessageDb::rcvdItemIds[3]).desc,
+	    MessageDbTables::msgsTbl.attrProps.value(MessageDb::rcvdItemIds[3]).desc,
 	    Qt::DisplayRole);
 	setHeaderData(SENDER_COL, Qt::Horizontal,
-	    msgsTbl.attrProps.value(MessageDb::rcvdItemIds[3]).type,
+	    MessageDbTables::msgsTbl.attrProps.value(MessageDb::rcvdItemIds[3]).type,
 	    ROLE_DB_ENTRY_TYPE);
 
 	setHeaderData(RECIP_COL, Qt::Horizontal,
-	    msgsTbl.attrProps.value(MessageDb::sntItemIds[2]).desc,
+	    MessageDbTables::msgsTbl.attrProps.value(MessageDb::sntItemIds[2]).desc,
 	    Qt::DisplayRole);
 	setHeaderData(RECIP_COL, Qt::Horizontal,
-	    msgsTbl.attrProps.value(MessageDb::sntItemIds[2]).type,
+	    MessageDbTables::msgsTbl.attrProps.value(MessageDb::sntItemIds[2]).type,
 	    ROLE_DB_ENTRY_TYPE);
 
 	setHeaderData(DELIVERY_COL, Qt::Horizontal,
-	    msgsTbl.attrProps.value(MessageDb::rcvdItemIds[4]).desc,
+	    MessageDbTables::msgsTbl.attrProps.value(MessageDb::rcvdItemIds[4]).desc,
 	    Qt::DisplayRole);
 	setHeaderData(DELIVERY_COL, Qt::Horizontal,
-	    msgsTbl.attrProps.value(MessageDb::rcvdItemIds[4]).type,
+	    MessageDbTables::msgsTbl.attrProps.value(MessageDb::rcvdItemIds[4]).type,
 	    ROLE_DB_ENTRY_TYPE);
 
 	setHeaderData(ACCEPT_COL, Qt::Horizontal,
-	    msgsTbl.attrProps.value(MessageDb::rcvdItemIds[5]).desc,
+	    MessageDbTables::msgsTbl.attrProps.value(MessageDb::rcvdItemIds[5]).desc,
 	    Qt::DisplayRole);
 	setHeaderData(ACCEPT_COL, Qt::Horizontal,
-	    msgsTbl.attrProps.value(MessageDb::rcvdItemIds[5]).type,
+	    MessageDbTables::msgsTbl.attrProps.value(MessageDb::rcvdItemIds[5]).type,
 	    ROLE_DB_ENTRY_TYPE);
 
 	setHeaderData(READLOC_COL, Qt::Horizontal,
-	    smsgdtTbl.attrProps.value(MessageDb::rcvdItemIds[6]).desc,
+	    MessageDbTables::smsgdtTbl.attrProps.value(MessageDb::rcvdItemIds[6]).desc,
 	    Qt::DisplayRole);
 	setHeaderData(READLOC_COL, Qt::Horizontal,
-	    smsgdtTbl.attrProps.value(MessageDb::rcvdItemIds[6]).type,
+	    MessageDbTables::smsgdtTbl.attrProps.value(MessageDb::rcvdItemIds[6]).type,
 	    ROLE_DB_ENTRY_TYPE);
 
 	setHeaderData(MSGSTAT_COL, Qt::Horizontal,
-	    msgsTbl.attrProps.value(MessageDb::sntItemIds[5]).desc,
+	    MessageDbTables::msgsTbl.attrProps.value(MessageDb::sntItemIds[5]).desc,
 	    Qt::DisplayRole);
 	setHeaderData(MSGSTAT_COL, Qt::Horizontal,
-	    msgsTbl.attrProps.value(MessageDb::sntItemIds[5]).type,
+	    MessageDbTables::msgsTbl.attrProps.value(MessageDb::sntItemIds[5]).type,
 	    ROLE_DB_ENTRY_TYPE);
 
 	setHeaderData(ATTDOWN_COL, Qt::Horizontal,
@@ -795,12 +804,12 @@ bool DbMsgsTblModel::overrideProcessing(const QList<MsgId> &msgIds,
 	return found;
 }
 
-Json::TagMsgIdList DbMsgsTblModel::msgIdList(bool testEnv) const
+Json::MsgId1List DbMsgsTblModel::msgIdList(bool testEnv) const
 {
-	Json::TagMsgIdList msgIds;
+	Json::MsgId1List msgIds;
 	for (int row = 0; row < rowCount(); ++row) {
 		qint64 dmId = TblModel::index(row, 0).data().toLongLong();
-		const Json::TagMsgId msgId(
+		const Json::MsgId1 msgId(
 		    testEnv ? Isds::Type::BOOL_TRUE : Isds::Type::BOOL_FALSE,
 		    dmId);
 		msgIds.append(msgId);
@@ -825,7 +834,7 @@ bool DbMsgsTblModel::setTagAssignments(bool testEnv, int col,
 
 	for (int row = 0; row < rowCount(); ++row) {
 		qint64 dmId = TblModel::index(row, 0).data().toLongLong();
-		const Json::TagMsgId msgId(
+		const Json::MsgId1 msgId(
 		    testEnv ? Isds::Type::BOOL_TRUE : Isds::Type::BOOL_FALSE,
 		    dmId);
 		TagItemList tagList = assignments.value(msgId, Json::TagEntryList());
@@ -913,6 +922,62 @@ void DbMsgsTblModel::updateTagAssignment(qint64 dmId,
 	}
 }
 
+bool DbMsgsTblModel::setTimestampTrackings(bool testEnv, int col,
+    const TstValidityHash &tstValues)
+{
+	/* Check indexes into column. */
+	if (col >= 0) {
+		if (col > columnCount()) {
+			return false;
+		}
+	} else {
+		col += columnCount();
+		if (col < 0) {
+			return false;
+		}
+	}
+
+	for (int row = 0; row < rowCount(); ++row) {
+		qint64 dmId = TblModel::index(row, 0).data().toLongLong();
+		const Json::MsgId1 msgId(
+		    testEnv ? Isds::Type::BOOL_TRUE : Isds::Type::BOOL_FALSE,
+		    dmId);
+		TstValidity value = tstValues.value(msgId, TstValidity());
+		m_data[row][col] = QVariant::fromValue(value);
+	}
+
+	emit dataChanged(TblModel::index(0, col),
+	    TblModel::index(rowCount() - 1, col));
+
+	return true;
+}
+
+void DbMsgsTblModel::updateTimestampTracking(qint64 dmId,
+    const TstValidity &tstValue)
+{
+	if (Q_UNLIKELY(dmId < 0)) {
+		return;
+	}
+
+	int tstCol = MAX_COLNUM + TST_TRAC_NEG_COL;
+
+	for (int row = 0; row < rowCount(); ++row) {
+		if (_data(row, DMID_COL, Qt::DisplayRole).toLongLong() != dmId) {
+			continue;
+		}
+
+		QVariant &value = m_data[row][tstCol];
+		if (!value.canConvert<TstValidity>()) {
+			continue;
+		}
+
+		/* Store into model. */
+		value = QVariant::fromValue(tstValue);
+		emit dataChanged(TblModel::index(row, tstCol),
+		    TblModel::index(row, tstCol));
+	}
+}
+
 bool DbMsgsTblModel::setRecordsManagementIcon(void)
 {
 	if (Q_NULLPTR == GlobInstcs::recMgmtDbPtr) {
@@ -1067,6 +1132,50 @@ QVariant DbMsgsTblModel::recMgmtData(const QModelIndex &index, int role) const
 	}
 }
 
+QVariant DbMsgsTblModel::tstData(const QModelIndex &index, int role) const
+{
+	TstValidity validity;
+	{
+		QVariant rawData(_data(index, Qt::DisplayRole));
+		if (Q_UNLIKELY(!rawData.canConvert<TstValidity>())) {
+			/*
+			 * Cannot call Q_ASSERT(0) here as this method
+			 * may get called on invalid data occasionally.
+			 */
+			logWarningNL("%s",
+			    "Did not get timestamp data.");
+			return QVariant();
+		}
+		validity = qvariant_cast<TstValidity>(rawData);
+	}
+
+	switch (role) {
+	case Qt::DecorationRole:
+		return validity.tstTracked
+		    ? inconContainer.icon(IconContainer::ICON_CLOCK)
+		    : QVariant();
+		break;
+	case Qt::ToolTipRole:
+		return validity.tstExpiration.isValid()
+		    ? tr("Message timestamp expires at %1.")
+		        .arg(validity.tstExpiration.toString(Utility::dateTimeDisplayFormat))
+		    : QVariant();
+		break;
+	case Qt::AccessibleTextRole:
+		return validity.tstTracked
+		    ? tr("Message timestamp is being tracked.")
+		    : QVariant();
+		break;
+	case ROLE_PROXYSORT:
+		return ModelHelper::sortRank(validity.tstTracked ? 0 : 1,
+		    getDmId(index));
+		break;
+	default:
+		return QVariant();
+		break;
+	}
+}
+
 QVariant DbMsgsTblModel::tagsData(const QModelIndex &index, int role) const
 {
 	switch (role) {
