@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2024 CZ.NIC
+ * Copyright (C) 2014-2025 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,7 +21,6 @@
  * the two.
  */
 
-#include <QRegularExpression>
 #include <QSettings>
 
 #include "src/datovka_shared/identifiers/account_id.h"
@@ -40,7 +39,7 @@ QStringList accountIds(const QSettings &settings)
 {
 	QStringList accountIds;
 
-	foreach (const QString credGroup, AccountsMap::sortedCredentialGroups(settings)) {
+	for (const QString &credGroup : AccountsMap::sortedCredentialGroups(settings)) {
 		accountIds.append(
 		    PrefsHelper::accountIdentifier(
 		        settings.value(credGroup + "/username").toString(),
@@ -95,8 +94,6 @@ void convertAccountTree(QSettings &settings, Prefs &prefs)
 	/* Remove all keys in current group 'account_tree'. */
 	settings.remove("");
 	settings.endGroup();
-
-	settings.sync();
 }
 
 /*!
@@ -110,7 +107,7 @@ void convertColumnWidths(QSettings &settings, Prefs &prefs)
 	if (keys.size() > 0) {
 		logInfoNL("%s", "Converting message list column width settings.");
 	}
-	foreach (const QString &key, keys) {
+	for (const QString &key : keys) {
 		if ((key == "received_1") || (key == "received_2") || (key == "sent_2")) {
 			bool iOk = false;
 			qint64 val = settings.value(key).toLongLong(&iOk);
@@ -136,8 +133,6 @@ void convertColumnWidths(QSettings &settings, Prefs &prefs)
 	/* Remove all keys in current group. */
 	settings.remove("");
 	settings.endGroup();
-
-	settings.sync();
 }
 
 /*!
@@ -152,7 +147,7 @@ void convertConnection(QSettings &settings, Prefs &prefs)
 	if (keys.size() > 0) {
 		logInfoNL("%s", "Converting connection settings.");
 	}
-	foreach (const QString &key, keys) {
+	for (const QString &key : keys) {
 		if (key == "http_proxy") {
 			QString val = settings.value(key).toString();
 			if (val == NO_PROXY_STR) {
@@ -195,8 +190,6 @@ void convertConnection(QSettings &settings, Prefs &prefs)
 	/* Remove all keys in current group. */
 	settings.remove("");
 	settings.endGroup();
-
-	settings.sync();
 #undef NO_PROXY_STR
 }
 
@@ -211,7 +204,7 @@ void convertCredentialsPartial(QSettings &settings, Prefs &prefs)
 		logInfoNL("%s", "Converting some credential settings.");
 	}
 
-	foreach (const QString &credGroup, credGroups) {
+	for (const QString &credGroup : credGroups) {
 		settings.beginGroup(credGroup);
 
 		const AcntId acntId(settings.value("username").toString(),
@@ -265,8 +258,6 @@ void convertCredentialsPartial(QSettings &settings, Prefs &prefs)
 		settings.remove("last_export_zfo_path");
 		settings.endGroup();
 	}
-
-	settings.sync();
 }
 
 /*!
@@ -282,7 +273,7 @@ void convertDefaultAccount(QSettings &settings, Prefs &prefs)
 	if (!soughtUsername.isEmpty()) {
 		logInfoNL("%s", "Converting default account settings.");
 
-		foreach (const QString &accountId, accountsIdList) {
+		for (const QString &accountId : accountsIdList) {
 			QString username;
 			bool testing = false;
 			if (Q_UNLIKELY(!PrefsHelper::accountIdentifierData(
@@ -301,8 +292,6 @@ void convertDefaultAccount(QSettings &settings, Prefs &prefs)
 	/* Remove all keys in current group. */
 	settings.remove("");
 	settings.endGroup();
-
-	settings.sync();
 }
 
 /*!
@@ -316,7 +305,7 @@ void convertLastDirectories(QSettings &settings, Prefs &prefs)
 	if (keys.size() > 0) {
 		logInfoNL("%s", "Converting last directories settings.");
 	}
-	foreach (const QString &key, keys) {
+	for (const QString &key : keys) {
 		if (key == "on_import_database_dir_activate") {
 			QString val = settings.value(key).toString();
 			if (!val.isEmpty()) {
@@ -329,8 +318,6 @@ void convertLastDirectories(QSettings &settings, Prefs &prefs)
 	/* Remove all keys in current group. */
 	settings.remove("");
 	settings.endGroup();
-
-	settings.sync();
 }
 
 /*!
@@ -344,7 +331,7 @@ void convertMessageOrdering(QSettings &settings, Prefs &prefs)
 	if (keys.size() > 0) {
 		logInfoNL("%s", "Converting message list ordering settings.");
 	}
-	foreach (const QString &key, keys) {
+	for (const QString &key : keys) {
 		if (key == "sort_column") {
 			bool iOk = false;
 			int val = settings.value(key).toLongLong(&iOk);
@@ -368,8 +355,6 @@ void convertMessageOrdering(QSettings &settings, Prefs &prefs)
 	/* Remove all keys in current group. */
 	settings.remove("");
 	settings.endGroup();
-
-	settings.sync();
 }
 
 /*!
@@ -383,7 +368,7 @@ void convertPanes(QSettings &settings, Prefs &prefs)
 	if (keys.size() > 0) {
 		logInfoNL("%s", "Converting splitter position settings.");
 	}
-	foreach (const QString &key, keys) {
+	for (const QString &key : keys) {
 		if ((key == "hpaned1") || (key == "message_pane") || (key == "message_display_pane")) {
 			bool iOk = false;
 			qint64 val = settings.value(key).toLongLong(&iOk);
@@ -403,8 +388,6 @@ void convertPanes(QSettings &settings, Prefs &prefs)
 	/* Remove all keys in current group. */
 	settings.remove("");
 	settings.endGroup();
-
-	settings.sync();
 }
 
 /*!
@@ -418,7 +401,7 @@ void convertPreferences(QSettings &settings, Prefs &prefs)
 	if (keys.size() > 0) {
 		logInfoNL("%s", "Converting preferences settings.");
 	}
-	foreach (const QString &key, keys) {
+	for (const QString &key : keys) {
 		if (key == "auto_download_whole_messages") {
 			prefs.setBoolVal(
 			    "action.sync_account.tie.action.download_message",
@@ -446,10 +429,6 @@ void convertPreferences(QSettings &settings, Prefs &prefs)
 		} else if (key == "check_new_versions") {
 			prefs.setBoolVal("application.version.check.enabled",
 			    settings.value(key).toBool());
-		} else if (key == "send_stats_with_version_checks") {
-			prefs.setBoolVal(
-			    "application.version.check.stats.send.enabled",
-			    settings.value(key).toBool());
 		} else if (key == "isds_download_timeout_ms") {
 			prefs.setIntVal("network.isds.communication.timeout.ms",
 			    settings.value(key).toLongLong());
@@ -544,8 +523,6 @@ void convertPreferences(QSettings &settings, Prefs &prefs)
 	/* Remove all keys in current group. */
 	settings.remove("");
 	settings.endGroup();
-
-	settings.sync();
 }
 
 /*!
@@ -559,7 +536,7 @@ void convertWindowPosition(QSettings &settings, Prefs &prefs)
 	if (keys.size() > 0) {
 		logInfoNL("%s", "Converting window position settings.");
 	}
-	foreach (const QString &key, keys) {
+	for (const QString &key : keys) {
 		if ((key == "x") || (key == "y") || (key == "w") || (key == "h")) {
 			bool iOk = false;
 			qint64 val = settings.value(key).toLongLong(&iOk);
@@ -582,8 +559,6 @@ void convertWindowPosition(QSettings &settings, Prefs &prefs)
 	/* Remove all keys in current group. */
 	settings.remove("");
 	settings.endGroup();
-
-	settings.sync();
 }
 
 void iniPreferencesToPrefs(QSettings &settings, Prefs &prefs)
@@ -598,4 +573,6 @@ void iniPreferencesToPrefs(QSettings &settings, Prefs &prefs)
 	convertPanes(settings, prefs);
 	convertPreferences(settings, prefs);
 	convertWindowPosition(settings, prefs);
+
+	settings.sync();
 }
