@@ -44,11 +44,14 @@ static const QString acntDbFile("messages.shelf.db"); /*!< Account db file. */
 static const QString tagDbFile("tag.db"); /*!< Tag db file. */
 static const QString recMgmtDbFile("records_management.db"); /*!< Records management db file. */
 static const QString prefsDbFile("prefs.db"); /*!< Preferences db file. */
+static const QString timestampDbFile("timestamp.db"); /*!< Timestamp db file. */
 
 INIPreferences::INIPreferences(void)
-    : confSubdir(DFLT_CONF_SUBDIR),
-    loadFromConf(DFLT_CONF_FILE),
-    saveToConf(DFLT_CONF_FILE)
+    : _confBaseDir(), /* Empty value means default. */
+    _confSubdir(), /* Empty value means default DFLT_CONF_SUBDIR. */
+    _confDir(), /* Empty value means _confBaseDir + _confSubdir . */
+    _loadFromConf(), /* Empty value means default DFLT_CONF_FILE. */
+    _saveToConf() /* Empty value means default DFLT_CONF_FILE. */
 {
 }
 
@@ -197,6 +200,22 @@ QString findAuxFilePath(const QString &confDir, const QString &confFileName)
 	return confDir % QDir::separator() % auxFiles.last();
 }
 
+/*!
+ * @brief Return default configuration file name if \a fileName is empty.
+ *
+ * @param[in] fileName File name.
+ * @return Default if \a fileName is empty.
+ */
+static
+QString confFileName(const QString &fileName)
+{
+	if (fileName.isEmpty()) {
+		return DFLT_CONF_FILE;
+	} else {
+		return fileName;
+	}
+}
+
 enum INIPreferences::ConfPresence INIPreferences::ensureConfPresence(
     QString *problemPathPtr) const
 {
@@ -235,7 +254,7 @@ enum INIPreferences::ConfPresence INIPreferences::ensureConfPresence(
 
 	/* Try finding an auxiliary file and renaming it to the original. */
 	if (Q_UNLIKELY(!QFileInfo(loadConfPath()).exists())) {
-		const QString auxFilePath = findAuxFilePath(confDir(), loadFromConf);
+		const QString auxFilePath = findAuxFilePath(confDir(), confFileName(_loadFromConf));
 
 		if (!auxFilePath.isEmpty()) {
 			/* Rename found file. */
@@ -357,19 +376,39 @@ enum INIPreferences::ConfPresence INIPreferences::checkConfiguration(
 	return ACCESSIBLE;
 }
 
+/*!
+ * @brief Return default configuration subdirectory if \a subdir is empty.
+ *
+ * @param[in] subdir Subdirectory name.
+ * @return Default if \a subdir is empty.
+ */
+static
+QString confSubdir(const QString &subdir)
+{
+	if (subdir.isEmpty()) {
+		return DFLT_CONF_SUBDIR;
+	} else {
+		return subdir;
+	}
+}
+
 QString INIPreferences::confDir(void) const
 {
-	return confDirPath(confSubdir);
+	if (_confDir.isEmpty()) {
+		return confDirPath(_confBaseDir, confSubdir(_confSubdir));
+	} else {
+		return _confDir;
+	}
 }
 
 QString INIPreferences::loadConfPath(void) const
 {
-	return confDir() % QDir::separator() % loadFromConf;
+	return confDir() % QDir::separator() % confFileName(_loadFromConf);
 }
 
 QString INIPreferences::saveConfPath(void) const
 {
-	return confDir() % QDir::separator() % saveToConf;
+	return confDir() % QDir::separator() % confFileName(_saveToConf);
 }
 
 QString INIPreferences::acntDbPath(void) const
@@ -391,3 +430,8 @@ QString INIPreferences::prefsDbPath(void) const
 {
 	return confDir() % QDir::separator() % prefsDbFile;
 }
+
+QString INIPreferences::timestampDbPath(void) const
+{
+	return confDir() % QDir::separator() % timestampDbFile;
+}
