@@ -246,6 +246,10 @@ void PrefsDefaults::setDefaults(Prefs &prefs)
 	    "action.edit_tag_assignment.simplified_tag_assignment.window_decorations.enabled",
 	    PrefsDb::VAL_BOOLEAN, true));
 
+	prefs.setDefault(Prefs::Entry(
+	    "action.email.creation_manner",
+	    PrefsDb::VAL_INTEGER, PrefsSpecific::EMAIL_CREATE_DEFAULT));
+
 	prefs.setDefault(Prefs::Entry(
 	    "message.limit.megabyte.bytes",
 	    PrefsDb::VAL_INTEGER, 1000000));
