@@ -53,6 +53,11 @@ namespace PrefsSpecific {
 		TEXT_UNDER_ICON = 3 /* Qt::ToolButtonTextUnderIcon */
 	};
 
+	enum EmailCreation {
+		EMAIL_CREATE_DEFAULT = 0, /* Prefer xdg-email on Linux, fall back to eml file. */
+		EMAIL_CREATE_EML_FILE = 1 /* Always create eml file and open it in default appliaction. */
+	};
+
 	/*!
 	 * @brief Get value "storage.database.messages.on_disk.enabled".
 	 */
