@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2024 CZ.NIC
+ * Copyright (C) 2014-2025 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
  */
 
 #include <cinttypes>
+#include <QDateTime>
 #include <QThread>
 
 #include "src/datovka_shared/isds/error.h"
@@ -36,18 +37,22 @@
 #include "src/io/message_db_set.h"
 #include "src/isds/message_conversion.h"
 #include "src/isds/services.h"
+#include "src/isds/session.h"
 #include "src/settings/accounts.h"
 #include "src/worker/message_emitter.h"
 #include "src/worker/task_download_message.h"
 
 TaskDownloadMessage::TaskDownloadMessage(const AcntIdDb &acntIdDb,
-    enum MessageDirection msgDirect, const MsgId &msgId, bool storeRawInDb,
-    int processFlags, const QString &recMgmtHierarchyId)
-    : m_result(DM_ERR),
+    const QString &transactId, enum MessageDirection msgDirect,
+    const MsgId &msgId, bool storeRawInDb, int processFlags,
+    const QString &recMgmtHierarchyId)
+    : QObject(),
+    m_result(DM_ERR),
     m_isdsError(),
     m_isdsLongError(),
     m_mId(msgId),
     m_acntIdDb(acntIdDb),
+    m_transactId(transactId),
     m_msgDirect(msgDirect),
     m_storeRawInDb(storeRawInDb),
     m_processFlags(processFlags),
@@ -79,6 +84,14 @@ void TaskDownloadMessage::run(void)
 
 	/* ### Worker task begin. ### */
 
+	Isds::Session *session =
+	    GlobInstcs::isdsSessionsPtr->session(m_acntIdDb.username());
+	if (Q_NULLPTR != session) {
+		connect(session, SIGNAL(progress(qint64, qint64, qint64, qint64)),
+		    this, SLOT(watchProgress(qint64, qint64, qint64, qint64)),
+		    Qt::DirectConnection); /* Qt::DirectConnection because it must be executed immediatelly. */
+	}
+
 	const AcntId acntId(m_acntIdDb);
 
 	logDebugLv1NL("%s", "-----------------------------------------------");
@@ -102,13 +115,21 @@ void TaskDownloadMessage::run(void)
 		    GlobInstcs::acntMapPtr->acntData(acntId).accountName().toUtf8().constData());
 	}
 
-	emit GlobInstcs::msgProcEmitterPtr->downloadMessageFinished(acntId,
+	if (Q_NULLPTR != session) {
+		session->disconnect(Q_NULLPTR, this, Q_NULLPTR);
+	}
+
+	Q_EMIT GlobInstcs::msgProcEmitterPtr->downloadProgressFinished(
+	    AcntId(m_acntIdDb.username(), m_acntIdDb.testing()), m_transactId,
+	    m_mId, m_result, m_isdsError + " " + m_isdsLongError);
+
+	Q_EMIT GlobInstcs::msgProcEmitterPtr->downloadMessageFinished(acntId,
 	    m_mId.dmId(), m_mId.deliveryTime(), m_result,
 	    (!m_isdsError.isEmpty() || !m_isdsLongError.isEmpty()) ?
 	        m_isdsError + " " + m_isdsLongError : "",
 	    m_processFlags, m_recMgmtHierarchyId);
 
-	emit GlobInstcs::msgProcEmitterPtr->progressChange(PL_IDLE, 0);
+	Q_EMIT GlobInstcs::msgProcEmitterPtr->progressChange(PL_IDLE, 0);
 
 	/* ### Worker task end. ### */
 
@@ -307,13 +328,14 @@ enum TaskDownloadMessage::Result TaskDownloadMessage::downloadAndStoreSignedMess
 	logDebugLv0NL("Trying to download complete message '%" PRId64 "'",
 	    UGLY_QINT64_CAST mId.dmId());
 
-	emit GlobInstcs::msgProcEmitterPtr->progressChange(progressLabel, 0);
+	Q_EMIT GlobInstcs::msgProcEmitterPtr->progressChange(progressLabel, 0);
 
 	if (Q_UNLIKELY(GlobInstcs::req_halt)) {
 		return DM_ABORTED;
 	}
 
-	Isds::Session *session = GlobInstcs::isdsSessionsPtr->session(acntIdDb.username());
+	Isds::Session *session =
+	    GlobInstcs::isdsSessionsPtr->session(acntIdDb.username());
 	if (Q_UNLIKELY(Q_NULLPTR == session)) {
 		logErrorNL("Missing active session for username '%s'.",
 		    acntIdDb.username().toUtf8().constData());
@@ -325,7 +347,7 @@ enum TaskDownloadMessage::Result TaskDownloadMessage::downloadAndStoreSignedMess
 	Isds::Message delInfo;
 	Isds::DmMessageAuthor dmMsgAuthor;
 
-	emit GlobInstcs::msgProcEmitterPtr->progressChange(progressLabel, 10);
+	Q_EMIT GlobInstcs::msgProcEmitterPtr->progressChange(progressLabel, 10);
 
 	if (Q_UNLIKELY(GlobInstcs::req_halt)) {
 		return DM_ABORTED;
@@ -337,7 +359,7 @@ enum TaskDownloadMessage::Result TaskDownloadMessage::downloadAndStoreSignedMess
 		Q_ASSERT(0);
 		logErrorNL("Cannot access database to store message '%" PRId64 "'.",
 		    UGLY_QINT64_CAST mId.dmId());
-		emit GlobInstcs::msgProcEmitterPtr->progressChange(progressLabel, 100);
+		Q_EMIT GlobInstcs::msgProcEmitterPtr->progressChange(progressLabel, 100);
 		return DM_DB_INS_ERR;
 	}
 	bool isVodz = messageDb->isVodz(mId.dmId());
@@ -349,7 +371,7 @@ enum TaskDownloadMessage::Result TaskDownloadMessage::downloadAndStoreSignedMess
 		return msgRes;
 	}
 
-	emit GlobInstcs::msgProcEmitterPtr->progressChange(progressLabel, 20);
+	Q_EMIT GlobInstcs::msgProcEmitterPtr->progressChange(progressLabel, 20);
 
 	if (Q_UNLIKELY(GlobInstcs::req_halt)) {
 		return DM_ABORTED;
@@ -362,7 +384,7 @@ enum TaskDownloadMessage::Result TaskDownloadMessage::downloadAndStoreSignedMess
 		    UGLY_QINT64_CAST mId.dmId());
 	}
 
-	emit GlobInstcs::msgProcEmitterPtr->progressChange(progressLabel, 30);
+	Q_EMIT GlobInstcs::msgProcEmitterPtr->progressChange(progressLabel, 30);
 
 	if (Q_UNLIKELY(GlobInstcs::req_halt)) {
 		return DM_ABORTED;
@@ -375,7 +397,7 @@ enum TaskDownloadMessage::Result TaskDownloadMessage::downloadAndStoreSignedMess
 		    UGLY_QINT64_CAST mId.dmId());
 	}
 
-	emit GlobInstcs::msgProcEmitterPtr->progressChange(progressLabel, 40);
+	Q_EMIT GlobInstcs::msgProcEmitterPtr->progressChange(progressLabel, 40);
 
 	if (Q_UNLIKELY(GlobInstcs::req_halt)) {
 		return DM_ABORTED;
@@ -394,7 +416,7 @@ enum TaskDownloadMessage::Result TaskDownloadMessage::downloadAndStoreSignedMess
 			Q_ASSERT(0);
 			logErrorNL("Cannot access database to store message '%" PRId64 "'.",
 			    UGLY_QINT64_CAST mId.dmId());
-			emit GlobInstcs::msgProcEmitterPtr->progressChange(progressLabel, 100);
+			Q_EMIT GlobInstcs::msgProcEmitterPtr->progressChange(progressLabel, 100);
 			return DM_DB_INS_ERR;
 		}
 
@@ -403,7 +425,7 @@ enum TaskDownloadMessage::Result TaskDownloadMessage::downloadAndStoreSignedMess
 		if (Q_UNLIKELY(!messageDb->beginTransaction())) {
 			logErrorNL("Cannot begin transaction to store message '%" PRId64 "'.",
 			    UGLY_QINT64_CAST mId.dmId());
-			emit GlobInstcs::msgProcEmitterPtr->progressChange(progressLabel, 100);
+			Q_EMIT GlobInstcs::msgProcEmitterPtr->progressChange(progressLabel, 100);
 			messageDb->unlock();
 			return DM_DB_INS_ERR;
 		}
@@ -441,7 +463,7 @@ enum TaskDownloadMessage::Result TaskDownloadMessage::downloadAndStoreSignedMess
 
 	Q_ASSERT(message.envelope().dmId() == mId.dmId());
 
-	emit GlobInstcs::msgProcEmitterPtr->progressChange(progressLabel, 50);
+	Q_EMIT GlobInstcs::msgProcEmitterPtr->progressChange(progressLabel, 50);
 
 	if (Q_UNLIKELY(GlobInstcs::req_halt)) {
 		goto abort;
@@ -451,7 +473,7 @@ enum TaskDownloadMessage::Result TaskDownloadMessage::downloadAndStoreSignedMess
 	Task::storeSignedMessage(IMPLICIT_TRANSACTION, msgDirect, *dbSet,
 	    message, storeRawInDb);
 
-	emit GlobInstcs::msgProcEmitterPtr->progressChange(progressLabel, 70);
+	Q_EMIT GlobInstcs::msgProcEmitterPtr->progressChange(progressLabel, 70);
 
 	if (Q_UNLIKELY(GlobInstcs::req_halt)) {
 		goto abort;
@@ -472,7 +494,7 @@ enum TaskDownloadMessage::Result TaskDownloadMessage::downloadAndStoreSignedMess
 		}
 	}
 
-	emit GlobInstcs::msgProcEmitterPtr->progressChange(progressLabel, 80);
+	Q_EMIT GlobInstcs::msgProcEmitterPtr->progressChange(progressLabel, 80);
 
 	if (Q_UNLIKELY(GlobInstcs::req_halt)) {
 		goto abort;
@@ -491,7 +513,7 @@ enum TaskDownloadMessage::Result TaskDownloadMessage::downloadAndStoreSignedMess
 		}
 	}
 
-	emit GlobInstcs::msgProcEmitterPtr->progressChange(progressLabel, 90);
+	Q_EMIT GlobInstcs::msgProcEmitterPtr->progressChange(progressLabel, 90);
 
 	if (Q_UNLIKELY(GlobInstcs::req_halt)) {
 		goto abort;
@@ -516,7 +538,7 @@ enum TaskDownloadMessage::Result TaskDownloadMessage::downloadAndStoreSignedMess
 	messageDb->unlock();
 #endif /* USE_EXPLICIT_TRANSACTION */
 
-	emit GlobInstcs::msgProcEmitterPtr->progressChange(progressLabel, 100);
+	Q_EMIT GlobInstcs::msgProcEmitterPtr->progressChange(progressLabel, 100);
 
 	logDebugLv0NL("Done with %s().", __func__);
 
@@ -527,10 +549,21 @@ abort:
 	messageDb->rollbackTransaction();
 	messageDb->unlock();
 #endif /* USE_EXPLICIT_TRANSACTION */
-	emit GlobInstcs::msgProcEmitterPtr->progressChange(progressLabel, 100);
+	Q_EMIT GlobInstcs::msgProcEmitterPtr->progressChange(progressLabel, 100);
 	return DM_ABORTED;
 
 #undef USE_EXPLICIT_TRANSACTION
 #undef IMPLICIT_TRANSACTION
 #undef TRANSACTION_ENABLE
 }
+
+void TaskDownloadMessage::watchProgress(qint64 uploadTotal, qint64 uploadCurrent,
+    qint64 downloadTotal, qint64 downloadCurrent)
+{
+	Q_UNUSED(uploadTotal);
+	Q_UNUSED(uploadCurrent);
+
+	Q_EMIT GlobInstcs::msgProcEmitterPtr->downloadProgress(
+	    AcntId(m_acntIdDb.username(), m_acntIdDb.testing()), m_transactId,
+	    m_mId, downloadTotal, downloadCurrent);
+}
