@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2024 CZ.NIC
+ * Copyright (C) 2014-2025 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -367,7 +367,7 @@ enum TaskDownloadMessageList::Result TaskDownloadMessageList::downloadMessageLis
 #define STORE_RAW_INTO_DB (env.dmVODZ() != Isds::Type::BOOL_TRUE) /* Set to false if to store as files -- determined according to VoDZ. */
 				TaskDownloadMessage *task =
 				    new (::std::nothrow) TaskDownloadMessage(
-				        acntIdDb, msgDirect, msgId,
+				        acntIdDb, QString(), msgDirect, msgId,
 				        STORE_RAW_INTO_DB,
 				        flags, recMgmtHierarchyId);
 #undef STORE_RAW_INTO_DB
@@ -383,7 +383,16 @@ enum TaskDownloadMessageList::Result TaskDownloadMessageList::downloadMessageLis
 		/* Message is in db (dmDbMsgStatus <> Isds::Type::MS_NULL), -> update */
 		} else {
 
-			/* Update message and envelope only if status or dmType has changed. */
+			/* VoDZ flag is set -> always update envelope. */
+			if (msg.envelope().dmVODZ() == Isds::Type::BOOL_TRUE) {
+				Task::updateMessageEnvelope(IMPLICIT_TRANSACTION,
+				    msgDirect, *messageDb, msg.envelope());
+			}
+
+			/*
+			 * Update message and envelope only if status or
+			 * dmType has changed.
+			 */
 			const enum Isds::Type::DmState dmNewMsgStatus =
 			    msg.envelope().dmMessageStatus();
 			const QChar dmDbMsgType =
@@ -418,7 +427,7 @@ enum TaskDownloadMessageList::Result TaskDownloadMessageList::downloadMessageLis
 #define STORE_RAW_INTO_DB (msg.envelope().dmVODZ() != Isds::Type::BOOL_TRUE) /* Set to false if to store as files -- determined according to VoDZ. */
 					TaskDownloadMessage *task =
 					    new (::std::nothrow) TaskDownloadMessage(
-					        acntIdDb, msgDirect, msgId,
+					        acntIdDb, QString(), msgDirect, msgId,
 					        STORE_RAW_INTO_DB,
 					        flags);
 #undef STORE_RAW_INTO_DB
